@@ -170,17 +170,30 @@ int
 cos_postop_init ( Slapi_PBlock *pb )
 {
 	int rc = 0;
+	Slapi_Entry *plugin_entry = NULL;
+	char *plugin_type = NULL;
+	int postadd = SLAPI_PLUGIN_POST_ADD_FN;
+	int postmod = SLAPI_PLUGIN_POST_MODIFY_FN;
+	int postmdn = SLAPI_PLUGIN_POST_MODRDN_FN;
+	int postdel = SLAPI_PLUGIN_POST_DELETE_FN;
+
+	if ((slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_ENTRY, &plugin_entry) == 0) &&
+		plugin_entry &&
+		(plugin_type = slapi_entry_attr_get_charptr(plugin_entry, "nsslapd-plugintype")) &&
+		plugin_type && strstr(plugin_type, "betxn")) {
+		postadd = SLAPI_PLUGIN_BE_TXN_POST_ADD_FN;
+		postmod = SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN;
+		postmdn = SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN;
+		postdel = SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN;
+	}
+	slapi_ch_free_string(&plugin_type);
 
 	if ( slapi_pblock_set( pb, SLAPI_PLUGIN_VERSION, 
 							SLAPI_PLUGIN_VERSION_01 ) != 0 ||
-		 slapi_pblock_set(pb, SLAPI_PLUGIN_POST_MODIFY_FN,
-							(void *)cos_post_op ) != 0 ||
-		 slapi_pblock_set(pb, SLAPI_PLUGIN_POST_MODRDN_FN,
-							(void *)cos_post_op ) != 0 ||
-		 slapi_pblock_set(pb, SLAPI_PLUGIN_POST_ADD_FN,
-							(void *) cos_post_op ) != 0 ||
-		 slapi_pblock_set(pb, SLAPI_PLUGIN_POST_DELETE_FN,
-							(void *) cos_post_op ) != 0 )
+		 slapi_pblock_set(pb, postmod, (void *)cos_post_op ) != 0 ||
+		 slapi_pblock_set(pb, postmdn, (void *)cos_post_op ) != 0 ||
+		 slapi_pblock_set(pb, postadd, (void *) cos_post_op ) != 0 ||
+		 slapi_pblock_set(pb, postdel, (void *) cos_post_op ) != 0 )
 	{
 		slapi_log_error( SLAPI_LOG_FATAL, COS_PLUGIN_SUBSYSTEM,
 						 "cos_postop_init: failed to register plugin\n" );
@@ -221,9 +234,17 @@ int cos_init( Slapi_PBlock *pb )
 {
 	int ret = 0;
 	void * plugin_identity=NULL;
+	Slapi_Entry *plugin_entry = NULL;
+	int is_betxn = 0;
+	const char *plugintype = "postoperation";
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "--> cos_init\n",0,0,0);
 
+	if ((slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_ENTRY, &plugin_entry) == 0) &&
+		plugin_entry) {
+		is_betxn = slapi_entry_attr_get_bool(plugin_entry, "nsslapd-pluginbetxn");
+	}
+
 	/*
 	** Store the plugin identity for later use.
 	** Used for internal operations
@@ -248,7 +269,10 @@ int cos_init( Slapi_PBlock *pb )
 		goto bailout;
 	}
 
-	ret = slapi_register_plugin("postoperation", 1 /* Enabled */,
+	if (is_betxn) {
+		plugintype = "betxnpostoperation";
+	}
+	ret = slapi_register_plugin(plugintype, 1 /* Enabled */,
 					"cos_postop_init", cos_postop_init,
 					"Class of Service postoperation plugin", NULL,
 					plugin_identity);
@@ -256,10 +280,12 @@ int cos_init( Slapi_PBlock *pb )
 		goto bailout;
 	}
 
-	ret = slapi_register_plugin("internalpostoperation", 1 /* Enabled */,
-					"cos_internalpostop_init", cos_internalpostop_init,
-					"Class of Service internalpostoperation plugin", NULL,
-					plugin_identity);
+	if (!is_betxn) {
+		ret = slapi_register_plugin("internalpostoperation", 1 /* Enabled */,
+									"cos_internalpostop_init", cos_internalpostop_init,
+									"Class of Service internalpostoperation plugin", NULL,
+									plugin_identity);
+	}
 
 bailout:
 	LDAPDebug( LDAP_DEBUG_TRACE, "<-- cos_init\n",0,0,0);
