@@ -46,6 +46,8 @@
 #include "deref.h"
 #include <nspr.h>
 
+int is_type_forbidden(const char *type); /* from proto-slap.h */
+
 #ifndef DN_SYNTAX_OID
 #define DN_SYNTAX_OID "1.3.6.1.4.1.1466.115.121.1.12"
 #endif
@@ -594,7 +596,6 @@ deref_do_deref_attr(Slapi_PBlock *pb, BerElement *ctrlber, const char *derefdn,
     Slapi_PBlock *derefpb = NULL;
     Slapi_Entry **entries = NULL;
     int rc;
-    void *txn = NULL;
 
     if (deref_check_access(pb, NULL, derefdn, attrs, &retattrs,
                            (SLAPI_ACL_SEARCH|SLAPI_ACL_READ))) {
@@ -605,11 +606,10 @@ deref_do_deref_attr(Slapi_PBlock *pb, BerElement *ctrlber, const char *derefdn,
     }
 
     derefpb = slapi_pblock_new();
-    slapi_pblock_get(pb, SLAPI_TXN, &txn);
     slapi_search_internal_set_pb(derefpb, derefdn, LDAP_SCOPE_BASE,
                                  "(objectclass=*)", retattrs, 0,
                                  NULL, NULL, deref_get_plugin_id(), 0);
-    slapi_pblock_set(derefpb, SLAPI_TXN, txn);
+
     slapi_search_internal_pb(derefpb);
     slapi_pblock_get(derefpb, SLAPI_PLUGIN_INTOP_RESULT, &rc);
     if (LDAP_SUCCESS == rc) {
@@ -634,6 +634,12 @@ deref_do_deref_attr(Slapi_PBlock *pb, BerElement *ctrlber, const char *derefdn,
                     int needpartialattr = 1; /* need PartialAttribute sequence? */
                     int needvalsset = 1;
 
+                    if (is_type_forbidden(retattrs[ii])) {
+                        slapi_log_error(SLAPI_LOG_PLUGIN, DEREF_PLUGIN_SUBSYSTEM,
+                            "skip forbidden attribute [%s]\n", derefdn);
+                        continue;
+                    }
+
                     deref_get_values(entries[0], retattrs[ii], &results, &type_name_disposition,
                                      &actual_type_name, flags, &buffer_flags);
 
