@@ -48,7 +48,7 @@
  */
 static void linked_attrs_fixup_task_destructor(Slapi_Task *task);
 static void linked_attrs_fixup_task_thread(void *arg);
-static void linked_attrs_fixup_links(struct configEntry *config, void *txn);
+static void linked_attrs_fixup_links(struct configEntry *config);
 static int linked_attrs_remove_backlinks_callback(Slapi_Entry *e, void *callback_data);
 static int linked_attrs_add_backlinks_callback(Slapi_Entry *e, void *callback_data);
 static const char *fetch_attr(Slapi_Entry *e, const char *attrname,
@@ -67,6 +67,7 @@ linked_attrs_fixup_task_add(Slapi_PBlock *pb, Slapi_Entry *e,
 	task_data *mytaskdata = NULL;
 	Slapi_Task *task = NULL;
 	const char *linkdn = NULL;
+	char *bind_dn;
 
 	*returncode = LDAP_SUCCESS;
 
@@ -81,6 +82,7 @@ linked_attrs_fixup_task_add(Slapi_PBlock *pb, Slapi_Entry *e,
 	linkdn = fetch_attr(e, "linkdn", 0);
 
 	/* setup our task data */
+	slapi_pblock_get(pb, SLAPI_REQUESTOR_DN, &bind_dn);
 	mytaskdata = (task_data*)slapi_ch_calloc(1, sizeof(task_data));
 	if (mytaskdata == NULL) {
 		*returncode = LDAP_OPERATIONS_ERROR;
@@ -91,6 +93,7 @@ linked_attrs_fixup_task_add(Slapi_PBlock *pb, Slapi_Entry *e,
 	if (linkdn) {
 	    mytaskdata->linkdn = slapi_dn_normalize(slapi_ch_strdup(linkdn));
 	}
+	mytaskdata->bind_dn = slapi_ch_strdup(bind_dn);
 
 	/* allocate new task now */
 	task = slapi_new_task(slapi_entry_get_ndn(e));
@@ -126,6 +129,7 @@ linked_attrs_fixup_task_destructor(Slapi_Task *task)
 		task_data *mydata = (task_data *)slapi_task_get_data(task);
 		if (mydata) {
 			slapi_ch_free_string(&mydata->linkdn);
+			slapi_ch_free_string(&mydata->bind_dn);
 			/* Need to cast to avoid a compiler warning */
 			slapi_ch_free((void **)&mydata);
 		}
@@ -144,6 +148,9 @@ linked_attrs_fixup_task_thread(void *arg)
 	/* Fetch our task data from the task */
 	td = (task_data *)slapi_task_get_data(task);
 
+	/* init and set the bind dn in the thread data */
+	slapi_td_set_dn(slapi_ch_strdup(td->bind_dn));
+
 	/* Log started message. */
 	slapi_task_begin(task, 1);
 	slapi_task_log_notice(task, "Linked attributes fixup task starting (link dn: \"%s\") ...\n",
@@ -170,7 +177,7 @@ linked_attrs_fixup_task_thread(void *arg)
                     slapi_log_error(SLAPI_LOG_FATAL, LINK_PLUGIN_SUBSYSTEM,
                          "Fixing up linked attribute pair (%s)\n", config_entry->dn);
 
-                    linked_attrs_fixup_links(config_entry, NULL);
+                    linked_attrs_fixup_links(config_entry);
                     break;
                 }
             } else {
@@ -180,7 +187,7 @@ linked_attrs_fixup_task_thread(void *arg)
                 slapi_log_error(SLAPI_LOG_FATAL, LINK_PLUGIN_SUBSYSTEM,
                        "Fixing up linked attribute pair (%s)\n", config_entry->dn);
 
-                linked_attrs_fixup_links(config_entry, NULL);
+                linked_attrs_fixup_links(config_entry);
             }
 
             list = PR_NEXT_LINK(list);
@@ -207,19 +214,12 @@ linked_attrs_fixup_task_thread(void *arg)
 	slapi_task_finish(task, rc);
 }
 
-struct fixup_cb_data {
-    char *attrtype;
-    void *txn;
-    struct configEntry *config;
-};
-
 static void 
-linked_attrs_fixup_links(struct configEntry *config, void *txn)
+linked_attrs_fixup_links(struct configEntry *config)
 {
     Slapi_PBlock *pb = slapi_pblock_new();
     char *del_filter = NULL;
     char *add_filter = NULL;
-    struct fixup_cb_data cb_data = {NULL, NULL, NULL};
 
     del_filter = slapi_ch_smprintf("%s=*", config->managedtype);
     add_filter = slapi_ch_smprintf("%s=*", config->linktype);
@@ -232,11 +232,8 @@ linked_attrs_fixup_links(struct configEntry *config, void *txn)
          * within the scope and remove the managed type. */
         slapi_search_internal_set_pb(pb, config->scope, LDAP_SCOPE_SUBTREE,
                 del_filter, 0, 0, 0, 0, linked_attrs_get_plugin_id(), 0);
-        slapi_pblock_set(pb, SLAPI_TXN, txn);
 
-        cb_data.attrtype = config->managedtype;
-        cb_data.txn = txn;
-        slapi_search_internal_callback_pb(pb, &cb_data, 0,
+        slapi_search_internal_callback_pb(pb, config->managedtype, 0,
                 linked_attrs_remove_backlinks_callback, 0);
 
         /* Clean out pblock for reuse. */
@@ -246,12 +243,8 @@ linked_attrs_fixup_links(struct configEntry *config, void *txn)
          * scope and add backlinks to the entries they point to. */
         slapi_search_internal_set_pb(pb, config->scope, LDAP_SCOPE_SUBTREE,
                 add_filter, 0, 0, 0, 0, linked_attrs_get_plugin_id(), 0);
-        slapi_pblock_set(pb, SLAPI_TXN, txn);
 
-        cb_data.attrtype = NULL;
-        cb_data.txn = txn;
-        cb_data.config = config;
-        slapi_search_internal_callback_pb(pb, &cb_data, 0,
+        slapi_search_internal_callback_pb(pb, config, 0,
                 linked_attrs_add_backlinks_callback, 0);
     } else {
         /* Loop through all non-private backend suffixes and
@@ -266,10 +259,7 @@ linked_attrs_fixup_links(struct configEntry *config, void *txn)
                                          LDAP_SCOPE_SUBTREE, del_filter,
                                          0, 0, 0, 0,
                                          linked_attrs_get_plugin_id(), 0);
-            slapi_pblock_set(pb, SLAPI_TXN, txn);
 
-            cb_data.attrtype = config->managedtype;
-            cb_data.txn = txn;
             slapi_search_internal_callback_pb(pb, config->managedtype, 0,
                     linked_attrs_remove_backlinks_callback, 0);
 
@@ -280,11 +270,8 @@ linked_attrs_fixup_links(struct configEntry *config, void *txn)
                                          LDAP_SCOPE_SUBTREE, add_filter,
                                          0, 0, 0, 0,
                                          linked_attrs_get_plugin_id(), 0);
-            slapi_pblock_set(pb, SLAPI_TXN, txn);
-            cb_data.attrtype = NULL;
-            cb_data.txn = txn;
-            cb_data.config = config;
-            slapi_search_internal_callback_pb(pb, &cb_data, 0,
+
+            slapi_search_internal_callback_pb(pb, config, 0,
                     linked_attrs_add_backlinks_callback, 0);
 
             /* Clean out pblock for reuse. */
@@ -307,8 +294,7 @@ linked_attrs_remove_backlinks_callback(Slapi_Entry *e, void *callback_data)
 {
     int rc = 0;
     Slapi_DN *sdn = slapi_entry_get_sdn(e);
-    struct fixup_cb_data *cb_data = (struct fixup_cb_data *)callback_data;
-    char *type = cb_data->attrtype;
+    char *type = (char *)callback_data;
     Slapi_PBlock *pb = slapi_pblock_new();
     char *val[1];
     LDAPMod mod;
@@ -331,7 +317,6 @@ linked_attrs_remove_backlinks_callback(Slapi_Entry *e, void *callback_data)
     /* Perform the operation. */
     slapi_modify_internal_set_pb_ext(pb, sdn, mods, 0, 0,
                                      linked_attrs_get_plugin_id(), 0);
-    slapi_pblock_set(pb, SLAPI_TXN, cb_data->txn);
     slapi_modify_internal_pb(pb);
 
     slapi_pblock_destroy(pb);
@@ -344,8 +329,7 @@ linked_attrs_add_backlinks_callback(Slapi_Entry *e, void *callback_data)
 {
     int rc = 0;
     char *linkdn = slapi_entry_get_dn(e);
-    struct fixup_cb_data *cb_data = (struct fixup_cb_data *)callback_data;
-    struct configEntry *config = cb_data->config;
+    struct configEntry *config = (struct configEntry *)callback_data;
     Slapi_PBlock *pb = slapi_pblock_new();
     int i = 0;
     char **targets = NULL;
@@ -371,7 +355,7 @@ linked_attrs_add_backlinks_callback(Slapi_Entry *e, void *callback_data)
         int perform_update = 0;
         Slapi_DN *targetsdn = NULL;
 
-        if (g_get_shutdown()) {
+        if (slapi_is_shutting_down()) {
             rc = -1;
             goto done;
         }
@@ -401,7 +385,6 @@ linked_attrs_add_backlinks_callback(Slapi_Entry *e, void *callback_data)
             /* Perform the modify operation. */
             slapi_modify_internal_set_pb_ext(pb, targetsdn, mods, 0, 0,
                                              linked_attrs_get_plugin_id(), 0);
-            slapi_pblock_set(pb, SLAPI_TXN, cb_data->txn);
             slapi_modify_internal_pb(pb);
 
             /* Initialize the pblock so we can reuse it. */
