@@ -77,10 +77,10 @@ int referint_postop_del( Slapi_PBlock *pb );
 int referint_postop_modrdn( Slapi_PBlock *pb ); 
 int referint_postop_start( Slapi_PBlock *pb);
 int referint_postop_close( Slapi_PBlock *pb);
-int update_integrity(char **argv, Slapi_DN *sDN, char *newrDN, Slapi_DN *newsuperior, int logChanges, void *txn);
+int update_integrity(char **argv, Slapi_DN *sDN, char *newrDN, Slapi_DN *newsuperior, int logChanges);
 void referint_thread_func(void *arg);
 int  GetNextLine(char *dest, int size_dest, PRFileDesc *stream);
-void writeintegritylog(char *logfilename, Slapi_DN *sdn, char *newrdn, Slapi_DN *newsuperior);
+void writeintegritylog(Slapi_PBlock *pb, char *logfilename, Slapi_DN *sdn, char *newrdn, Slapi_DN *newsuperior, Slapi_DN *requestorsdn);
 int my_fgetc(PRFileDesc *stream);
 
 /* global thread control stuff */
@@ -165,12 +165,10 @@ referint_postop_del( Slapi_PBlock *pb )
 	int delay;
 	int logChanges=0;
 	int isrepop = 0;
-	void *txn = NULL;
 
 	if ( slapi_pblock_get( pb, SLAPI_IS_REPLICATED_OPERATION, &isrepop ) != 0  ||
-		 slapi_pblock_get( pb, SLAPI_DELETE_TARGET_SDN, &sdn ) != 0  ||
-	     slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &oprc) != 0  ||
-	     slapi_pblock_get(pb, SLAPI_TXN, &txn) != 0) 
+	     slapi_pblock_get( pb, SLAPI_DELETE_TARGET_SDN, &sdn ) != 0  ||
+	     slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &oprc) != 0) 
         {
             slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                              "referint_postop_del: could not get parameters\n" );
@@ -198,7 +196,7 @@ referint_postop_del( Slapi_PBlock *pb )
 	
 	if(argv == NULL){
 	  slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
-		     "referint_postop_modrdn, args are NULL\n" );
+		     "referint_postop_del, args are NULL\n" );
 	  return( -1 ); 
 	}
 	
@@ -215,10 +213,10 @@ referint_postop_del( Slapi_PBlock *pb )
 		}else if(delay == 0){
 		  /* no delay */
  		  /* call function to update references to entry */
-		  rc = update_integrity(argv, sdn, NULL, NULL, logChanges, txn);
+		  rc = update_integrity(argv, sdn, NULL, NULL, logChanges);
 		}else{
 		  /* write the entry to integrity log */
-		  writeintegritylog(argv[1], sdn, NULL, NULL);
+		  writeintegritylog(pb, argv[1], sdn, NULL, NULL, NULL /* slapi_get_requestor_sdn(pb) */);
 		  rc = 0;
 		}
 	} else {
@@ -244,14 +242,12 @@ referint_postop_modrdn( Slapi_PBlock *pb )
 	int delay;
 	int logChanges=0;
 	int isrepop = 0;
-	void *txn = NULL;
 
 	if ( slapi_pblock_get( pb, SLAPI_IS_REPLICATED_OPERATION, &isrepop ) != 0  ||
 		 slapi_pblock_get( pb, SLAPI_MODRDN_TARGET_SDN, &sdn ) != 0 ||
 		 slapi_pblock_get( pb, SLAPI_MODRDN_NEWRDN, &newrdn ) != 0 ||
 		 slapi_pblock_get( pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &newsuperior ) != 0 ||
-		 slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &oprc) != 0 ||
-		 slapi_pblock_get(pb, SLAPI_TXN, &txn) != 0) {
+		 slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &oprc) != 0 ){
 
 		slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
 		    "referint_postop_modrdn: could not get parameters\n" );
@@ -300,11 +296,10 @@ referint_postop_modrdn( Slapi_PBlock *pb )
 	}else if(delay == 0){
 	  /* no delay */
 	  /* call function to update references to entry */
-	  rc = update_integrity(argv, sdn, newrdn,
-	                        newsuperior, logChanges, txn);
+	  rc = update_integrity(argv, sdn, newrdn, newsuperior, logChanges);
 	}else{
 	  /* write the entry to integrity log */
-	  writeintegritylog(argv[1], sdn, newrdn, newsuperior);
+	  writeintegritylog(pb, argv[1], sdn, newrdn, newsuperior, NULL /* slapi_get_requestor_sdn(pb) */);
 	  rc = 0;
 	}
 
@@ -332,13 +327,11 @@ int isFatalSearchError(int search_result)
 }
 
 static int
-_do_modify(Slapi_PBlock *mod_pb, Slapi_DN *entrySDN, LDAPMod **mods, void *txn)
+_do_modify(Slapi_PBlock *mod_pb, Slapi_DN *entrySDN, LDAPMod **mods)
 {
     int rc = 0;
 
     slapi_pblock_init(mod_pb);
-    /* set the transaction to use */
-    slapi_pblock_set(mod_pb, SLAPI_TXN, txn);
 
     /* Use internal operation API */
     slapi_modify_internal_set_pb_ext(mod_pb, entrySDN, mods, NULL, NULL,
@@ -356,10 +349,10 @@ static int
 _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
                     Slapi_Attr *attr,        /* referred attribute */
                     char *attrName,
-                    const char *origDN,      /* original DN that was modified */
+                    Slapi_DN *origDN,        /* original DN that was modified */
                     char *newRDN,            /* new RDN from modrdn */
                     const char *newsuperior, /* new superior from modrdn */
-                    Slapi_PBlock *mod_pb, void *txn)
+                    Slapi_PBlock *mod_pb)
 {
     LDAPMod *list_of_mods[3];
     char *values_del[2];
@@ -374,7 +367,7 @@ _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
     if (NULL == newRDN && NULL == newsuperior) {
         /* in delete mode */
         /* delete old dn so set that up */
-        values_del[0] = (char *)origDN;
+        values_del[0] = (char *)slapi_sdn_get_dn(origDN);
         values_del[1] = NULL;
         attribute1.mod_type = attrName;
         attribute1.mod_op = LDAP_MOD_DELETE;
@@ -382,11 +375,11 @@ _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
         list_of_mods[0] = &attribute1;
         /* terminate list of mods. */
         list_of_mods[1] = NULL;
-        rc = _do_modify(mod_pb, entrySDN, list_of_mods, txn);
+        rc = _do_modify(mod_pb, entrySDN, list_of_mods);
         if (rc) {
             slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                 "_update_one_value: entry %s: deleting \"%s: %s\" failed (%d)"
-                "\n", slapi_sdn_get_dn(entrySDN), attrName, origDN, rc);
+                "\n", slapi_sdn_get_dn(entrySDN), attrName, slapi_sdn_get_dn(origDN), rc);
         }
     } else {
         /* in modrdn mode */
@@ -400,11 +393,11 @@ _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
             goto bail;
         }
         /* need to put together rdn into a dn */
-        dnParts = slapi_ldap_explode_dn( origDN, 0 );
+        dnParts = slapi_ldap_explode_dn( slapi_sdn_get_dn(origDN), 0 );
         if (NULL == dnParts) {
             slapi_log_error(SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                             "_update_one_value: failed to explode dn %s\n",
-                            origDN);
+                            slapi_sdn_get_dn(origDN));
             goto bail;
         }
         if (NULL == newRDN) {
@@ -413,11 +406,12 @@ _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
         if (newsuperior) {
             superior = newsuperior;
         } else {
-            /* no need to free superior */
-            superior = slapi_dn_find_parent(origDN);
+            /* do not free superior */
+            superior = slapi_dn_find_parent(slapi_sdn_get_dn(origDN));
         }
         /* newRDN and superior are already normalized. */
         newDN = slapi_ch_smprintf("%s,%s", newRDN, superior);
+        slapi_dn_ignore_case(newDN);
         /* 
          * Compare the modified dn with the value of 
          * the target attribute of referint to find out
@@ -454,7 +448,7 @@ _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
             }
             /* else: (rc < 0) Ignore the DN normalization error for now. */
 
-            p = PL_strstr(sval, origDN);
+            p = PL_strstr(sval, slapi_sdn_get_ndn(origDN));
             if (p == sval) {
                 /* (case 1) */
                 values_del[0] = sval;
@@ -471,13 +465,13 @@ _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
                 attribute2.mod_values = values_add;
                 list_of_mods[1] = &attribute2;
                 list_of_mods[2] = NULL;
-                rc = _do_modify(mod_pb, entrySDN, list_of_mods, txn);
+                rc = _do_modify(mod_pb, entrySDN, list_of_mods);
                 if (rc) {
                     slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                         "_update_one_value: entry %s: replacing \"%s: %s\" "
                         "with \"%s: %s\" failed (%d)\n",
                         slapi_sdn_get_dn(entrySDN), attrName, 
-                        origDN, attrName, newDN, rc);
+                        slapi_sdn_get_dn(origDN), attrName, newDN, rc);
                 }
             } else if (p) {
                 char bak;
@@ -501,7 +495,7 @@ _update_one_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
                 attribute2.mod_values = values_add;
                 list_of_mods[1] = &attribute2;
                 list_of_mods[2] = NULL;
-                rc = _do_modify(mod_pb, entrySDN, list_of_mods, txn);
+                rc = _do_modify(mod_pb, entrySDN, list_of_mods);
                 if (rc) {
                     slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                         "_update_one_value: entry %s: replacing \"%s: %s\" "
@@ -532,10 +526,10 @@ static int
 _update_all_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
                     Slapi_Attr *attr,        /* referred attribute */
                     char *attrName,
-                    const char *origDN,      /* original DN that was modified */
+                    Slapi_DN *origDN,        /* original DN that was modified */
                     char *newRDN,            /* new RDN from modrdn */
                     const char *newsuperior, /* new superior from modrdn */
-                    Slapi_PBlock *mod_pb, void *txn)
+                    Slapi_PBlock *mod_pb)
 {
     Slapi_Mods *smods = NULL;
     char *newDN = NULL;
@@ -554,7 +548,7 @@ _update_all_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
         LDAPMod attribute1;
 
         /* delete old dn so set that up */
-        values_del[0] = (char *)origDN;
+        values_del[0] = (char *)slapi_sdn_get_dn(origDN);
         values_del[1] = NULL;
         attribute1.mod_type = attrName;
         attribute1.mod_op = LDAP_MOD_DELETE;
@@ -562,11 +556,11 @@ _update_all_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
         mods[0] = &attribute1;
         /* terminate list of mods. */
         mods[1] = NULL;
-        rc = _do_modify(mod_pb, entrySDN, mods, txn);
+        rc = _do_modify(mod_pb, entrySDN, mods);
         if (rc) {
             slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                 "_update_all_per_mod: entry %s: deleting \"%s: %s\" failed (%d)"
-                "\n", slapi_sdn_get_dn(entrySDN), attrName, origDN, rc);
+                "\n", slapi_sdn_get_dn(entrySDN), attrName, slapi_sdn_get_dn(origDN), rc);
         }
     } else {
         /* in modrdn mode */
@@ -580,11 +574,11 @@ _update_all_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
             goto bail;
         }
         /* need to put together rdn into a dn */
-        dnParts = slapi_ldap_explode_dn( origDN, 0 );
+        dnParts = slapi_ldap_explode_dn( slapi_sdn_get_dn(origDN), 0 );
         if (NULL == dnParts) {
             slapi_log_error(SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                             "_update_all_per_mod: failed to explode dn %s\n",
-                            origDN);
+                            slapi_sdn_get_dn(origDN));
             goto bail;
         }
         if (NULL == newRDN) {
@@ -593,10 +587,12 @@ _update_all_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
         if (newsuperior) {
             superior = newsuperior;
         } else {
-            /* no need to free superior */
-            superior = slapi_dn_find_parent(origDN);
+            /* do not free superior */
+            superior = slapi_dn_find_parent(slapi_sdn_get_dn(origDN));
         }
+        /* newRDN and superior are already normalized. */
         newDN = slapi_ch_smprintf("%s,%s", newRDN, superior);
+        slapi_dn_ignore_case(newDN);
         /* 
          * Compare the modified dn with the value of 
          * the target attribute of referint to find out
@@ -637,7 +633,7 @@ _update_all_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
             }
             /* else: (rc < 0) Ignore the DN normalization error for now. */
 
-            p = PL_strstr(sval, origDN);
+            p = PL_strstr(sval, slapi_sdn_get_ndn(origDN));
             if (p == sval) {
                 /* (case 1) */
                 slapi_mods_add_string(smods, LDAP_MOD_DELETE, attrName, sval);
@@ -654,7 +650,7 @@ _update_all_per_mod(Slapi_DN *entrySDN,      /* DN of the searched entry */
             /* else: value does not include the modified DN.  Ignore it. */
             slapi_ch_free_string(&sval);
         }
-        rc = _do_modify(mod_pb, entrySDN, slapi_mods_get_ldapmods_byref(smods), txn);
+        rc = _do_modify(mod_pb, entrySDN, slapi_mods_get_ldapmods_byref(smods));
         if (rc) {
             slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,
                         "_update_all_per_mod: entry %s failed (%d)\n",
@@ -676,7 +672,7 @@ bail:
 int
 update_integrity(char **argv, Slapi_DN *origSDN,
                  char *newrDN, Slapi_DN *newsuperior, 
-                 int logChanges, void *txn)
+                 int logChanges)
 {
     Slapi_PBlock *search_result_pb = NULL;
     Slapi_PBlock *mod_pb = slapi_pblock_new();
@@ -697,7 +693,7 @@ update_integrity(char **argv, Slapi_DN *origSDN,
         rc = -1;
         goto free_and_return;
     } 
-
+  
     /* for now, just putting attributes to keep integrity on in conf file,
        until resolve the other timing mode issue */
     search_result_pb = slapi_pblock_new();
@@ -721,8 +717,6 @@ update_integrity(char **argv, Slapi_DN *origSDN,
 
                 /* Use new search API */
                 slapi_pblock_init(search_result_pb);
-                /* set the parent txn for the search ops */
-                slapi_pblock_set(search_result_pb, SLAPI_TXN, txn);
                 slapi_search_internal_set_pb(search_result_pb, search_base, 
                     LDAP_SCOPE_SUBTREE, filter, attrs, 0 /* attrs only */,
                     NULL, NULL, referint_plugin_identity, 0);
@@ -772,15 +766,15 @@ update_integrity(char **argv, Slapi_DN *origSDN,
                                 if (nval > 128) {
                                     rc = _update_one_per_mod(
                                          slapi_entry_get_sdn(search_entries[j]),
-                                         attr, attrName, origDN, newrDN, 
+                                         attr, attrName, origSDN, newrDN, 
                                          slapi_sdn_get_dn(newsuperior),
-                                         mod_pb, txn);
+                                         mod_pb);
                                 } else {
                                     rc = _update_all_per_mod(
                                          slapi_entry_get_sdn(search_entries[j]),
-                                         attr, attrName, origDN, newrDN, 
+                                         attr, attrName, origSDN, newrDN, 
                                          slapi_sdn_get_dn(newsuperior),
-                                         mod_pb, txn);
+                                         mod_pb);
                                 }
                                 /* Should we stop if one modify returns an error? */
                             }
@@ -921,6 +915,11 @@ referint_thread_func(void *arg)
       return;
     }
 
+    /* initialize the thread data index
+    if(slapi_td_dn_init()){
+    	slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,"Failed to create thread data index\n");
+
+    } */
 
     delay = atoi(plugin_argv[0]);
     logfilename = plugin_argv[1]; 
@@ -986,9 +985,16 @@ referint_thread_func(void *arg)
 	    } else {
 	        tmpsuperior = slapi_sdn_new_normdn_byref(ptoken);
 	    }
+	    ptoken = ldap_utf8strtok_r (NULL, delimiter, &iter);
+	    if (strcasecmp(ptoken, "NULL") != 0) {
+	    	/* Set the bind DN in the thread data */
+	        if(slapi_td_set_dn(slapi_ch_strdup(ptoken))){
+	        	slapi_log_error( SLAPI_LOG_FATAL, REFERINT_PLUGIN_SUBSYSTEM,"Failed to set thread data\n");
+	        }
+	    }
       
 	    update_integrity(plugin_argv, sdn, tmprdn,
-	                     tmpsuperior, logChanges, NULL);
+	                     tmpsuperior, logChanges);
       
 	    slapi_sdn_free(&sdn);
 	    slapi_ch_free_string(&tmprdn);
@@ -1105,13 +1111,16 @@ GetNextLine(char *dest, int size_dest, PRFileDesc *stream) {
 }
 
 void
-writeintegritylog(char *logfilename, Slapi_DN *sdn, 
-                  char *newrdn, Slapi_DN *newsuperior)
+writeintegritylog(Slapi_PBlock *pb, char *logfilename, Slapi_DN *sdn,
+                  char *newrdn, Slapi_DN *newsuperior, Slapi_DN *requestorsdn)
 {
     PRFileDesc *prfd;
     char buffer[MAX_LINE];
     int len_to_write = 0;
     int rc;
+    const char *requestordn = NULL;
+    const char *newsuperiordn = NULL;
+    size_t reqdn_len = 0;
     /* write this record to the file */
 
     /* use this lock to protect file data when update integrity is occuring */
@@ -1134,8 +1143,8 @@ writeintegritylog(char *logfilename, Slapi_DN *sdn,
        before trying to write it 
      */
 
-	/* add length of dn +  4(two tabs, a newline, and terminating \0) */
-    len_to_write = slapi_sdn_get_ndn_len(sdn) + 4;
+	/* add length of dn +  5(three tabs, a newline, and terminating \0) */
+    len_to_write = slapi_sdn_get_ndn_len(sdn) + 5;
 
     if(newrdn == NULL)
     {
@@ -1145,7 +1154,8 @@ writeintegritylog(char *logfilename, Slapi_DN *sdn,
         /* add the length of the newrdn */
         len_to_write += strlen(newrdn);
     }
-    if(NULL == newsuperior)
+    newsuperiordn = slapi_sdn_get_dn(newsuperior);
+    if(NULL == newsuperiordn)
     {
         /* add the length of "NULL" */
         len_to_write += 4;
@@ -1153,6 +1163,13 @@ writeintegritylog(char *logfilename, Slapi_DN *sdn,
         /* add the length of the newsuperior */
         len_to_write += slapi_sdn_get_ndn_len(newsuperior);
     }
+    slapi_pblock_get(pb, SLAPI_REQUESTOR_DN, &requestordn);
+    if (requestorsdn && (requestordn = slapi_sdn_get_udn(requestorsdn)) &&
+        (reqdn_len = strlen(requestordn))) {
+        len_to_write += reqdn_len;
+    } else {
+        len_to_write += 4; /* "NULL" */
+    }
 
     if(len_to_write > MAX_LINE )
     {
@@ -1161,11 +1178,11 @@ writeintegritylog(char *logfilename, Slapi_DN *sdn,
                          " line length exceeded. It will not be able"
                          " to update references to this entry.\n");
     }else{
-       PR_snprintf(buffer, MAX_LINE, "%s\t%s\t%s\t\n", 
+       PR_snprintf(buffer, MAX_LINE, "%s\t%s\t%s\t%s\t\n", 
 				   slapi_sdn_get_dn(sdn),
 				   (newrdn != NULL) ? newrdn : "NULL",
-				   (newsuperior != NULL) ? slapi_sdn_get_dn(newsuperior) :
-				                           "NULL");
+				   (newsuperiordn != NULL) ? newsuperiordn : "NULL",
+                   requestordn ? requestordn : "NULL");
         if (PR_Write(prfd,buffer,strlen(buffer)) < 0){
            slapi_log_error(SLAPI_LOG_FATAL,REFERINT_PLUGIN_SUBSYSTEM,
 	       " writeintegritylog: PR_Write failed : The disk"
