@@ -117,6 +117,7 @@ typedef struct repl5agmt {
 	time_t last_update_end_time; /* Local end time of last update session */
 	char last_update_status[STATUS_LEN]; /* Status of last update. Format = numeric code <space> textual description */
 	PRBool update_in_progress;
+	PRBool is_enabled;
 	time_t last_init_start_time; /* Local start time of last total init */
 	time_t last_init_end_time; /* Local end time of last total init */
 	char last_init_status[STATUS_LEN]; /* Status of last total init. Format = numeric code <space> textual description */
@@ -137,7 +138,10 @@ typedef struct repl5agmt {
 	void *priv; /* private data, used for windows-specific agreement data 
 	               for sync agreements or for replication session plug-in
 	               private data for normal replication agreements */
+	char **attrs_to_strip; /* for fractional replication, if a "mod" is empty, strip out these attributes:
+	                        * modifiersname, modifytimestamp, internalModifiersname, internalModifyTimestamp, etc */
 	int agreement_type;
+	int cleanruv_notified[CLEANRIDSIZ + 1]; /* specifies if the replica has been notified of a CLEANALLRUV task */
 } repl5agmt;
 
 /* Forward declarations */
@@ -218,6 +222,26 @@ agmt_is_valid(Repl_Agmt *ra)
 						slapi_sdn_get_dn(ra->dn), type_nsds5TransportInfo, type_nsds5ReplicaBindMethod);
 		return_value = 0;
 	}
+	/*
+	 * If we are not using GSSAPI or SSL Client Auth, then a bind dn and password must be present
+	 */
+	if(BINDMETHOD_SASL_GSSAPI != ra->bindmethod && BINDMETHOD_SSL_CLIENTAUTH != ra->bindmethod){
+		if(strcmp(ra->binddn,"") == 0 || ra->creds->bv_val == NULL){
+			char *auth_mech;
+
+			if(ra->bindmethod == BINDMETHOD_SIMPLE_AUTH){
+				auth_mech = "SIMPLE";
+			} else if (ra->bindmethod == BINDMETHOD_SASL_DIGEST_MD5){
+				auth_mech = "SASL/DIGEST-MD5";
+			} else {
+				auth_mech = "Unknown";
+			}
+			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Replication agreement \"%s\" "
+				"is malformed: a bind DN and password must be supplied for authentication "
+				"method \"%s\"\n", slapi_sdn_get_dn(ra->dn), auth_mech);
+			return_value = 0;
+		}
+	}
 	return return_value;
 }
 
@@ -226,10 +250,10 @@ Repl_Agmt *
 agmt_new_from_entry(Slapi_Entry *e)
 {
 	Repl_Agmt *ra;
-	char *tmpstr;
 	Slapi_Attr *sattr;
+	char *tmpstr;
+	char **clean_vals = NULL;
 	char **denied_attrs = NULL;
-
 	char *auto_initialize = NULL;
 	char *val_nsds5BeginReplicaRefresh = "start";
 
@@ -317,6 +341,26 @@ agmt_new_from_entry(Slapi_Entry *e)
 		ra->replarea = slapi_sdn_new_dn_passin(tmpstr);
 	}
 
+	/* Replica enabled */
+	tmpstr = slapi_entry_attr_get_charptr(e, type_nsds5ReplicaEnabled);
+	if (NULL != tmpstr)
+	{
+		if(strcasecmp(tmpstr, "off") == 0){
+			ra->is_enabled = PR_FALSE;
+		} else if(strcasecmp(tmpstr, "on") == 0){
+			ra->is_enabled = PR_TRUE;
+		} else {
+			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Warning invalid value "
+			    "for nsds5ReplicaEnabled (%s), value must be \"on\" or \"off\".  Ignoring "
+			    "this repl agreement.\n",tmpstr);
+			slapi_ch_free_string(&tmpstr);
+			goto loser;
+		}
+		slapi_ch_free_string(&tmpstr);
+	} else {
+		ra->is_enabled = PR_TRUE;
+	}
+
 	/* Replication schedule */
 	ra->schedule = schedule_new(update_window_state_change_callback, ra, agmt_get_long_name(ra));
 	if (slapi_entry_attr_find(e, type_nsds5ReplicaUpdateSchedule, &sattr) == 0)
@@ -387,6 +431,20 @@ agmt_new_from_entry(Slapi_Entry *e)
 	ra->last_init_start_time = 0UL;
 	ra->last_init_status[0] = '\0';
 	
+	/* cleanruv notification */
+	clean_vals = slapi_entry_attr_get_charray(e, type_nsds5ReplicaCleanRUVnotified);
+	if(clean_vals){
+		int i;
+		for (i = 0; i < CLEANRIDSIZ && clean_vals[i]; i++){
+			ra->cleanruv_notified[i] = atoi(clean_vals[i]);
+		}
+		if(i < CLEANRIDSIZ)
+		    ra->cleanruv_notified[i + 1] = 0;
+		slapi_ch_array_free(clean_vals);
+	} else {
+		ra->cleanruv_notified[0] = 0;
+	}
+
 	/* Fractional attributes */
 	slapi_entry_attr_find(e, type_nsds5ReplicatedAttributeList, &sattr);
 
@@ -434,6 +492,16 @@ agmt_new_from_entry(Slapi_Entry *e)
 		slapi_ch_array_free(denied_attrs);
 		goto loser;
 	}
+	/*
+	 *  Extract the attributes to strip for "empty" mods
+	 */
+	ra->attrs_to_strip = NULL;
+	tmpstr = slapi_entry_attr_get_charptr(e, type_nsds5ReplicaStripAttrs);
+	if (NULL != tmpstr)
+	{
+		ra->attrs_to_strip = slapi_str2charray_ext(tmpstr, " ", 0);
+		slapi_ch_free_string(&tmpstr);
+	}
 
 	if (!agmt_is_valid(ra))
 	{
@@ -539,6 +607,10 @@ agmt_delete(void **rap)
 		windows_agreement_delete(ra);
 	}
 
+	if(ra->attrs_to_strip){
+		slapi_ch_array_free(ra->attrs_to_strip);
+	}
+
 	schedule_destroy(ra->schedule);
 	slapi_ch_free((void **)&ra->long_name);
 	slapi_ch_free((void **)rap);
@@ -1633,6 +1705,20 @@ agmt_set_timeout_from_entry(Repl_Agmt *ra, const Slapi_Entry *e)
 	return return_value;
 }
 
+int
+agmt_set_timeout(Repl_Agmt *ra, long timeout)
+{
+    PR_Lock(ra->lock);
+    if (ra->stop_in_progress){
+        PR_Unlock(ra->lock);
+        return -1;
+    }
+    ra->timeout = timeout;
+    PR_Unlock(ra->lock);
+
+    return 0;
+}
+
 /*
  * Set or reset the busywaittime
  *
@@ -2046,6 +2132,7 @@ agmt_set_last_update_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *
 		else if (ldaprc != LDAP_SUCCESS)
 		{
 			char *replmsg = NULL;
+
 			if ( replrc ) {
 				replmsg = protocol_response2string(replrc);
 				/* Do not mix the unknown replication error with the known ldap one */
@@ -2053,19 +2140,9 @@ agmt_set_last_update_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *
 					replmsg = NULL;
 				}
 			}
-			if (ldaprc > 0) {
-				PR_snprintf(ra->last_update_status, STATUS_LEN,
-							"%d %s%sLDAP error: %s%s%s",
-							ldaprc, 
-							message?message:"",message?"":" - ",
-							ldap_err2string(ldaprc),
-							replmsg ? " - " : "", replmsg ? replmsg : "");
-			} else { /* ldaprc is < 0 */
-				PR_snprintf(ra->last_update_status, STATUS_LEN,
-							"%d %s%sSystem error%s%s",
-							ldaprc,message?message:"",message?"":" - ",
-							replmsg ? " - " : "", replmsg ? replmsg : "");
-			}
+			PR_snprintf(ra->last_update_status, STATUS_LEN, "%d %s%sLDAP error: %s%s%s",
+				ldaprc, message?message:"",message?"":" - ",
+				slapi_err2string(ldaprc), replmsg ? " - " : "", replmsg ? replmsg : "");
 		}
 		/* ldaprc == LDAP_SUCCESS */
 		else if (replrc != 0)
@@ -2123,6 +2200,7 @@ agmt_set_last_init_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *me
 		if (ldaprc != LDAP_SUCCESS)
 		{
 			char *replmsg = NULL;
+
 			if ( replrc ) {
 				replmsg = protocol_response2string(replrc);
 				/* Do not mix the unknown replication error with the known ldap one */
@@ -2130,19 +2208,9 @@ agmt_set_last_init_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *me
 					replmsg = NULL;
 				}
 			}
-			if (ldaprc > 0) {
-				PR_snprintf(ra->last_init_status, STATUS_LEN,
-							"%d %s%sLDAP error: %s%s%s",
-							ldaprc, 
-							message?message:"",message?"":" - ",
-							ldap_err2string(ldaprc),
-							replmsg ? " - " : "", replmsg ? replmsg : "");
-			} else { /* ldaprc is < 0 */
-				PR_snprintf(ra->last_init_status, STATUS_LEN,
-							"%d %s%sSystem error%s%s",
-							ldaprc,message?message:"",message?"":" - ",
-							replmsg ? " - " : "", replmsg ? replmsg : "");
-			}
+			PR_snprintf(ra->last_init_status, STATUS_LEN, "%d %s%sLDAP error: %s%s%s",
+				ldaprc, message?message:"",message?"":" - ",
+				slapi_err2string(ldaprc), replmsg ? " - " : "", replmsg ? replmsg : "");
 		}
 		/* ldaprc == LDAP_SUCCESS */
 		else if (replrc != 0)
@@ -2197,6 +2265,17 @@ agmt_set_update_in_progress (Repl_Agmt *ra, PRBool in_progress)
 	}
 }
 
+PRBool
+agmt_get_update_in_progress (const Repl_Agmt *ra)
+{
+	PR_ASSERT(NULL != ra);
+	if (NULL != ra)
+	{
+		return ra->update_in_progress;
+	}
+	return PR_FALSE;
+}
+
 void
 agmt_inc_last_update_changecount (Repl_Agmt *ra, ReplicaId rid, int skipped)
 {
@@ -2425,6 +2504,15 @@ ReplicaId agmt_get_consumerRID(Repl_Agmt *ra)
 	return ra->consumerRID;
 }
 
+void* agmt_get_connection(Repl_Agmt *ra)
+{
+	if(ra->protocol){
+		return (void *)prot_get_connection(ra->protocol);
+	} else {
+		return NULL;
+	}
+}
+
 int
 agmt_has_protocol(Repl_Agmt *agmt)
 {
@@ -2433,3 +2521,202 @@ agmt_has_protocol(Repl_Agmt *agmt)
 	}
 	return 0;
 }
+
+PRBool
+agmt_is_enabled(Repl_Agmt *ra)
+{
+	PRBool state;
+	PR_Lock(ra->lock);
+	state = ra->is_enabled;
+	PR_Unlock(ra->lock);
+
+	return state;
+}
+
+int
+agmt_set_enabled_from_entry(Repl_Agmt *ra, Slapi_Entry *e, char *returntext){
+	char *attr_val = NULL;
+	int rc = 0;
+
+	if(ra == NULL){
+		return -1;
+	}
+
+	PR_Lock(ra->lock);
+	attr_val = slapi_entry_attr_get_charptr(e, type_nsds5ReplicaEnabled);
+	if(attr_val){
+		PRBool is_enabled = PR_TRUE;
+		if(strcasecmp(attr_val,"off") == 0){
+			is_enabled = PR_FALSE;
+		} else if(strcasecmp(attr_val,"on") == 0){
+			is_enabled = PR_TRUE;
+		} else {
+			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "agmt_set_enabled_from_entry: invalid "
+			    "value for nsds5ReplicaEnabled (%s), the value must be \"on\" or \"off\".\n", attr_val);
+			PR_snprintf(returntext, SLAPI_DSE_RETURNTEXT_SIZE, "Invalid value for nsds5ReplicaEnabled, "
+			    "the value must be \"on\" or \"off\".\n");
+			slapi_ch_free_string(&attr_val);
+			PR_Unlock(ra->lock);
+			return -1;
+		}
+		slapi_ch_free_string(&attr_val);
+		if(is_enabled){
+			if(!ra->is_enabled){
+				ra->is_enabled = PR_TRUE;
+				slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name, "agmt_set_enabled_from_entry: "
+					"agreement is now enabled (%s)\n",ra->long_name);
+				PR_Unlock(ra->lock);
+				agmt_start(ra);
+				return rc;
+			}
+		} else {
+			if(ra->is_enabled){
+				ra->is_enabled = PR_FALSE;
+				slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name, "agmt_set_enabled_from_entry: "
+					"agreement is now disabled (%s)\n",ra->long_name);
+				PR_Unlock(ra->lock);
+				agmt_stop(ra);
+				agmt_update_consumer_ruv(ra);
+				agmt_set_last_update_status(ra,0,0,"agreement disabled");
+				return rc;
+			}
+		}
+	} else {
+		rc = -1;
+	}
+	PR_Unlock(ra->lock);
+
+	return rc;
+}
+
+char **
+agmt_get_attrs_to_strip(Repl_Agmt *ra)
+{
+	if(ra){
+		return ra->attrs_to_strip;
+	} else {
+		return NULL;
+	}
+}
+
+int
+agmt_set_attrs_to_strip(Repl_Agmt *ra, Slapi_Entry *e)
+{
+    char *tmpstr = NULL;
+
+    PR_Lock(ra->lock);
+
+    tmpstr = slapi_entry_attr_get_charptr(e, type_nsds5ReplicaStripAttrs);
+    if (NULL != tmpstr){
+        if(ra->attrs_to_strip){
+            slapi_ch_array_free(ra->attrs_to_strip);
+        }
+        ra->attrs_to_strip = slapi_str2charray_ext(tmpstr, " ", 0);
+        PR_Unlock(ra->lock);
+        prot_notify_agmt_changed(ra->protocol, ra->long_name);
+        slapi_ch_free_string(&tmpstr);
+        return 0;
+    }
+
+    PR_Unlock(ra->lock);
+
+    return -1;
+}
+
+int
+agmt_is_cleanruv_notified(Repl_Agmt *ra, ReplicaId rid){
+    int notified = 0;
+    int i;
+
+    PR_Lock(ra->lock);
+    for(i = 0; i < CLEANRIDSIZ && ra->cleanruv_notified[i]; i++){
+        if(ra->cleanruv_notified[i] == rid){
+            notified = 1;
+            break;
+        }
+    }
+    PR_Unlock(ra->lock);
+
+    return notified;
+}
+
+/*
+ *  This will trigger agmt_set_cleanruv_notified_from_entry() to be called,
+ *  which will update the in memory agmt.
+ *
+ *  op can be:  CLEANRUV_NOTIFIED or CLEANRUV_RELEASED
+ */
+int
+agmt_set_cleanruv_data(Repl_Agmt *ra, ReplicaId rid, int op){
+    Slapi_PBlock *pb;
+    LDAPMod *mods[2];
+    LDAPMod mod;
+    struct berval *vals[2];
+    struct berval val;
+    char data[6];
+    int rc = 0;
+
+    if(ra == NULL){
+        return -1;
+    }
+
+    if(op == CLEANRUV_NOTIFIED){
+        /* add the cleanruv data */
+    	mod.mod_op  = LDAP_MOD_ADD|LDAP_MOD_BVALUES;
+    } else {
+        /* remove the cleanruv data */
+    	mod.mod_op  = LDAP_MOD_DELETE|LDAP_MOD_BVALUES;
+    }
+
+    pb = slapi_pblock_new();
+    val.bv_len = PR_snprintf(data, sizeof(data), "%d", (int)rid);
+    mod.mod_type = (char *)type_nsds5ReplicaCleanRUVnotified;
+    mod.mod_bvalues = vals;
+    vals [0] = &val;
+    vals [1] = NULL;
+    val.bv_val = data;
+    mods[0] = &mod;
+    mods[1] = NULL;
+
+    slapi_modify_internal_set_pb_ext (pb, ra->dn, mods, NULL, NULL,
+        repl_get_plugin_identity (PLUGIN_MULTIMASTER_REPLICATION), 0);
+    slapi_modify_internal_pb (pb);
+    slapi_pblock_destroy(pb);
+
+    return rc;
+}
+
+void
+agmt_set_cleanruv_notified_from_entry(Repl_Agmt *ra, Slapi_Entry *e){
+    char **attr_vals = NULL;
+    int i;
+
+    PR_Lock(ra->lock);
+    attr_vals = slapi_entry_attr_get_charray(e, type_nsds5ReplicaCleanRUVnotified);
+    if(attr_vals){
+        for (i = 0; i < CLEANRIDSIZ && attr_vals[i]; i++){
+            ra->cleanruv_notified[i] = atoi(attr_vals[i]);
+        }
+        if( i < CLEANRIDSIZ )
+            ra->cleanruv_notified[i + 1] = 0;
+        slapi_ch_array_free(attr_vals);
+    } else {
+        ra->cleanruv_notified[0] = 0;
+    }
+    PR_Unlock(ra->lock);
+}
+
+/* this is called whenever an update (total/incremental)
+   is completed */
+void 
+agmt_update_done(Repl_Agmt *agmt, int is_total)
+{
+    /* we could do a lot of stuff here - consolidate all of the other stuff that gets
+       done at the end of an update - setting status, times, etc.
+       but for now, all we want to do is "flush" any pending changes made
+       during the update into the proper structures so they are in place for the
+       next run
+    */
+    windows_update_done(agmt, is_total);
+}
+
