@@ -140,7 +140,7 @@ static void repl5_debug_timeout_callback(time_t when, void *arg);
 static void close_connection_internal(Repl_Connection *conn);
 
 /*
- * Create a new conenction object. Returns a pointer to the object, or
+ * Create a new connection object. Returns a pointer to the object, or
  * NULL if an error occurs.
  */
 Repl_Connection *
@@ -216,6 +216,17 @@ conn_delete_internal(Repl_Connection *conn)
 	slapi_ch_free((void **)&conn->plain);
 }
 
+/*
+ *  Used by CLEANALLRUV - free it all!
+ */
+void
+conn_delete_internal_ext(Repl_Connection *conn)
+{
+    conn_delete_internal(conn);
+    PR_DestroyLock(conn->lock);
+    slapi_ch_free((void **)&conn);
+}
+
 /*
  * Destroy a connection. It is an error to use the connection object
  * after conn_delete() has been called.
@@ -1710,6 +1721,16 @@ conn_get_timeout(Repl_Connection *conn)
 	return retval;
 }
 
+LDAP *
+conn_get_ldap(Repl_Connection *conn)
+{
+	if(conn){
+		return conn->ld;
+	} else {
+		return NULL;
+	}
+}
+
 void conn_set_agmt_changed(Repl_Connection *conn)
 {
 	PR_ASSERT(NULL != conn);
@@ -1908,3 +1929,19 @@ repl5_debug_timeout_callback(time_t when, void *arg)
 		"repl5_debug_timeout_callback: set debug level to %d at %ld\n",
 		s_debug_level, when);
 }
+
+void
+conn_lock(Repl_Connection *conn)
+{
+	if(conn){
+		PR_Lock(conn->lock);
+	}
+}
+
+void
+conn_unlock(Repl_Connection *conn)
+{
+	if(conn){
+		PR_Unlock(conn->lock);
+	}
+}
