@@ -118,6 +118,22 @@ static char *response_name_list[] = {
 		NSDS_REPL_NAME_PREFIX " Response",
 		NULL
 };
+static char *cleanruv_oid_list[] = {
+		REPL_CLEANRUV_OID,
+		NULL
+};
+static char *cleanruv_name_list[] = {
+		NSDS_REPL_NAME_PREFIX " Cleanruv",
+		NULL
+};
+static char *cleanruv_abort_oid_list[] = {
+		REPL_ABORT_CLEANRUV_OID,
+		NULL
+};
+static char *cleanruv_abort_name_list[] = {
+		NSDS_REPL_NAME_PREFIX " Cleanruv Abort",
+		NULL
+};
 
 /* List of plugin identities for every plugin registered. Plugin identity
    is passed by the server in the plugin init function and must be supplied 
@@ -434,6 +450,51 @@ multimaster_response_extop_init( Slapi_PBlock *pb )
     return rc;
 }
 
+int
+multimaster_cleanruv_extop_init( Slapi_PBlock *pb )
+{
+	int rc= 0; /* OK */
+	void *identity = NULL;
+
+	/* get plugin identity and store it to pass to internal operations */
+	slapi_pblock_get (pb, SLAPI_PLUGIN_IDENTITY, &identity);
+	PR_ASSERT (identity);
+
+	if (slapi_pblock_set( pb, SLAPI_PLUGIN_VERSION, SLAPI_PLUGIN_VERSION_01 ) != 0 ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_DESCRIPTION, (void *)&multimasterextopdesc ) != 0 ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_EXT_OP_OIDLIST, (void *)cleanruv_oid_list ) != 0  ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_EXT_OP_NAMELIST, (void *)cleanruv_name_list ) != 0  ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_EXT_OP_FN, (void *)multimaster_extop_cleanruv ))
+	{
+		slapi_log_error( SLAPI_LOG_PLUGIN, repl_plugin_name, "multimaster_cleanruv_extop_init failed\n" );
+		rc= -1;
+	}
+
+	return rc;
+}
+
+int
+multimaster_cleanruv_abort_extop_init( Slapi_PBlock *pb )
+{
+	int rc= 0; /* OK */
+	void *identity = NULL;
+
+	/* get plugin identity and store it to pass to internal operations */
+	slapi_pblock_get (pb, SLAPI_PLUGIN_IDENTITY, &identity);
+	PR_ASSERT (identity);
+
+	if (slapi_pblock_set( pb, SLAPI_PLUGIN_VERSION, SLAPI_PLUGIN_VERSION_01 ) != 0 ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_DESCRIPTION, (void *)&multimasterextopdesc ) != 0 ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_EXT_OP_OIDLIST, (void *)cleanruv_abort_oid_list ) != 0  ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_EXT_OP_NAMELIST, (void *)cleanruv_abort_name_list ) != 0  ||
+		slapi_pblock_set( pb, SLAPI_PLUGIN_EXT_OP_FN, (void *)multimaster_extop_abort_cleanruv ))
+	{
+		slapi_log_error( SLAPI_LOG_PLUGIN, repl_plugin_name, "multimaster_cleanruv_abort_extop_init failed\n" );
+		rc= -1;
+	}
+
+	return rc;
+}
 
 static PRBool
 check_for_ldif_dump(Slapi_PBlock *pb)
@@ -544,24 +605,22 @@ multimaster_stop( Slapi_PBlock *pb )
     int rc= 0; /* OK */
 
     if (!multimaster_stopped_flag)
-	{
-		if (!is_ldif_dump)
-		{
-			agmtlist_shutdown(); /* Shut down replication agreements */
-		}
-
+    {
+        if (!is_ldif_dump)
+        {
+            /* Shut down replication agreements */
+            agmtlist_shutdown();
+        }
+        /* if we are cleaning a ruv, stop */
+        stop_ruv_cleaning();
         /* unregister backend state change notification */
         slapi_unregister_backend_state_change((void *)multimaster_be_state_change);
-
-		changelog5_cleanup(); /* Shut down the changelog */
-		multimaster_mtnode_extension_destroy(); /* Destroy mapping tree node exts */
+        changelog5_cleanup(); /* Shut down the changelog */
+        multimaster_mtnode_extension_destroy(); /* Destroy mapping tree node exts */
         replica_destroy_name_hash(); /* destroy the hash and its remaining content */
         replica_config_destroy (); /* Destroy replica config info */
-    	multimaster_stopped_flag = 1;
-		/* JCMREPL - Wait for all our threads to stop */
-		/* JCMREPL - Shut down the replication plugin */
-		/* JCMREPL - Mark all the replication plugin interfaces at not enabled. */
-	}
+        multimaster_stopped_flag = 1;
+    }
     return rc;
 }
 
@@ -618,6 +677,8 @@ int replication_multimaster_plugin_init(Slapi_PBlock *pb)
 		rc= slapi_register_plugin("extendedop", 1 /* Enabled */, "multimaster_end_extop_init", multimaster_end_extop_init, "Multimaster replication end extended operation plugin", NULL, identity);
 		rc= slapi_register_plugin("extendedop", 1 /* Enabled */, "multimaster_total_extop_init", multimaster_total_extop_init, "Multimaster replication total update extended operation plugin", NULL, identity);
 		rc= slapi_register_plugin("extendedop", 1 /* Enabled */, "multimaster_response_extop_init", multimaster_response_extop_init, "Multimaster replication extended response plugin", NULL, identity);
+		rc= slapi_register_plugin("extendedop", 1 /* Enabled */, "multimaster_cleanruv_extop_init", multimaster_cleanruv_extop_init, "Multimaster replication cleanruv extended operation plugin", NULL, identity);
+		rc= slapi_register_plugin("extendedop", 1 /* Enabled */, "multimaster_cleanruv_abort_extop_init", multimaster_cleanruv_abort_extop_init, "Multimaster replication cleanruv abort extended operation plugin", NULL, identity);
 		if (0 == rc)
 		{
 			multimaster_initialised = 1;
