@@ -997,6 +997,11 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 		return 0;
 	}
 
+	slapi_pblock_get(pb, SLAPI_RESULT_CODE, &rc);
+	if (rc) { /* op failed - just return */
+		return 0;
+	}
+
 	/* we only log changes for operations applied to a replica */
 	repl_obj = replica_get_replica_for_op (pb);
 	if (repl_obj == NULL)
@@ -1059,6 +1064,12 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 			op_params->target_address.uniqueid = slapi_ch_strdup (uniqueid);
 		} 
 
+		if( is_cleaned_rid(csn_get_replicaid(op_params->csn))){
+			/* this RID has been cleaned */
+			object_release (repl_obj);
+			return 0;
+		}
+
 		/* we might have stripped all the mods - in that case we do not
 		   log the operation */
 		if (op_params->operation_type != SLAPI_OPERATION_MODIFY ||
@@ -1313,7 +1324,7 @@ process_operation (Slapi_PBlock *pb, const CSN *csn)
     ruv = (RUV*)object_get_data (ruv_obj);
     PR_ASSERT (ruv);
  
-    rc = ruv_add_csn_inprogress (ruv, csn);    
+    rc = ruv_add_csn_inprogress (ruv, csn);
 
     object_release (ruv_obj);
     object_release (r_obj);
@@ -1361,6 +1372,9 @@ static const char *replica_get_purl_for_op (const Replica *r, Slapi_PBlock *pb,
 		Slapi_Connection *conn;
 		consumer_connection_extension *connext;
 		slapi_pblock_get(pb, SLAPI_CONNECTION, &conn);
+		/* TEL 20120531: There is a slim chance we want to take exclusive access
+		 * to this instead.  However, it isn't clear to me that it is worth the 
+		 * risk of changing this working code. */
 		connext = (consumer_connection_extension *)repl_con_get_ext(
 			REPL_CON_EXT_CONN, conn);
 		if (NULL == connext || NULL == connext->supplier_ruv)
