@@ -46,7 +46,10 @@
 #include "repl5.h"
 #include "repl5_prot_private.h"
 #include "cl5_api.h"
-
+#define ENABLE_TEST_TICKET_374
+#ifdef ENABLE_TEST_TICKET_374
+#include <unistd.h> /* for usleep */
+#endif
 
 /*
  * repl_extop.c - there are two types of functions in this file:
@@ -71,6 +74,7 @@
  *
  */
 static int check_replica_id_uniqueness(Replica *replica, RUV *supplier_ruv);
+static multimaster_mtnode_extension *replica_config_get_mtnode_by_dn(const char *dn);
 
 static int 
 encode_ruv (BerElement *ber, const RUV *ruv)
@@ -659,12 +663,16 @@ multimaster_extop_StartNSDS50ReplicationRequest(Slapi_PBlock *pb)
 	 * make sure it's there.
 	 */
 	slapi_pblock_get(pb, SLAPI_CONNECTION, &conn);
-	connext = (consumer_connection_extension *)repl_con_get_ext(
-			REPL_CON_EXT_CONN, conn);
+	connext = consumer_connection_extension_acquire_exclusive_access(conn, connid, opid);
 	if (NULL == connext)
 	{
-		/* Something bad happened. Don't go any further */
-		response = NSDS50_REPL_INTERNAL_ERROR;
+		/* TEL 20120531: This used to be a much worse and unexpected thing 
+		 * before acquiring exclusive access to the connext.  Now it should
+		 * be highly unusual, but not completely unheard of.  We don't want to
+		 * return an internal error here as before, because it will eventually
+		 * result in a fatal error on the other end.  Better to tell it
+		 * we are busy instead--which is also probably true. */
+		response = NSDS50_REPL_REPLICA_BUSY;
 		goto send_response;
 	}
 
@@ -732,7 +740,7 @@ multimaster_extop_StartNSDS50ReplicationRequest(Slapi_PBlock *pb)
 		/* Stash info that this is a total update session */
 		if (NULL != connext)
 		{
-			connext->repl_protocol_version = REPL_PROTOCOL_71_TOTALUPDATE;
+			connext->repl_protocol_version = REPL_PROTOCOL_50_TOTALUPDATE;
 		}
 		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
 				"conn=%" NSPRIu64 " op=%d repl=\"%s\": Begin 7.1 total protocol\n",
@@ -896,6 +904,27 @@ multimaster_extop_StartNSDS50ReplicationRequest(Slapi_PBlock *pb)
         replica_object = NULL;
 	}
 
+	/* remove this code once ticket 374 is fixed */
+#ifdef ENABLE_TEST_TICKET_374
+	if (getenv("SLAPD_TEST_TICKET_374") && (opid > 20)) {
+		int i = 0;
+		int max = 480 * 5;
+		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+						"conn=%" NSPRIu64 " op=%d repl=\"%s\": "
+						"374 - Starting sleep: connext->repl_protocol_version == %d\n",
+						connid, opid, repl_root, connext->repl_protocol_version);
+        
+		while (REPL_PROTOCOL_50_INCREMENTAL == connext->repl_protocol_version && i++ < max) {
+			usleep(200000);
+		}
+        
+		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+						"conn=%" NSPRIu64 " op=%d repl=\"%s\": "
+						"374 - Finished sleep: connext->repl_protocol_version == %d\n",
+						connid, opid, repl_root, connext->repl_protocol_version);
+	}
+#endif
+
 	/* If this is incremental protocol get replica's ruv to return to the supplier */
     if (connext->repl_protocol_version == REPL_PROTOCOL_50_INCREMENTAL)
     {
@@ -920,12 +949,14 @@ multimaster_extop_StartNSDS50ReplicationRequest(Slapi_PBlock *pb)
 	 */
 	ruv_copy_and_destroy(&supplier_ruv, (RUV **)&connext->supplier_ruv);
 
+    /* incremental update protocol */
     if (connext->repl_protocol_version == REPL_PROTOCOL_50_INCREMENTAL)
     {
 		/* The supplier ruv may have changed, so let's update the referrals */
 		consumer5_set_mapping_tree_state_for_replica(replica, connext->supplier_ruv);
     }
-    else /* full protocol */
+    /* total update protocol */
+    else if (connext->repl_protocol_version == REPL_PROTOCOL_50_TOTALUPDATE)
     {
 		char *mtnstate = slapi_mtn_get_state(repl_root_sdn);
 		char **mtnreferral = slapi_mtn_get_referral(repl_root_sdn);
@@ -968,6 +999,26 @@ multimaster_extop_StartNSDS50ReplicationRequest(Slapi_PBlock *pb)
 		charray_free(mtnreferral);
 		mtnreferral = NULL;
     }
+    /* something unexpected at this point, like REPL_PROTOCOL_UNKNOWN */
+    else
+    {
+		/* TEL 20120529: This condition isn't supposed to happen, but it
+		 * has been observed in the past when the consumer is under such
+		 * stress that the supplier sends additional start extops before
+		 * the consumer has finished processing an earlier one.  Fixing
+		 * the underlying race should prevent this from happening in the
+		 * future at all, but just in case it is still worth testing the
+		 * requested protocol explictly and returning an error here rather
+		 * than assuming a total update was requested. 
+		 * https://fedorahosted.org/389/ticket/374 */
+		response = NSDS50_REPL_INTERNAL_ERROR;
+		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+				"conn=%" NSPRIu64 " op=%d repl=\"%s\": "
+				"Unexpected update protocol received: %d.  "
+				"Expected incremental or total.\n",
+				connid, opid, repl_root, connext->repl_protocol_version);
+		goto send_response;
+    }
 
 	response = NSDS50_REPL_REPLICA_READY;
 	/* Set the "is replication session" flag in the connection extension */
@@ -1147,6 +1198,13 @@ send_response:
 	{
 		ber_bvecfree(ruv_bervals);
 	}
+	/* connext (our hold on it at least) */
+	if (NULL != connext)
+	{
+		/* don't free it, just let go of it */
+		consumer_connection_extension_relinquish_exclusive_access(conn, connid, opid, PR_FALSE);
+		connext = NULL;
+	}
 
 	return return_value;
 }
@@ -1185,8 +1243,15 @@ multimaster_extop_EndNSDS50ReplicationRequest(Slapi_PBlock *pb)
 		 */
 		/* Get a hold of the connection extension object */
 		slapi_pblock_get(pb, SLAPI_CONNECTION, &conn);
-		connext = (consumer_connection_extension *)repl_con_get_ext(
-				REPL_CON_EXT_CONN, conn);
+		slapi_pblock_get (pb, SLAPI_OPERATION_ID, &opid);
+		if (opid) slapi_pblock_get (pb, SLAPI_CONN_ID, &connid);
+        
+		/* TEL 20120531: unlike the replica, exclusive access to the connext should
+		 * have been dropped at the end of the 'start' op.  the only reason we couldn't
+		 * get access to it would be if some other start or end op currently has it.
+		 * if that is the case, the result of our getting it would be unpredictable anyway.
+		 */
+		connext = consumer_connection_extension_acquire_exclusive_access(conn, connid, opid);
 		if (NULL != connext && NULL != connext->replica_acquired)
 		{
 			int zero= 0;
@@ -1260,13 +1325,10 @@ multimaster_extop_EndNSDS50ReplicationRequest(Slapi_PBlock *pb)
 			{
 				/* The ruv from the supplier may have changed. Report the change on the
 					consumer side */
-
 				replica_update_ruv_consumer(r, connext->supplier_ruv);
 			}
 
 			/* Relinquish control of the replica */
-			slapi_pblock_get (pb, SLAPI_OPERATION_ID, &opid);
-			if (opid) slapi_pblock_get (pb, SLAPI_CONN_ID, &connid);
 			replica_relinquish_exclusive_access(r, connid, opid);
 			object_release ((Object*)connext->replica_acquired);
 			connext->replica_acquired = NULL;
@@ -1310,10 +1372,392 @@ free_and_return:
 	{
 		ber_bvfree(resp_bval);
 	}
+	/* connext (our hold on it at least) */
+	if (NULL != connext)
+	{
+		/* don't free it, just let go of it */
+		consumer_connection_extension_relinquish_exclusive_access(conn, connid, opid, PR_FALSE);
+		connext = NULL;
+	}
 
 	return return_value;
 }
 
+/*
+ *  Return the mtnode extension of the dn
+ */
+static multimaster_mtnode_extension *
+replica_config_get_mtnode_by_dn(const char *dn)
+{
+	Slapi_DN *sdn;
+	mapping_tree_node *mtnode;
+	multimaster_mtnode_extension *ext = NULL;
+
+	sdn = slapi_sdn_new_dn_byval(dn);
+	mtnode = slapi_get_mapping_tree_node_by_dn (sdn);
+	if (mtnode)	{
+		/* check if the replica object already exists in the subtree */
+		ext = (multimaster_mtnode_extension *)repl_con_get_ext (REPL_CON_EXT_MTNODE, mtnode);
+	}
+	slapi_sdn_free (&sdn);
+
+	return ext;
+}
+
+/*
+ * Decode the ber element passed to us by the cleanAllRUV task
+ */
+int
+decode_cleanruv_payload(struct berval *extop_value, char **payload)
+{
+	BerElement *tmp_bere = NULL;
+	int rc = 0;
+
+	if ((tmp_bere = ber_init(extop_value)) == NULL){
+		rc = -1;
+		goto free_and_return;
+	}
+	if (ber_scanf(tmp_bere, "{") == LBER_ERROR){
+		rc = -1;
+		goto free_and_return;
+	}
+	if (ber_get_stringa(tmp_bere, payload) == LBER_DEFAULT){
+		rc = -1;
+		goto free_and_return;
+	}
+
+	if (ber_scanf(tmp_bere, "}") == LBER_ERROR){
+		rc = -1;
+		goto free_and_return;
+	}
+
+free_and_return:
+	if (-1 == rc){
+		slapi_ch_free_string(payload);
+	}
+	if (NULL != tmp_bere){
+		ber_free(tmp_bere, 1);
+		tmp_bere = NULL;
+	}
+	return rc;
+}
+
+int
+multimaster_extop_abort_cleanruv(Slapi_PBlock *pb)
+{
+	multimaster_mtnode_extension *mtnode_ext;
+	PRThread *thread = NULL;
+	cleanruv_data *data;
+	Replica *r;
+	ReplicaId rid;
+	CSN *maxcsn;
+	struct berval *extop_payload;
+	char *extop_oid;
+	char *repl_root;
+	char *payload = NULL;
+	char *certify_all;
+	char *iter;
+	int rc = 0;
+
+	slapi_pblock_get(pb, SLAPI_EXT_OP_REQ_OID, &extop_oid);
+	slapi_pblock_get(pb, SLAPI_EXT_OP_REQ_VALUE, &extop_payload);
+
+	if (NULL == extop_oid || strcmp(extop_oid, REPL_CLEANRUV_OID) != 0 ||
+		NULL == extop_payload || NULL == extop_payload->bv_val){
+		/* something is wrong, error out */
+		return LDAP_OPERATIONS_ERROR;
+	}
+	/*
+	 *  Decode the payload, and grab our settings
+	 */
+	if(decode_cleanruv_payload(extop_payload, &payload)){
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Abort cleanAllRUV task: failed to decode payload.  Aborting ext op\n");
+		return LDAP_OPERATIONS_ERROR;
+	}
+	rid = atoi(ldap_utf8strtok_r(payload, ":", &iter));
+	repl_root = ldap_utf8strtok_r(iter, ":", &iter);
+	certify_all = ldap_utf8strtok_r(iter, ":", &iter);
+
+	if(!is_cleaned_rid(rid) || is_task_aborted(rid)){
+		/* This replica has already been aborted, or was never cleaned, or already finished cleaning */
+		goto out;
+	} else {
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Abort cleanAllRUV task: aborting cleanallruv task for rid(%d)\n", rid);
+	}
+	/*
+	 *  Get the node, so we can get the replica and its agreements
+	 */
+	if((mtnode_ext = replica_config_get_mtnode_by_dn(repl_root)) == NULL){
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Abort cleanAllRUV task: failed to get replication node "
+			"from (%s), aborting operation\n", repl_root);
+		rc = LDAP_OPERATIONS_ERROR;
+		goto out;
+	}
+	if (mtnode_ext->replica){
+		object_acquire (mtnode_ext->replica);
+	} else {
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Abort cleanAllRUV task: replica is missing from (%s), "
+			"aborting operation\n",repl_root);
+		rc = LDAP_OPERATIONS_ERROR;
+		goto out;
+	}
+	r = (Replica*)object_get_data (mtnode_ext->replica);
+	if(r == NULL){
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Abort cleanAllRUV task: replica is NULL, aborting task\n");
+		rc = LDAP_OPERATIONS_ERROR;
+		goto out;
+	}
+	/*
+	 *  Prepare the abort data
+	 */
+	data = (cleanruv_data*)slapi_ch_calloc(1, sizeof(cleanruv_data));
+	if (data == NULL) {
+		slapi_log_error( SLAPI_LOG_REPL, repl_plugin_name, "Abort cleanAllRUV task: failed to allocate "
+			"abort_cleanruv_data.  Aborting task.\n");
+		rc = LDAP_OPERATIONS_ERROR;
+		goto out;
+	}
+	data->repl_obj = mtnode_ext->replica; /* released in replica_abort_task_thread() */
+	data->replica = r;
+	data->task = NULL;
+	data->payload = slapi_ch_bvdup(extop_payload);
+	data->rid = rid;
+	data->repl_root = slapi_ch_strdup(repl_root);
+	data->certify = slapi_ch_strdup(certify_all);
+	/*
+	 *  Stop the cleaning, and delete the rid
+	 */
+	maxcsn = replica_get_cleanruv_maxcsn(r, rid);
+	delete_cleaned_rid(r, rid, maxcsn);
+	csn_free(&maxcsn);
+	add_aborted_rid(rid, r, repl_root);
+	stop_ruv_cleaning();
+	/*
+	 *  Send out the extended ops to the replicas
+	 */
+	thread = PR_CreateThread(PR_USER_THREAD, replica_abort_task_thread,
+			(void *)data, PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD,
+			PR_UNJOINABLE_THREAD, SLAPD_DEFAULT_THREAD_STACKSIZE);
+	if (thread == NULL) {
+		if(mtnode_ext->replica){
+			object_release(mtnode_ext->replica);
+		}
+		slapi_log_error( SLAPI_LOG_REPL, repl_plugin_name, "Abort cleanAllRUV task: unable to create abort "
+			"thread.  Aborting task.\n");
+		slapi_ch_free_string(&data->repl_root);
+		slapi_ch_free_string(&data->certify);
+		rc = LDAP_OPERATIONS_ERROR;
+	}
+
+out:
+    slapi_ch_free_string(&payload);
+
+	return rc;
+}
+/*
+ *  Process the REPL_CLEANRUV_OID extended operation.
+ *
+ *  The payload consists of the replica ID, repl root dn, and the maxcsn.  Since this is
+ *  basically a replication operation, it could of originated here and bounced
+ *  back from another master.  So check the rid against the "cleaned_rid".  If
+ *  it's a match, then we were already here, and we can just return success.
+ *
+ *  Otherwise, we the set the cleaned_rid from the payload, fire off extended ops
+ *  to all the replica agreements on this replica.  Then perform the actual
+ *  cleanruv_task on this replica.
+ */
+int
+multimaster_extop_cleanruv(Slapi_PBlock *pb)
+{
+	multimaster_mtnode_extension *mtnode_ext;
+	PRThread *thread = NULL;
+	Replica *r = NULL;
+	cleanruv_data *data = NULL;
+	CSN *maxcsn = NULL;
+	struct berval *extop_payload;
+	struct berval *resp_bval = NULL;
+	BerElement *resp_bere = NULL;
+	char *extop_oid;
+	char *repl_root;
+	char *payload = NULL;
+	char *csnstr = NULL;
+	char *iter;
+	int release_it = 0;
+	int rid = 0;
+	int rc = 0;
+
+	slapi_pblock_get(pb, SLAPI_EXT_OP_REQ_OID, &extop_oid);
+	slapi_pblock_get(pb, SLAPI_EXT_OP_REQ_VALUE, &extop_payload);
+
+	if (NULL == extop_oid || strcmp(extop_oid, REPL_CLEANRUV_OID) != 0 ||
+	    NULL == extop_payload || NULL == extop_payload->bv_val){
+		/* something is wrong, error out */
+		rc = -1;
+		goto free_and_return;
+	}
+	/*
+	 *  Decode the payload
+	 */
+	if(decode_cleanruv_payload(extop_payload, &payload)){
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: failed to decode payload.  Aborting ext op\n");
+		rc = -1;
+		goto free_and_return;
+	}
+	rid = atoi(ldap_utf8strtok_r(payload, ":", &iter));
+	repl_root = ldap_utf8strtok_r(iter, ":", &iter);
+	csnstr = ldap_utf8strtok_r(iter, ":", &iter);
+	maxcsn = csn_new();
+	csn_init_by_string(maxcsn, csnstr);
+	/*
+	 *  If we already cleaned this server, just return success
+	 */
+	if(is_cleaned_rid(rid)){
+		csn_free(&maxcsn);
+		rc = 1;
+		goto free_and_return;
+	}
+
+	/*
+	 *  Get the node, so we can get the replica and its agreements
+	 */
+	if((mtnode_ext = replica_config_get_mtnode_by_dn(repl_root)) == NULL){
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: failed to get replication node "
+			"from (%s), aborting operation\n", repl_root);
+		rc = -1;
+		goto free_and_return;
+	}
+
+	if (mtnode_ext->replica){
+		object_acquire (mtnode_ext->replica);
+		release_it = 1;
+	}
+	if (mtnode_ext->replica == NULL){
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: replica is missing from (%s), "
+			"aborting operation\n",repl_root);
+		rc = LDAP_OPERATIONS_ERROR;
+		goto free_and_return;
+	}
+
+	r = (Replica*)object_get_data (mtnode_ext->replica);
+	if(r == NULL){
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: replica is NULL, aborting task\n");
+		rc = -1;
+		goto free_and_return;
+	}
+
+	if(replica_get_type(r) != REPLICA_TYPE_READONLY){
+		/*
+		 *  Launch the cleanruv monitoring thread.  Once all the replicas are cleaned it will release the rid
+		 *
+		 *  This will also release mtnode_ext->replica
+		 */
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: launching cleanAllRUV thread...\n");
+		data = (cleanruv_data*)slapi_ch_calloc(1, sizeof(cleanruv_data));
+		if (data == NULL) {
+			slapi_log_error( SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: failed to allocate "
+				"cleanruv_Data\n");
+			rc = -1;
+			goto free_and_return;
+		}
+		data->repl_obj = mtnode_ext->replica;
+		data->replica = r;
+		data->rid = rid;
+		data->task = NULL;
+		data->maxcsn = maxcsn;
+		data->payload = slapi_ch_bvdup(extop_payload);
+
+		thread = PR_CreateThread(PR_USER_THREAD, replica_cleanallruv_thread_ext,
+				(void *)data, PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD,
+				PR_UNJOINABLE_THREAD, SLAPD_DEFAULT_THREAD_STACKSIZE);
+		if (thread == NULL) {
+		    rc = -1;
+			slapi_log_error( SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: unable to create cleanAllRUV "
+				"monitoring thread.  Aborting task.\n");
+		}
+	} else { /* this is a read-only consumer */
+		/*
+		 * wait for the maxcsn to be covered
+		 */
+		Object *ruv_obj;
+		const RUV *ruv;
+
+		ruv_obj = replica_get_ruv(r);
+		ruv = object_get_data (ruv_obj);
+
+		while(!is_task_aborted(rid) && !slapi_is_shutting_down()){
+			if(!ruv_contains_replica(ruv, rid)){
+				/* we've already been cleaned */
+				break;
+			}
+			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: checking if we're caught up...\n");
+			if(ruv_covers_csn_cleanallruv(ruv,maxcsn) || csn_get_replicaid(maxcsn) == 0){
+				/* We are caught up */
+				break;
+			} else {
+				char csnstr[CSN_STRSIZE];
+				csn_as_string(maxcsn, PR_FALSE, csnstr);
+				slapi_log_error( SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: not ruv caught up maxcsn(%s)\n", csnstr);
+			}
+			DS_Sleep(PR_SecondsToInterval(5));
+		}
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: we're caught up...\n");
+		/*
+		 *  Set cleaned rid in memory only - does not survive a server restart
+		 */
+		set_cleaned_rid(rid);
+		/*
+		 *  Clean the ruv
+		 */
+		replica_execute_cleanruv_task_ext(mtnode_ext->replica, rid);
+
+		/* free everything */
+		object_release(ruv_obj);
+		csn_free(&maxcsn);
+		if (mtnode_ext->replica && release_it)
+			object_release (mtnode_ext->replica);
+		/*
+		 *  This read-only replica has no easy way to tell when it's safe to release the rid.
+		 *  So we won't release it, not until a server restart.
+		 */
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: You must restart the server if you want to reuse rid(%d).\n", rid);
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "cleanAllRUV_task: Successfully cleaned rid(%d).\n", rid);
+	}
+
+free_and_return:
+	if(rc && release_it){
+		if (mtnode_ext->replica)
+			object_release (mtnode_ext->replica);
+	}
+	if(rc)
+	    csn_free(&maxcsn);
+	slapi_ch_free_string(&payload);
+
+	/*
+	 *   Craft a message so we know this replica supports the task
+	 */
+	if ((resp_bere = der_alloc())){
+
+		ber_int_t response = 1;
+
+		ber_printf(resp_bere, "{e}", response);
+		ber_flatten(resp_bere, &resp_bval);
+		slapi_pblock_set(pb, SLAPI_EXT_OP_RET_VALUE, resp_bval);
+		slapi_send_ldap_result(pb, LDAP_SUCCESS, NULL, NULL, 0, NULL);
+		/* resp_bere */
+		if (NULL != resp_bere)
+		{
+			ber_free(resp_bere, 1);
+		}
+		/* resp_bval */
+		if (NULL != resp_bval)
+		{
+			ber_bvfree(resp_bval);
+		}
+	}
+
+	return rc;
+}
+
 /*
  * This plugin entry point is a noop entry
  * point. It's used when registering extops that
