@@ -53,10 +53,10 @@
 extern int slapi_log_urp;
 
 static int urp_add_resolve_parententry (Slapi_PBlock *pb, char *sessionid, Slapi_Entry *entry, Slapi_Entry *parententry, CSN *opcsn);
-static int urp_annotate_dn (char *sessionid, Slapi_Entry *entry, CSN *opcsn, const char *optype, void *txn);
+static int urp_annotate_dn (char *sessionid, Slapi_Entry *entry, CSN *opcsn, const char *optype);
 static int urp_naming_conflict_removal (Slapi_PBlock *pb, char *sessionid, CSN *opcsn, const char *optype);
-static int mod_namingconflict_attr (const char *uniqueid, const Slapi_DN *entrysdn, const Slapi_DN *conflictsdn, CSN *opcsn, void *txn);
-static int del_replconflict_attr (Slapi_Entry *entry, CSN *opcsn, int opflags, void *txn);
+static int mod_namingconflict_attr (const char *uniqueid, const Slapi_DN *entrysdn, const Slapi_DN *conflictsdn, CSN *opcsn);
+static int del_replconflict_attr (Slapi_Entry *entry, CSN *opcsn, int opflags);
 static char *get_dn_plus_uniqueid(char *sessionid,const char *olddn,const char *uniqueid);
 static char *get_rdn_plus_uniqueid(char *sessionid,const char *olddn,const char *uniqueid);
 static int is_suffix_entry (Slapi_PBlock *pb, Slapi_Entry *entry, Slapi_DN **parenddn);
@@ -123,14 +123,12 @@ urp_add_operation( Slapi_PBlock *pb )
 	int op_result= 0;
 	int rc= 0; /* OK */
 	Slapi_DN *sdn = NULL;
-	void *txn = NULL;
 
 	if ( slapi_op_abandoned(pb) )
 	{
 		return rc;
 	}
 
-	slapi_pblock_get( pb, SLAPI_TXN, &txn );
 	slapi_pblock_get( pb, SLAPI_ADD_EXISTING_UNIQUEID_ENTRY, &existing_uniqueid_entry );
 	if (existing_uniqueid_entry!=NULL)
 	{
@@ -139,7 +137,17 @@ urp_add_operation( Slapi_PBlock *pb )
 		 * - It could be a replay of the same Add, or
 		 * - It could be a UUID generation collision, or
 		 */
-		op_result = LDAP_SUCCESS;
+		/* 
+		 * This operation won't be replayed.  That is, this CSN won't update
+		 * the max csn in RUV. The CSN is left uncommitted in RUV unless an
+		 * error is set to op_result.  Just to get rid of this CSN from RUV,
+		 * setting an error to op_result
+		 */
+		/* op_result = LDAP_SUCCESS; */
+		slapi_log_error(slapi_log_urp, sessionid,
+		          "urp_add (%s): an entry with this uniqueid already exists.\n",
+		          slapi_entry_get_dn_const(existing_uniqueid_entry));
+		op_result= LDAP_UNWILLING_TO_PERFORM;
 		slapi_pblock_set(pb, SLAPI_RESULT_CODE, &op_result);
 		rc= -1; /* Ignore this Operation */
 		PROFILE_POINT; /* Add Conflict; UniqueID Exists;  Ignore */
@@ -229,7 +237,7 @@ urp_add_operation( Slapi_PBlock *pb )
 	else if(r>0)
 	{
 		/* Existing entry is a loser */
-		if (!urp_annotate_dn(sessionid, existing_dn_entry, opcsn, "ADD", txn))
+		if (!urp_annotate_dn(sessionid, existing_dn_entry, opcsn, "ADD"))
 		{
 			op_result= LDAP_OPERATIONS_ERROR;
 			slapi_pblock_set(pb, SLAPI_RESULT_CODE, &op_result);
@@ -251,7 +259,17 @@ urp_add_operation( Slapi_PBlock *pb )
 		 * b) We've seen the Operation before.
 		 * Let's go with (b) and ignore the little bastard.
 		 */
-		op_result= LDAP_SUCCESS;
+		/* 
+		 * This operation won't be replayed.  That is, this CSN won't update
+		 * the max csn in RUV. The CSN is left uncommitted in RUV unless an
+		 * error is set to op_result.  Just to get rid of this CSN from RUV,
+		 * setting an error to op_result
+		 */
+		/* op_result = LDAP_SUCCESS; */
+		slapi_log_error(slapi_log_urp, sessionid,
+		"urp_add (%s): The CSN of the Operation and the Entry DN are the same.",
+		slapi_entry_get_dn_const(existing_dn_entry));
+		op_result= LDAP_UNWILLING_TO_PERFORM;
 		slapi_pblock_set(pb, SLAPI_RESULT_CODE, &op_result);
 		rc= -1; /* Ignore this Operation */
 		PROFILE_POINT; /* Add Conflict; Entry Exists; Same CSN */
@@ -288,14 +306,12 @@ urp_modrdn_operation( Slapi_PBlock *pb )
 	int op_result= 0;
     int rc= 0; /* OK */
 	int del_old_replconflict_attr = 0;
-	void *txn = NULL;
 
 	if ( slapi_op_abandoned(pb) )
 	{
 		return rc;
 	}
 
-	slapi_pblock_get (pb, SLAPI_TXN, &txn);
    	slapi_pblock_get (pb, SLAPI_MODRDN_TARGET_ENTRY, &target_entry);
 	if(target_entry==NULL)
 	{
@@ -315,7 +331,17 @@ urp_modrdn_operation( Slapi_PBlock *pb )
 		 * The Operation CSN is not newer than the DN CSN.
 		 * Either we're beaten by another ModRDN or we've applied the op.
 		 */
-		op_result= LDAP_SUCCESS;
+		/* op_result= LDAP_SUCCESS; */
+		/* 
+		 * This operation won't be replayed.  That is, this CSN won't update
+		 * the max csn in RUV. The CSN is left uncommitted in RUV unless an
+		 * error is set to op_result.  Just to get rid of this CSN from RUV,
+		 * setting an error to op_result
+		 */
+		slapi_log_error(slapi_log_urp, sessionid,
+		           "urp_modrdn (%s): operation CSN is newer than the DN CSN.\n",
+		           slapi_entry_get_dn_const(target_entry));
+		op_result= LDAP_UNWILLING_TO_PERFORM;
 		slapi_pblock_set(pb, SLAPI_RESULT_CODE, &op_result);
 		rc= -1; /* Ignore the modrdn */
 		PROFILE_POINT; /* ModRDN Conflict; Entry with Target DN Exists; OPCSN is not newer. */
@@ -324,10 +350,11 @@ urp_modrdn_operation( Slapi_PBlock *pb )
 
 	/* The DN CSN is older than the Operation CSN. Apply the operation */
 	target_sdn = slapi_entry_get_sdn_const (target_entry);
+	/* newrdn is no need to be case-ignored (get_rdn_plus_uniqueid) */
 	slapi_pblock_get(pb, SLAPI_MODRDN_NEWRDN, &newrdn);
 	slapi_pblock_get(pb, SLAPI_TARGET_UNIQUEID, &op_uniqueid);
-   	slapi_pblock_get(pb, SLAPI_MODRDN_PARENT_ENTRY, &parent_entry);
-   	slapi_pblock_get(pb, SLAPI_MODRDN_NEWPARENT_ENTRY, &new_parent_entry);
+	slapi_pblock_get(pb, SLAPI_MODRDN_PARENT_ENTRY, &parent_entry);
+	slapi_pblock_get(pb, SLAPI_MODRDN_NEWPARENT_ENTRY, &new_parent_entry);
 	slapi_pblock_get(pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &newsuperior);
 
 	if ( is_tombstone_entry (target_entry) )
@@ -364,26 +391,29 @@ urp_modrdn_operation( Slapi_PBlock *pb )
 		goto bailout;
 	}
 
-   	slapi_pblock_get(pb, SLAPI_MODRDN_EXISTING_ENTRY, &existing_entry);
-    if(existing_entry!=NULL) 
+	slapi_pblock_get(pb, SLAPI_MODRDN_EXISTING_ENTRY, &existing_entry);
+	if(existing_entry!=NULL) 
 	{
 	    /*
 	     * An entry with the target DN already exists.
-		 * The smaller dncsn wins. The loser changes its RDN to
-		 * uniqueid+baserdn, and adds operational attribute
-		 * ATTR_NSDS5_REPLCONFLIC
+	     * The smaller dncsn wins. The loser changes its RDN to
+	     * uniqueid+baserdn, and adds operational attribute
+	     * ATTR_NSDS5_REPLCONFLIC
 	     */
 
 		existing_uniqueid = slapi_entry_get_uniqueid (existing_entry);
 		existing_sdn = slapi_entry_get_sdn_const ( existing_entry);
 
 		/*
-		 * Dismiss the operation if the existing entry is the same as the target one.
+		 * It used to dismiss the operation if the existing entry is 
+		 * the same as the target one.
+		 * But renaming the RDN with the one which only cases are different,
+		 * cn=ABC --> cn=Abc, this case matches.  We should go forward the op.
 		 */
 		if (strcmp(op_uniqueid, existing_uniqueid) == 0) {
 			op_result= LDAP_SUCCESS;
 			slapi_pblock_set(pb, SLAPI_RESULT_CODE, &op_result);
-			rc = -1; /* Ignore the op */
+			rc = 0; /* Don't ignore the op */
 			PROFILE_POINT; /* ModRDN Replay */
 			goto bailout;
 		}
@@ -421,7 +451,7 @@ urp_modrdn_operation( Slapi_PBlock *pb )
 								  Unique ID already in RDN - Change to Lost and Found entry */
 				goto bailout;
 			}
-			mod_namingconflict_attr (op_uniqueid, target_sdn, existing_sdn, opcsn, txn);
+			mod_namingconflict_attr (op_uniqueid, target_sdn, existing_sdn, opcsn);
 			slapi_pblock_set(pb, SLAPI_MODRDN_NEWRDN, newrdn_with_uniqueid); 
 			slapi_log_error(slapi_log_urp, sessionid,
 					"Naming conflict MODRDN. Rename target entry to %s\n",
@@ -436,7 +466,7 @@ urp_modrdn_operation( Slapi_PBlock *pb )
 		{
 			/* The existing entry is a loser */
 
-			int resolve = urp_annotate_dn (sessionid, existing_entry, opcsn, "MODRDN", txn);
+			int resolve = urp_annotate_dn (sessionid, existing_entry, opcsn, "MODRDN");
 			if(!resolve)
 			{
 				op_result= LDAP_OPERATIONS_ERROR;
@@ -547,7 +577,7 @@ urp_modrdn_operation( Slapi_PBlock *pb )
 bailout:
 	if ( del_old_replconflict_attr && rc == 0 )
 	{
-		del_replconflict_attr (target_entry, opcsn, 0, txn);
+		del_replconflict_attr (target_entry, opcsn, 0);
 	}
 	if ( parentdn )
 		slapi_sdn_free(&parentdn);
@@ -565,14 +595,12 @@ urp_delete_operation( Slapi_PBlock *pb )
 	char sessionid[REPL_SESSION_ID_SIZE];
 	int op_result= 0;
     int rc= 0; /* OK */
-	void *txn = NULL;
 
 	if ( slapi_op_abandoned(pb) )
 	{
 		return rc;
 	}
 
-	slapi_pblock_get(pb, SLAPI_TXN, &txn);
    	slapi_pblock_get(pb, SLAPI_DELETE_EXISTING_ENTRY, &deleteentry);
 
 	if(deleteentry==NULL) /* uniqueid can't be found */
@@ -598,14 +626,14 @@ urp_delete_operation( Slapi_PBlock *pb )
 		if(!slapi_entry_has_children(deleteentry))
 		{
 			/* Remove possible conflict attributes */
-			del_replconflict_attr (deleteentry, opcsn, 0, txn);
+			del_replconflict_attr (deleteentry, opcsn, 0);
 			rc= 0; /* OK, to delete the entry */
 			PROFILE_POINT; /* Delete Operation; OK. */
 		}
 		else
 		{
 			/* Turn this entry into a glue_absent_parent entry */
-			entry_to_glue(sessionid, deleteentry, REASON_RESURRECT_ENTRY, opcsn, txn);
+			entry_to_glue(sessionid, deleteentry, REASON_RESURRECT_ENTRY, opcsn);
 
 			/* Turn the Delete into a No-Op */
 			op_result= LDAP_SUCCESS;
@@ -718,7 +746,7 @@ urp_post_delete_operation( Slapi_PBlock *pb )
 }
 
 int
-urp_fixup_add_entry (Slapi_Entry *e, const char *target_uniqueid, const char *parentuniqueid, CSN *opcsn, int opflags, void *txn)
+urp_fixup_add_entry (Slapi_Entry *e, const char *target_uniqueid, const char *parentuniqueid, CSN *opcsn, int opflags)
 {
 	Slapi_PBlock *newpb;
 	Slapi_Operation *op;
@@ -749,7 +777,6 @@ urp_fixup_add_entry (Slapi_Entry *e, const char *target_uniqueid, const char *pa
 	slapi_pblock_get ( newpb, SLAPI_OPERATION, &op );
 	operation_set_csn ( op, opcsn );
 
-	slapi_pblock_set ( newpb, SLAPI_TXN, txn );
 	slapi_add_internal_pb ( newpb );
 	slapi_pblock_get ( newpb, SLAPI_PLUGIN_INTOP_RESULT, &op_result );
 	slapi_pblock_destroy ( newpb );
@@ -758,7 +785,7 @@ urp_fixup_add_entry (Slapi_Entry *e, const char *target_uniqueid, const char *pa
 }
 
 int
-urp_fixup_rename_entry (Slapi_Entry *entry, const char *newrdn, int opflags, void *txn)
+urp_fixup_rename_entry (Slapi_Entry *entry, const char *newrdn, int opflags)
 {
 	Slapi_PBlock *newpb;
     Slapi_Operation *op;
@@ -787,7 +814,6 @@ urp_fixup_rename_entry (Slapi_Entry *entry, const char *newrdn, int opflags, voi
     slapi_pblock_get (newpb, SLAPI_OPERATION, &op);
     operation_set_csn (op, opcsn);
 
-	slapi_pblock_set(newpb, SLAPI_TXN, txn);
 	slapi_modrdn_internal_pb(newpb); 
     slapi_pblock_get(newpb, SLAPI_PLUGIN_INTOP_RESULT, &op_result);
 
@@ -796,7 +822,7 @@ urp_fixup_rename_entry (Slapi_Entry *entry, const char *newrdn, int opflags, voi
 }
 
 int
-urp_fixup_delete_entry (const char *uniqueid, const char *dn, CSN *opcsn, int opflags, void *txn)
+urp_fixup_delete_entry (const char *uniqueid, const char *dn, CSN *opcsn, int opflags)
 {
 	Slapi_PBlock *newpb;
 	Slapi_Operation *op;
@@ -818,7 +844,6 @@ urp_fixup_delete_entry (const char *uniqueid, const char *dn, CSN *opcsn, int op
 	slapi_pblock_get ( newpb, SLAPI_OPERATION, &op );
 	operation_set_csn ( op, opcsn );
 
-	slapi_pblock_set ( newpb, SLAPI_TXN, txn );
 	slapi_delete_internal_pb ( newpb );
 	slapi_pblock_get ( newpb, SLAPI_PLUGIN_INTOP_RESULT, &op_result );
 	slapi_pblock_destroy ( newpb );
@@ -827,7 +852,7 @@ urp_fixup_delete_entry (const char *uniqueid, const char *dn, CSN *opcsn, int op
 }
 
 int
-urp_fixup_modify_entry (const char *uniqueid, const Slapi_DN *sdn, CSN *opcsn, Slapi_Mods *smods, int opflags, void *txn)
+urp_fixup_modify_entry (const char *uniqueid, const Slapi_DN *sdn, CSN *opcsn, Slapi_Mods *smods, int opflags)
 {
 	Slapi_PBlock *newpb;
 	Slapi_Operation *op;
@@ -848,7 +873,6 @@ urp_fixup_modify_entry (const char *uniqueid, const Slapi_DN *sdn, CSN *opcsn, S
 	slapi_pblock_get (newpb, SLAPI_OPERATION, &op);
 	operation_set_csn (op, opcsn);
 
-	slapi_pblock_set (newpb, SLAPI_TXN, txn);
 	/* do modify */
 	slapi_modify_internal_pb (newpb);
 	slapi_pblock_get (newpb, SLAPI_PLUGIN_INTOP_RESULT, &op_result);
@@ -972,7 +996,7 @@ bailout:
  * a new entry (the operation entry) see urp_add_operation.
  */
 static int
-urp_annotate_dn (char *sessionid, Slapi_Entry *entry, CSN *opcsn, const char *optype, void *txn)
+urp_annotate_dn (char *sessionid, Slapi_Entry *entry, CSN *opcsn, const char *optype)
 {
 	int rc = 0; /* Fail */
 	int op_result;
@@ -980,7 +1004,6 @@ urp_annotate_dn (char *sessionid, Slapi_Entry *entry, CSN *opcsn, const char *op
 	const char *uniqueid;
 	const Slapi_DN *basesdn;
 	const char *basedn;
-	char ebuf[BUFSIZ];
 
 	uniqueid = slapi_entry_get_uniqueid (entry);
 	basesdn = slapi_entry_get_sdn_const (entry);
@@ -988,14 +1011,14 @@ urp_annotate_dn (char *sessionid, Slapi_Entry *entry, CSN *opcsn, const char *op
 	newrdn = get_rdn_plus_uniqueid ( sessionid, basedn, uniqueid );
 	if(newrdn!=NULL)
 	{
-		mod_namingconflict_attr (uniqueid, basesdn, basesdn, opcsn, txn);
-		op_result = urp_fixup_rename_entry ( entry, newrdn, 0, txn );
+		mod_namingconflict_attr (uniqueid, basesdn, basesdn, opcsn);
+		op_result = urp_fixup_rename_entry ( entry, newrdn, 0 );
 		switch(op_result)
 		{
 		case LDAP_SUCCESS:
 			slapi_log_error(slapi_log_urp, sessionid,
 				"Naming conflict %s. Renamed existing entry to %s\n",
-				optype, escape_string (newrdn, ebuf));
+				optype, newrdn);
 			rc = 1;
 			break;
 		case LDAP_NO_SUCH_OBJECT:
@@ -1017,7 +1040,7 @@ urp_annotate_dn (char *sessionid, Slapi_Entry *entry, CSN *opcsn, const char *op
 			 */
 			slapi_log_error( SLAPI_LOG_FATAL, sessionid,
 				"Entry %s exists in cache but not in DB\n",
-				escape_string (basedn, ebuf) );
+				basedn );
 			rc = LDAP_NO_SUCH_OBJECT;
 			break;
 		default:
@@ -1051,9 +1074,7 @@ urp_get_min_naming_conflict_entry ( Slapi_PBlock *pb, char *sessionid, CSN *opcs
 	int i = 0;
 	int min_i = -1;
 	int op_result = LDAP_SUCCESS;
-	void *txn = NULL;
 
-	slapi_pblock_get (pb, SLAPI_TXN, &txn);
 	slapi_pblock_get (pb, SLAPI_URP_NAMING_COLLISION_DN, &basedn);
 	if (NULL == basedn || strncmp (basedn, SLAPI_ATTR_UNIQUEID, strlen(SLAPI_ATTR_UNIQUEID)) == 0)
 		return NULL;
@@ -1080,7 +1101,6 @@ urp_get_min_naming_conflict_entry ( Slapi_PBlock *pb, char *sessionid, CSN *opcs
 								 NULL, /* UniqueID */
 								 repl_get_plugin_identity(PLUGIN_MULTIMASTER_REPLICATION),
 								 0);
-	slapi_pblock_set(newpb, SLAPI_TXN, txn);
 	slapi_search_internal_pb(newpb);
 	slapi_pblock_get(newpb, SLAPI_PLUGIN_INTOP_RESULT, &op_result);
 	slapi_pblock_get(newpb, SLAPI_PLUGIN_INTOP_SEARCH_ENTRIES, &entries);
@@ -1146,9 +1166,6 @@ urp_naming_conflict_removal ( Slapi_PBlock *pb, char *sessionid, CSN *opcsn, con
 	Slapi_RDN *oldrdn, *newrdn;
 	const char *oldrdnstr, *newrdnstr;
 	int op_result;
-	void *txn = NULL;
-
-	slapi_pblock_get (pb, SLAPI_TXN, &txn);
 
 	/*
 	 * Backend op has set SLAPI_URP_NAMING_COLLISION_DN to the basedn.
@@ -1174,7 +1191,7 @@ urp_naming_conflict_removal ( Slapi_PBlock *pb, char *sessionid, CSN *opcsn, con
 	 * is done after DB lock was released. The backend modrdn
 	 * will acquire the DB lock if it sees this flag.
 	 */
-	op_result = urp_fixup_rename_entry (min_naming_conflict_entry, newrdnstr, OP_FLAG_ACTION_INVOKE_FOR_REPLOP, txn);
+	op_result = urp_fixup_rename_entry (min_naming_conflict_entry, newrdnstr, OP_FLAG_ACTION_INVOKE_FOR_REPLOP);
 	if ( op_result != LDAP_SUCCESS )
 	{
 	    slapi_log_error (slapi_log_urp, sessionid,
@@ -1189,7 +1206,7 @@ urp_naming_conflict_removal ( Slapi_PBlock *pb, char *sessionid, CSN *opcsn, con
 	 * A fixup op will not invoke urp_modrdn_operation(). Even it does,
 	 * urp_modrdn_operation() will do nothing because of the same CSN.
 	 */
-	op_result = del_replconflict_attr (min_naming_conflict_entry, opcsn, OP_FLAG_ACTION_INVOKE_FOR_REPLOP, txn);
+	op_result = del_replconflict_attr (min_naming_conflict_entry, opcsn, OP_FLAG_ACTION_INVOKE_FOR_REPLOP);
 	if (op_result != LDAP_SUCCESS) {
 		slapi_log_error(SLAPI_LOG_REPL, sessionid,
 			"Failed to remove nsds5ReplConflict for %s, err=%d\n",
@@ -1300,7 +1317,7 @@ is_suffix_dn ( Slapi_PBlock *pb, const Slapi_DN *dn, Slapi_DN **parentdn )
 
 static int
 mod_namingconflict_attr (const char *uniqueid, const Slapi_DN *entrysdn,
-                         const Slapi_DN *conflictsdn, CSN *opcsn, void *txn)
+                         const Slapi_DN *conflictsdn, CSN *opcsn)
 {
 	Slapi_Mods smods;
 	char buf[BUFSIZ];
@@ -1323,13 +1340,13 @@ mod_namingconflict_attr (const char *uniqueid, const Slapi_DN *entrysdn,
 		 */
 		slapi_mods_add (&smods, LDAP_MOD_REPLACE, ATTR_NSDS5_REPLCONFLICT, strlen(buf), buf);
 	}
-	op_result = urp_fixup_modify_entry (uniqueid, entrysdn, opcsn, &smods, 0, txn);
+	op_result = urp_fixup_modify_entry (uniqueid, entrysdn, opcsn, &smods, 0);
 	slapi_mods_done (&smods);
 	return op_result;
 }
 
 static int
-del_replconflict_attr (Slapi_Entry *entry, CSN *opcsn, int opflags, void *txn)
+del_replconflict_attr (Slapi_Entry *entry, CSN *opcsn, int opflags)
 {
 	Slapi_Attr *attr;
 	int op_result = 0;
@@ -1344,7 +1361,7 @@ del_replconflict_attr (Slapi_Entry *entry, CSN *opcsn, int opflags, void *txn)
 		entrysdn = slapi_entry_get_sdn_const (entry);
 		slapi_mods_init (&smods, 2);
 		slapi_mods_add (&smods, LDAP_MOD_DELETE, ATTR_NSDS5_REPLCONFLICT, 0, NULL);
-		op_result = urp_fixup_modify_entry (uniqueid, entrysdn, opcsn, &smods, opflags, txn);
+		op_result = urp_fixup_modify_entry (uniqueid, entrysdn, opcsn, &smods, opflags);
 		slapi_mods_done (&smods);
 	}
 	return op_result;
