@@ -414,6 +414,11 @@ windows_inc_run(Private_Repl_Protocol *prp)
 						state2name(current_state));
 					protocol_sleep(prp, PR_INTERVAL_NO_TIMEOUT);
 				  }
+				else if (event_occurred(prp, EVENT_RUN_DIRSYNC)) /* periodic_dirsync */
+				  {
+					/* just ignore it and go to sleep */
+					protocol_sleep(prp, PR_INTERVAL_NO_TIMEOUT);
+				  }
 				else
 				  {
 					/* wait until window opens or an event occurs */
@@ -769,6 +774,7 @@ windows_inc_run(Private_Repl_Protocol *prp)
 	    /* richm: We at least need to let monitors know that the protocol has been
 	       shutdown - maybe they can figure out why */
 	    agmt_set_last_update_status(prp->agmt, 0, 0, "Protocol stopped");
+	    agmt_update_done(prp->agmt, 0);
 	    break;
 	  } 
 
@@ -903,6 +909,7 @@ windows_inc_run(Private_Repl_Protocol *prp)
 
 	agmt_set_last_update_end(prp->agmt, current_time());
 	agmt_set_update_in_progress(prp->agmt, PR_FALSE);
+	agmt_update_done(prp->agmt, 0);
 	/* If timed out, close the connection after released the replica */
 	windows_release_replica(prp);
 	if (rc == UPDATE_TIMEOUT) {
