@@ -86,6 +86,8 @@ time_t windows_private_get_sync_interval(const Repl_Agmt *ra);
 void windows_private_set_sync_interval(Repl_Agmt *ra, char *str);
 PRBool windows_private_get_one_way(const Repl_Agmt *ra);
 void windows_private_set_one_way(const Repl_Agmt *ra, PRBool value);
+int windows_private_get_move_action(const Repl_Agmt *ra);
+void windows_private_set_move_action(const Repl_Agmt *ra, int value);
 
 /* in windows_connection.c */
 ConnResult windows_conn_connect(Repl_Connection *conn);
@@ -141,11 +143,20 @@ int windows_check_user_password(Repl_Connection *conn, Slapi_DN *sdn, char *pass
 #define ONE_WAY_SYNC_FROM_AD 1
 #define ONE_WAY_SYNC_TO_AD 2
 
+/*
+ * Specifies what action for sync to take if it detects an AD entry has
+ * moved out of scope
+ */
+#define MOVE_DOES_NOTHING 0
+#define MOVE_DOES_UNSYNC 1
+#define MOVE_DOES_DELETE 2
+
 /* called for each replication agreement - so the winsync
    plugin can be agreement specific and store agreement
    specific data
 */
 void windows_plugin_init(Repl_Agmt *ra);
+void windows_plugin_cleanup_agmt(Repl_Agmt *ra);
 
 void winsync_plugin_call_dirsync_search_params_cb(const Repl_Agmt *ra, const char *agmt_dn, char **base, int *scope, char **filter, char ***attrs, LDAPControl ***serverctrls);
 /* called before searching for a single entry from AD - agmt_dn will be NULL */
@@ -179,6 +190,41 @@ void winsync_plugin_call_end_update_cb(const Repl_Agmt *ra, const Slapi_DN *ds_s
 void winsync_plugin_call_destroy_agmt_cb(const Repl_Agmt *ra,
                                          const Slapi_DN *ds_subtree,
                                          const Slapi_DN *ad_subtree);
+void winsync_plugin_call_post_ad_mod_user_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                             Slapi_Entry *ad_entry, Slapi_Entry *ds_entry,
+                                             Slapi_Mods *smods, int *result);
+void winsync_plugin_call_post_ad_mod_group_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                              Slapi_Entry *ad_entry, Slapi_Entry *ds_entry,
+                                              Slapi_Mods *smods, int *result);
+void winsync_plugin_call_post_ds_mod_user_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                             Slapi_Entry *ad_entry, Slapi_Entry *ds_entry,
+                                             Slapi_Mods *smods, int *result);
+void winsync_plugin_call_post_ds_mod_group_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                              Slapi_Entry *ad_entry, Slapi_Entry *ds_entry,
+                                              Slapi_Mods *smods, int *result);
+void winsync_plugin_call_post_ds_add_user_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                             Slapi_Entry *ad_entry, Slapi_Entry *ds_entry, int *result);
+void winsync_plugin_call_post_ds_add_group_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                              Slapi_Entry *ad_entry, Slapi_Entry *ds_entry, int *result);
+void winsync_plugin_call_pre_ad_add_user_cb(const Repl_Agmt *ra, Slapi_Entry *ad_entry,
+                                            Slapi_Entry *ds_entry);
+void winsync_plugin_call_pre_ad_add_group_cb(const Repl_Agmt *ra, Slapi_Entry *ad_entry,
+                                             Slapi_Entry *ds_entry);
+void winsync_plugin_call_post_ad_add_user_cb(const Repl_Agmt *ra, Slapi_Entry *ad_entry,
+                                             Slapi_Entry *ds_entry, int *result);
+void winsync_plugin_call_post_ad_add_group_cb(const Repl_Agmt *ra, Slapi_Entry *ad_entry,
+                                              Slapi_Entry *ds_entry, int *result);
+void winsync_plugin_call_post_ad_mod_user_mods_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                                  const Slapi_DN *local_dn,
+                                                  const Slapi_Entry *ds_entry,
+                                                  LDAPMod * const *origmods,
+                                                  Slapi_DN *remote_dn, LDAPMod **modstosend, int *result);
+void winsync_plugin_call_post_ad_mod_group_mods_cb(const Repl_Agmt *ra, const Slapi_Entry *rawentry,
+                                                   const Slapi_DN *local_dn,
+                                                   const Slapi_Entry *ds_entry,
+                                                   LDAPMod * const *origmods,
+                                                   Slapi_DN *remote_dn, LDAPMod **modstosend, int *result);
+
 /*
   Call stack for all places where windows_LDAPMessage2Entry is called:
 
