@@ -45,22 +45,22 @@ static Slapi_PluginDesc pdesc = {
         "USN", VENDOR, DS_PACKAGE_VERSION,
         "USN (Update Sequence Number) plugin" };
 
-static CSNGen *_usn_csngen = NULL;
-
 static void *_usn_identity = NULL;
 
 static int usn_preop_init(Slapi_PBlock *pb);
 static int usn_bepreop_init(Slapi_PBlock *pb);
+static int usn_betxnpreop_init(Slapi_PBlock *pb);
 static int usn_bepostop_init(Slapi_PBlock *pb);
 static int usn_rootdse_init();
 
 static int usn_preop_delete(Slapi_PBlock *pb);
-static int usn_bepreop_add(Slapi_PBlock *pb);
-static int usn_bepreop_delete(Slapi_PBlock *pb);
 static int usn_bepreop_modify(Slapi_PBlock *pb);
+static int usn_betxnpreop_add(Slapi_PBlock *pb);
+static int usn_betxnpreop_delete(Slapi_PBlock *pb);
 static int usn_bepostop(Slapi_PBlock *pb);
 static int usn_bepostop_delete (Slapi_PBlock *pb);
 static int usn_bepostop_modify (Slapi_PBlock *pb);
+
 static int usn_start(Slapi_PBlock *pb);
 static int usn_close(Slapi_PBlock *pb);
 static int usn_get_attr(Slapi_PBlock *pb, const char* type, void *value);
@@ -78,12 +78,21 @@ usn_init(Slapi_PBlock *pb)
 {
     int rc = 0;
     void *identity = NULL;
+    Slapi_Entry *plugin_entry = NULL;
+    int is_betxn = 0;
+    const char *plugintype;
 
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
                     "--> usn_init\n");
 
     slapi_pblock_get(pb, SLAPI_PLUGIN_IDENTITY, &identity);
 
+    if ((slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_ENTRY, &plugin_entry) == 0) &&
+        plugin_entry) {
+        is_betxn = slapi_entry_attr_get_bool(plugin_entry,
+                                             "nsslapd-pluginbetxn");
+    }
+
     /* slapi_register_plugin always returns SUCCESS (0) */
     if (slapi_pblock_set(pb, SLAPI_PLUGIN_VERSION,
                          SLAPI_PLUGIN_VERSION_01) != 0 ||
@@ -104,13 +113,28 @@ usn_init(Slapi_PBlock *pb)
         goto bail;
     }
 
-    rc = slapi_register_plugin("preoperation", 1 /* Enabled */,
+    /* usn_preop_init: plugintype is preoperation (not be/betxn) */
+    plugintype = "preoperation";
+    rc = slapi_register_plugin(plugintype, 1 /* Enabled */,
                                "usn_preop_init", usn_preop_init,
                                "USN preoperation plugin", NULL, identity);
-    rc |= slapi_register_plugin("bepreoperation", 1 /* Enabled */,
+
+    /* usn_bepreop_init: plugintype is bepreoperation (not betxn) */
+    plugintype = "bepreoperation";
+    rc |= slapi_register_plugin(plugintype, 1 /* Enabled */,
                                "usn_bepreop_init", usn_bepreop_init,
                                "USN bepreoperation plugin", NULL, identity);
-    rc |= slapi_register_plugin("bepostoperation", 1 /* Enabled */,
+
+    /* usn_bepreop_init: plugintype is betxnpreoperation */
+    plugintype = "betxnpreoperation";
+    rc |= slapi_register_plugin(plugintype, 1 /* Enabled */,
+                               "usn_betxnpreop_init", usn_betxnpreop_init,
+                               "USN betxnpreoperation plugin", NULL, identity);
+    plugintype = "bepostoperation";
+    if (is_betxn) {
+        plugintype = "betxnpostoperation";
+    }
+    rc |= slapi_register_plugin(plugintype, 1 /* Enabled */,
                                "usn_bepostop_init", usn_bepostop_init,
                                "USN bepostoperation plugin", NULL, identity);
     usn_set_identity(identity);
@@ -120,21 +144,14 @@ bail:
     return rc;
 }
 
+/* This ops must be preop not be/betxn */
 static int
 usn_preop_init(Slapi_PBlock *pb)
 {
     int rc = 0;
+    int predel = SLAPI_PLUGIN_PRE_DELETE_FN;
 
-    /* set up csn generator for tombstone */
-    _usn_csngen = csngen_new(USN_CSNGEN_ID, NULL);
-    if (NULL == _usn_csngen) {
-        slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
-                        "usn_preop_init: csngen_new failed\n");
-        rc = -1;
-    }
-
-    if (slapi_pblock_set(pb, SLAPI_PLUGIN_PRE_DELETE_FN,
-                                 (void *)usn_preop_delete) != 0) {
+    if (slapi_pblock_set(pb, predel, (void *)usn_preop_delete) != 0) {
         slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
                         "usn_preop_init: failed to register preop plugin\n");
         rc = -1;
@@ -147,17 +164,34 @@ static int
 usn_bepreop_init(Slapi_PBlock *pb)
 {
     int rc = 0;
+    int premod = SLAPI_PLUGIN_BE_PRE_MODIFY_FN;
+    int premdn = SLAPI_PLUGIN_BE_PRE_MODRDN_FN;
 
-    if (slapi_pblock_set(pb, SLAPI_PLUGIN_BE_PRE_ADD_FN,
-                                 (void *)usn_bepreop_add) != 0 ||
-        slapi_pblock_set(pb, SLAPI_PLUGIN_BE_PRE_DELETE_FN,
-                                 (void *)usn_bepreop_delete) != 0 ||
-        slapi_pblock_set(pb, SLAPI_PLUGIN_BE_PRE_MODIFY_FN,
-                                 (void *)usn_bepreop_modify) != 0 ||
-        slapi_pblock_set(pb, SLAPI_PLUGIN_BE_PRE_MODRDN_FN,
-                                 (void *)usn_bepreop_modify) != 0) {
+    /* usn_bepreop functions are called at BE_PRE_OP timing,
+     * not at BE_TXN_PREOP */
+    /* modify/modrdn updates mods which is evaluated before the 
+     * transaction start */
+    if ((slapi_pblock_set(pb, premod, (void *)usn_bepreop_modify) != 0) ||
+        (slapi_pblock_set(pb, premdn, (void *)usn_bepreop_modify) != 0)) {
         slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
-                    "usn_bepreop_init: failed to register bepreop plugin\n");
+                       "usn_bepreop_init: failed to register bepreop plugin\n");
+        rc = -1;
+    }
+
+    return rc;
+}
+
+static int
+usn_betxnpreop_init(Slapi_PBlock *pb)
+{
+    int rc = 0;
+    int preadd = SLAPI_PLUGIN_BE_TXN_PRE_ADD_FN;
+    int predel = SLAPI_PLUGIN_BE_TXN_PRE_DELETE_TOMBSTONE_FN;
+
+    if ((slapi_pblock_set(pb, preadd, (void *)usn_betxnpreop_add) != 0) ||
+        (slapi_pblock_set(pb, predel, (void *)usn_betxnpreop_delete) != 0)) { 
+        slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
+                 "usn_betxnpreop_init: failed to register betxnpreop plugin\n");
         rc = -1;
     }
 
@@ -168,17 +202,31 @@ static int
 usn_bepostop_init(Slapi_PBlock *pb)
 {
     int rc = 0;
+    Slapi_Entry *plugin_entry = NULL;
+    char *plugin_type = NULL;
+    int postadd = SLAPI_PLUGIN_BE_POST_ADD_FN;
+    int postmod = SLAPI_PLUGIN_BE_POST_MODIFY_FN;
+    int postmdn = SLAPI_PLUGIN_BE_POST_MODRDN_FN;
+    int postdel = SLAPI_PLUGIN_BE_POST_DELETE_FN;
 
-    if (slapi_pblock_set(pb, SLAPI_PLUGIN_BE_POST_ADD_FN,
-                                 (void *)usn_bepostop) != 0 ||
-        slapi_pblock_set(pb, SLAPI_PLUGIN_BE_POST_DELETE_FN,
-                                 (void *)usn_bepostop_delete) != 0 ||
-        slapi_pblock_set(pb, SLAPI_PLUGIN_BE_POST_MODIFY_FN,
-                                 (void *)usn_bepostop_modify) != 0 ||
-        slapi_pblock_set(pb, SLAPI_PLUGIN_BE_POST_MODRDN_FN,
-                                 (void *)usn_bepostop) != 0) {
+    if ((slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_ENTRY, &plugin_entry) == 0) &&
+        plugin_entry &&
+        (plugin_type = slapi_entry_attr_get_charptr(plugin_entry,
+                                                    "nsslapd-plugintype")) &&
+        plugin_type && strstr(plugin_type, "betxn")) {
+        postadd = SLAPI_PLUGIN_BE_TXN_POST_ADD_FN;
+        postmod = SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN;
+        postmdn = SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN;
+        postdel = SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN;
+    }
+    slapi_ch_free_string(&plugin_type);
+
+    if ((slapi_pblock_set(pb, postadd, (void *)usn_bepostop) != 0) ||
+        (slapi_pblock_set(pb, postdel, (void *)usn_bepostop_delete) != 0) ||
+        (slapi_pblock_set(pb, postmod, (void *)usn_bepostop_modify) != 0) ||
+        (slapi_pblock_set(pb, postmdn, (void *)usn_bepostop) != 0)) {
         slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
-                    "usn_bepostop_init: failed to register bepostop plugin\n");
+                     "usn_bepostop_init: failed to register bepostop plugin\n");
         rc = -1;
     }
 
@@ -245,15 +293,11 @@ bail:
     return rc;
 }
 
-/*
- * usn_close: release the csn generator used to convert an entry to tombstone
- */
 static int
 usn_close(Slapi_PBlock *pb)
 {
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM, "--> usn_close\n");
 
-    csngen_free(&_usn_csngen);
     g_plugin_started = 0;
 
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM, "<-- usn_close\n");
@@ -268,42 +312,22 @@ static int
 usn_preop_delete(Slapi_PBlock *pb)
 {
     int rc = 0;
-    CSN *csn = NULL;
-    CSN *orig_csn = NULL;
     Slapi_Operation *op = NULL;
 
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
                     "--> usn_preop_delete\n");
 
     slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-    orig_csn = operation_get_csn(op);
+    slapi_operation_set_replica_attr_handler(op, (void *)usn_get_attr);
 
-    if (NULL == orig_csn) {
-        /* 
-         * No other plugins hasn't set csn yet, so let's set USN's csn.
-         * If other plugin overrides csn and replica_attr_handler, that's fine.
-         */
-        rc = csngen_new_csn(_usn_csngen, &csn, PR_FALSE /* notify */);
-        if (CSN_SUCCESS != rc) {
-            slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
-                            "usn_preop_delete: csngen_new failed (%d)\n", rc);
-            csn_free(&csn);
-            goto bail;
-        }
-        operation_set_csn(op, csn);
-        slapi_operation_set_replica_attr_handler(op, (void *)usn_get_attr);
-    }
-bail:
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
                     "<-- usn_preop_delete\n");
 
     return rc;
 }
 
-#define KEEP_PREV_USN 1
-
 static void
-_usn_add_next_usn(Slapi_Entry *e, Slapi_Backend *be, int flags)
+_usn_add_next_usn(Slapi_Entry *e, Slapi_Backend *be)
 {
     struct berval usn_berval = {0};
     Slapi_Attr* attr = NULL;
@@ -327,14 +351,6 @@ _usn_add_next_usn(Slapi_Entry *e, Slapi_Backend *be, int flags)
         slapi_value_free(&usn_value);
     } else { /* ENTRYUSN exists; replace it */
         struct berval *new_bvals[2];
-        struct berval **prev_values = NULL;
-        if (KEEP_PREV_USN == flags) {
-            if (0 == slapi_attr_get_bervals_copy(attr, &prev_values)) {
-                slapi_entry_add_values(e,
-                                SLAPI_ATTR_ENTRYUSN_PREV, prev_values);
-                ber_bvecfree(prev_values);
-            }
-        }
         new_bvals[0] = &usn_berval;
         new_bvals[1] = NULL;
         slapi_entry_attr_replace(e, SLAPI_ATTR_ENTRYUSN, new_bvals);
@@ -384,17 +400,17 @@ _usn_mod_next_usn(LDAPMod ***mods, Slapi_Backend *be)
 }
 
 /*
- * usn_bepreop_add - add next USN to the entry to be added
+ * usn_betxnpreop_add - add next USN to the entry to be added
  */
 static int
-usn_bepreop_add(Slapi_PBlock *pb)
+usn_betxnpreop_add(Slapi_PBlock *pb)
 {
     Slapi_Entry *e = NULL;
     Slapi_Backend *be = NULL;
     int rc = LDAP_SUCCESS;
 
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
-                    "--> usn_bepreop_add\n");
+                    "--> usn_betxnpreop_add\n");
 
     /* add next USN to the entry; "be" contains the usn counter */
     slapi_pblock_get(pb, SLAPI_ADD_ENTRY, &e);
@@ -407,26 +423,26 @@ usn_bepreop_add(Slapi_PBlock *pb)
         rc = LDAP_PARAM_ERROR;    
         goto bail;
     }
-    _usn_add_next_usn(e, be, 0);
+    _usn_add_next_usn(e, be);
 bail:
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
-                    "<-- usn_bepreop_add\n");
+                    "<-- usn_betxnpreop_add\n");
     return rc;
 }
 
 /* 
- * usn_bepreop_delete -- add/replace next USN to the entry
+ * usn_betxnpreop_delete -- add/replace next USN to the entry
  *                       bepreop_delete is not called if the entry is tombstone
  */
 static int
-usn_bepreop_delete(Slapi_PBlock *pb)
+usn_betxnpreop_delete(Slapi_PBlock *pb)
 {
     Slapi_Entry *e = NULL;
     Slapi_Backend *be = NULL;
     int rc = LDAP_SUCCESS;
 
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
-                    "--> usn_bepreop_delete\n");
+                    "--> usn_betxnpreop_delete\n");
 
     /* add next USN to the entry; "be" contains the usn counter */
     slapi_pblock_get(pb, SLAPI_DELETE_BEPREOP_ENTRY, &e);
@@ -439,22 +455,17 @@ usn_bepreop_delete(Slapi_PBlock *pb)
         rc = LDAP_PARAM_ERROR;    
         goto bail;
     }
-    if (e->e_flags & SLAPI_ENTRY_FLAG_TOMBSTONE) {
-        Slapi_Operation *op = NULL;
-        slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-        slapi_operation_set_flag(op, OP_FLAG_TOMBSTONE_ENTRY);
-    } else {
-        _usn_add_next_usn(e, be, KEEP_PREV_USN);
-    }
+    _usn_add_next_usn(e, be);
 bail:
     slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
-                    "<-- usn_bepreop_delete\n");
+                    "<-- usn_betxnpreop_delete\n");
     return rc;
 }
 
 /*
  * usn_bepreop_modify - add/replace next USN to the mods; 
  *                      shared by modify and modrdn
+ * Note: bepreop should not return other than LDAP_SUCCESS.
  */
 static int
 usn_bepreop_modify (Slapi_PBlock *pb)
@@ -470,7 +481,8 @@ usn_bepreop_modify (Slapi_PBlock *pb)
     slapi_pblock_get(pb, SLAPI_MODIFY_MODS, &mods);
     slapi_pblock_get(pb, SLAPI_BACKEND, &be);
     if (NULL == be) {
-        rc = LDAP_PARAM_ERROR;    
+        slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
+                    "usn_bepreop_modify: no backend.\n");
         goto bail;
     }
     if (LDAP_SUCCESS == _usn_mod_next_usn(&mods, be)) {
@@ -535,7 +547,7 @@ usn_bepostop_modify (Slapi_PBlock *pb)
     for (i = 0; mods && mods[i]; i++) {
         if (0 == strcasecmp(mods[i]->mod_type, SLAPI_ATTR_ENTRYUSN)) {
             if (mods[i]->mod_op & LDAP_MOD_IGNORE) {
-    slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
+                slapi_log_error(SLAPI_LOG_TRACE, USN_PLUGIN_SUBSYSTEM,
                     "usn_bepostop_mod: MOD_IGNORE detected\n");
                 goto bail; /* conflict occurred.
                               skip incrementing the counter. */
@@ -574,15 +586,6 @@ usn_bepostop_delete (Slapi_PBlock *pb)
     /* if op is not successful, don't increment the counter */
     slapi_pblock_get(pb, SLAPI_RESULT_CODE, &rc);
     if (LDAP_SUCCESS != rc) {
-        Slapi_Entry *e = NULL;
-
-        slapi_pblock_get(pb, SLAPI_DELETE_BEPOSTOP_ENTRY, &e);
-        if (NULL == e) {
-            rc = LDAP_NO_SUCH_OBJECT;    
-            goto bail;
-        }
-        /* okay to return the rc from slapi_entry_delete_values */
-        rc = slapi_entry_delete_values(e, SLAPI_ATTR_ENTRYUSN_PREV, NULL);
         goto bail;
     }
 
@@ -715,5 +718,5 @@ usn_rootdse_search(Slapi_PBlock *pb, Slapi_Entry* e, Slapi_Entry* entryAfter,
 int
 usn_is_started()
 {
-	return g_plugin_started;
+    return g_plugin_started;
 }
