@@ -91,7 +91,6 @@ do_add( Slapi_PBlock *pb )
 	Slapi_Entry		*e = NULL;
 	int			err;
 	int			rc;
-	char			ebuf[ BUFSIZ ];
 	PRBool  searchsubentry=PR_TRUE;
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "do_add\n", 0, 0, 0 );
@@ -183,6 +182,7 @@ do_add( Slapi_PBlock *pb )
 
 		normtype = slapi_attr_syntax_normalize(type);
 		if ( !normtype || !*normtype ) {
+			char ebuf[ BUFSIZ ];
 			rc = LDAP_INVALID_SYNTAX;
 			PR_snprintf (ebuf, BUFSIZ, "invalid type '%s'", type);
 			op_shared_log_error_access (pb, "ADD", slapi_sdn_get_dn (slapi_entry_get_sdn_const(e)), ebuf);
@@ -202,7 +202,7 @@ do_add( Slapi_PBlock *pb )
 				slapi_log_access( LDAP_DEBUG_STATS, 
 					"conn=%" NSPRIu64 " op=%d ADD dn=\"%s\", add values for type %s failed\n",
 					pb->pb_conn->c_connid, operation->o_opid, 
-					escape_string( slapi_entry_get_dn_const(e), ebuf ), normtype );
+					slapi_entry_get_dn_const(e), normtype );
 				send_ldap_result( pb, rc, NULL, NULL, 0, NULL );
 
 				slapi_ch_free( (void**)&normtype );
@@ -445,7 +445,6 @@ static void op_shared_add (Slapi_PBlock *pb)
 	Slapi_Entry	*e, *pse;
 	Slapi_Backend *be = NULL;
 	int	err;
-	char ebuf[BUFSIZ];
 	int internal_op, repl_op, legacy_op, lastmod;
 	char *pwdtype = NULL;
 	Slapi_Value **unhashed_password_vals = NULL;
@@ -489,7 +488,7 @@ static void op_shared_add (Slapi_PBlock *pb)
 			slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d ADD dn=\"%s\"%s\n",
 							 pb->pb_conn->c_connid, 
 							 operation->o_opid,
-							 escape_string(slapi_entry_get_dn_const(e), ebuf),
+							 slapi_entry_get_dn_const(e),
 							 proxystr ? proxystr : "");
 		}
 		else
@@ -497,7 +496,7 @@ static void op_shared_add (Slapi_PBlock *pb)
 			slapi_log_access(LDAP_DEBUG_ARGS, "conn=%s op=%d ADD dn=\"%s\"\n",
 							 LOG_INTERNAL_OP_CON_ID,
 							 LOG_INTERNAL_OP_OP_ID,
-							 escape_string(slapi_entry_get_dn_const(e), ebuf));
+							 slapi_entry_get_dn_const(e));
 		}
 	}
 
@@ -580,7 +579,7 @@ static void op_shared_add (Slapi_PBlock *pb)
 		}
 
        /* look for multiple backend local credentials or replication local credentials */
-        for ( p = get_plugin_list(PLUGIN_LIST_REVER_PWD_STORAGE_SCHEME); p != NULL;
+        for ( p = get_plugin_list(PLUGIN_LIST_REVER_PWD_STORAGE_SCHEME); p != NULL && !repl_op;
             p = p->plg_next )
         {
             char *L_attr = NULL;
@@ -626,30 +625,31 @@ static void op_shared_add (Slapi_PBlock *pb)
 							   !slapdFrontendConfig->pw_policy.pw_must_change);
 	}
 
-	/* can get lastmod only after backend is selected */
-	slapi_pblock_get(pb, SLAPI_BE_LASTMOD, &lastmod);
-	if (!repl_op && lastmod)
+
+	if (!repl_op)
 	{
-		if (add_created_attrs(operation, e) != 0)
+		/* can get lastmod only after backend is selected */
+		slapi_pblock_get(pb, SLAPI_BE_LASTMOD, &lastmod);
+
+		if (lastmod && add_created_attrs(operation, e) != 0)
 		{
 			send_ldap_result(pb, LDAP_UNWILLING_TO_PERFORM, NULL,
 				"cannot insert computed attributes", 0, NULL);
 			goto done;
 		}
+		/* expand objectClass values to reflect the inheritance hierarchy */
+		slapi_schema_expand_objectclasses( e );
 	}
 
-	/* expand objectClass values to reflect the inheritance hierarchy */
-	slapi_schema_expand_objectclasses( e );
-
-
     /* uniqueid needs to be generated for entries added during legacy replication */
-    if (legacy_op)
-	if (add_uniqueid(e) != UID_SUCCESS)
-	{
-		send_ldap_result(pb, LDAP_UNWILLING_TO_PERFORM, NULL,
-			"cannot insert computed attributes", 0, NULL);
-		goto done;
-	}
+    if (legacy_op){
+    	if (add_uniqueid(e) != UID_SUCCESS)
+    	{
+    		send_ldap_result(pb, LDAP_UNWILLING_TO_PERFORM, NULL,
+    				"cannot insert computed attributes", 0, NULL);
+    		goto done;
+    	}
+    }
 
 	/*
 	 * call the pre-add plugins. if they succeed, call
@@ -665,6 +665,7 @@ static void op_shared_add (Slapi_PBlock *pb)
 		int	rc;
 		Slapi_Entry	*ec;
 		Slapi_DN *add_target_sdn = NULL;
+		Slapi_Entry *save_e = NULL;
 
 		slapi_pblock_set(pb, SLAPI_PLUGIN, be->be_database);
 		set_db_default_result_handlers(pb);
@@ -678,6 +679,8 @@ static void op_shared_add (Slapi_PBlock *pb)
 		if (be->be_add != NULL)
 		{
 			rc = (*be->be_add)(pb);
+			/* backend may change this if errors and not consumed */
+			slapi_pblock_get(pb, SLAPI_ADD_ENTRY, &save_e);
 			slapi_pblock_set(pb, SLAPI_ADD_ENTRY, ec);
 			if (rc == 0)
 			{
@@ -703,6 +706,10 @@ static void op_shared_add (Slapi_PBlock *pb)
 			}
 			else
 			{
+				/* restore e so we can free it below */
+				if (save_e) {
+					e = save_e;
+				}
 				if (rc == SLAPI_FAIL_DISKFULL)
 				{
 					operation_out_of_disk_space();
@@ -741,23 +748,51 @@ static int
 add_created_attrs(Operation *op, Slapi_Entry *e)
 {
 	char   buf[20];
+	char   *binddn = NULL;
 	struct berval	bv;
 	struct berval	*bvals[2];
 	time_t		curtime;
 	struct tm	ltm;
+	slapdFrontendConfig_t *slapdFrontendConfig = getFrontendConfig();
 
 	LDAPDebug(LDAP_DEBUG_TRACE, "add_created_attrs\n", 0, 0, 0);
 
 	bvals[0] = &bv;
 	bvals[1] = NULL;
 	
-	if (slapi_sdn_isempty(&op->o_sdn)) {
-		bv.bv_val = "";
-		bv.bv_len = strlen(bv.bv_val);
+	if(slapdFrontendConfig->plugin_track){
+		/* plugin bindDN tracking is enabled, grab the dn from thread local storage */
+		if(slapi_sdn_isempty(&op->o_sdn)){
+			bv.bv_val = "";
+			bv.bv_len = strlen(bv.bv_val);
+		} else {
+			bv.bv_val = (char*)slapi_sdn_get_dn(&op->o_sdn);
+			bv.bv_len = strlen(bv.bv_val);
+		}
+		slapi_entry_attr_replace(e, "internalCreatorsName", bvals);
+		slapi_entry_attr_replace(e, "internalModifiersName", bvals);
+
+		/* Grab the thread data(binddn) */
+		slapi_td_get_dn(&binddn);
+
+		if(binddn == NULL){
+			/* anonymous bind */
+			bv.bv_val = "";
+			bv.bv_len = strlen(bv.bv_val);
+		} else {
+			bv.bv_val = binddn;
+			bv.bv_len = strlen(bv.bv_val);
+		}
 	} else {
-		bv.bv_val = (char*)slapi_sdn_get_dn(&op->o_sdn);
-		bv.bv_len = strlen(bv.bv_val);
+		if (slapi_sdn_isempty(&op->o_sdn)) {
+			bv.bv_val = "";
+			bv.bv_len = strlen(bv.bv_val);
+		} else {
+			bv.bv_val = (char*)slapi_sdn_get_dn(&op->o_sdn);
+			bv.bv_len = strlen(bv.bv_val);
+		}
 	}
+
 	slapi_entry_attr_replace(e, "creatorsname", bvals);
 	slapi_entry_attr_replace(e, "modifiersname", bvals);
 
