@@ -404,6 +404,7 @@ int
 slapi_attr_value_find( const Slapi_Attr *a, const struct berval *v )
 {
 	struct ava	ava;
+	unsigned long a_flags;
 
 	if ( NULL == a ) {
 		return( -1 );
@@ -411,7 +412,12 @@ slapi_attr_value_find( const Slapi_Attr *a, const struct berval *v )
 
 	ava.ava_type = a->a_type;
 	ava.ava_value = *v;
-	ava.ava_private = NULL;
+	if (a->a_flags & SLAPI_ATTR_FLAG_NORMALIZED) {
+	    a_flags = a->a_flags;
+	    ava.ava_private = &a_flags;
+	} else {
+	    ava.ava_private = NULL;
+	}
 	return(plugin_call_syntax_filter_ava( a, LDAP_FILTER_EQUALITY, &ava ));
 }
 
@@ -545,21 +551,56 @@ slapi_attr_value_cmp( const Slapi_Attr *a, const struct berval *v1, const struct
 	}
 	else
 	{
-	    Slapi_Attr a2;
-	    struct ava ava;
-   		Slapi_Value *cvals[2];
-	    Slapi_Value tmpcval;
+		Slapi_Attr a2;
+		struct ava ava;
+		Slapi_Value *cvals[2];
+		Slapi_Value tmpcval;
 
-    	a2 = *a;
-	    cvals[0] = &tmpcval;
-   		cvals[0]->v_csnset = NULL;
-   		cvals[0]->bv = *v1;
-   		cvals[0]->v_flags = 0;
-   		cvals[1] = NULL;
-   		a2.a_present_values.va = cvals; /* JCM - PUKE */
-   		ava.ava_type = a->a_type;
-   		ava.ava_value = *v2;
-    	ava.ava_private = NULL;
+		a2 = *a;
+		cvals[0] = &tmpcval;
+		cvals[0]->v_csnset = NULL;
+		cvals[0]->bv = *v1;
+		cvals[0]->v_flags = 0;
+		cvals[1] = NULL;
+		a2.a_present_values.va = cvals; /* JCM - PUKE */
+		ava.ava_type = a->a_type;
+		ava.ava_value = *v2;
+		ava.ava_private = NULL;
+		retVal = plugin_call_syntax_filter_ava(&a2, LDAP_FILTER_EQUALITY, &ava);
+	}
+	return retVal;
+}
+
+int
+slapi_attr_value_cmp_ext(const Slapi_Attr *a, Slapi_Value *v1, Slapi_Value *v2)
+{
+	int retVal;
+	const struct berval *bv2 = slapi_value_get_berval(v2);
+
+	if ( a->a_flags & SLAPI_ATTR_FLAG_CMP_BITBYBIT )
+	{
+		const struct berval *bv1 = slapi_value_get_berval(v1);
+		return slapi_attr_value_cmp(a, bv1, bv2);
+	}
+	else
+	{
+		Slapi_Attr a2;
+		struct ava ava;
+		Slapi_Value *cvals[2];
+		unsigned long v2_flags = v2->v_flags;
+
+		a2 = *a;
+		cvals[0] = v1;
+		cvals[1] = NULL;
+		a2.a_present_values.va = cvals; /* JCM - PUKE */
+
+		ava.ava_type = a->a_type;
+		ava.ava_value = *bv2;
+		if (v2_flags) {
+			ava.ava_private = &v2_flags;
+		} else {
+			ava.ava_private = NULL;
+		}
 		retVal = plugin_call_syntax_filter_ava(&a2, LDAP_FILTER_EQUALITY, &ava);
 	}
 	return retVal;
@@ -805,7 +846,14 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
         for ( i = 0; vals[i] != NULL; ++i ) {
             if ( slapi_attr_value_find( a, slapi_value_get_berval(vals[i]) ) == 0 ) {
                 duplicate_index = i;
-                rc = LDAP_TYPE_OR_VALUE_EXISTS;
+                if (is_type_forbidden(a->a_type)) {
+                    /* If the attr is in the forbidden list
+                     * (e.g., unhashed password),
+                     * we don't return any useful info to the clients. */
+                    rc = LDAP_OTHER;
+                } else {
+                    rc = LDAP_TYPE_OR_VALUE_EXISTS;
+                }
                 break;
             }
         }
@@ -822,7 +870,6 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
      * LDAP_OPERATIONS_ERROR
      */
     else if ( duplicate_index >= 0 ) {
-        char avdbuf[BUFSIZ];
         char bvvalcopy[BUFSIZ];
         char *duplicate_string = "null or non-ASCII";
 
@@ -849,7 +896,7 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
                 "attribute type \"%s\" in entry \"%s\" failed: %s\n", 
                 duplicate_string,
                 a->a_type,
-                dn ? escape_string(dn,avdbuf) : "<null>", 
+                dn ? dn : "<null>", 
                 (was_present_null ? "duplicate new value" : "value exists"));
     }
     return( rc );
