@@ -294,7 +294,7 @@ dump_hash(Hashtable *ht)
             continue;
         }
         do {
-            PR_snprintf(ep_id, 16, "%u", ((struct backcommon *)e)->ep_id);
+            PR_snprintf(ep_id, 16, "%u-%u", ((struct backcommon *)e)->ep_id, ((struct backcommon *)e)->ep_refcnt);
             len = strlen(ep_id);
             if (ids_size < len + 1) {
                 LDAPDebug1Arg(LDAP_DEBUG_ANY, "%s\n", ep_ids);
@@ -857,7 +857,7 @@ entrycache_remove_int(struct cache *cache, struct backentry *e)
     const char *uuid;
 #endif
 
-    LOG("=> entrycache_remove_int (%s)\n", backentry_get_ndn(e), 0, 0);
+    LOG("=> entrycache_remove_int (%s) (%u) (%u)\n", backentry_get_ndn(e), e->ep_id, e->ep_refcnt);
     if (e->ep_state & ENTRY_STATE_NOTINCACHE)
     {
         return ret;
@@ -876,13 +876,22 @@ entrycache_remove_int(struct cache *cache, struct backentry *e)
     {
         LOG("remove %s from dn hash failed\n", ndn, 0, 0);
     }
-    if (remove_hash(cache->c_idtable, &(e->ep_id), sizeof(ID)))
+    /* if entry was added tentatively, it will be in the dntable
+       but not in the idtable - we cannot just remove it from
+       the idtable - in the case of modrdn, this will remove
+       the _real_ entry from the idtable, leading to a cache
+       imbalance
+    */
+    if (!(e->ep_state & ENTRY_STATE_CREATING))
     {
-       ret = 0;
-    }
-    else
-    {
-        LOG("remove %d from id hash failed\n", e->ep_id, 0, 0);
+        if (remove_hash(cache->c_idtable, &(e->ep_id), sizeof(ID)))
+        {
+            ret = 0;
+        }
+        else
+        {
+            LOG("remove %d from id hash failed\n", e->ep_id, 0, 0);
+        }
     }
 #ifdef UUIDCACHE_ON 
     uuid = slapi_entry_get_uniqueid(e->ep_entry);
@@ -907,6 +916,11 @@ entrycache_remove_int(struct cache *cache, struct backentry *e)
 
     /* mark for deletion (will be erased when refcount drops to zero) */
     e->ep_state |= ENTRY_STATE_DELETED;
+#if 0
+    if (slapi_is_loglevel_set(SLAPI_LOG_CACHE)) {
+        dump_hash(cache->c_idtable);
+    }
+#endif
     LOG("<= entrycache_remove_int: %d\n", ret, 0, 0);
     return ret;
 }
@@ -999,14 +1013,23 @@ static int entrycache_replace(struct cache *cache, struct backentry *olde,
      * cache tables, operation error 
      */
     if ( (olde->ep_state & ENTRY_STATE_NOTINCACHE) == 0 ) {
-
-        found = remove_hash(cache->c_dntable, (void *)oldndn, strlen(oldndn));
-        found &= remove_hash(cache->c_idtable, &(olde->ep_id), sizeof(ID));
+        int found_in_dn = remove_hash(cache->c_dntable, (void *)oldndn, strlen(oldndn));
+        int found_in_id = remove_hash(cache->c_idtable, &(olde->ep_id), sizeof(ID));
 #ifdef UUIDCACHE_ON
-        found &= remove_hash(cache->c_uuidtable, (void *)olduuid, strlen(olduuid));
+        int found_in_uuid = remove_hash(cache->c_uuidtable, (void *)olduuid, strlen(olduuid));
+#endif
+        found = found_in_dn && found_in_id;
+#ifdef UUIDCACHE_ON
+        found = found && found_in_uuid;
 #endif
         if (!found) {
-            LOG("entry cache replace: cache index tables out of sync\n", 0, 0, 0);
+#ifdef UUIDCACHE_ON
+            LOG("entry cache replace: cache index tables out of sync - found dn [%d] id [%d] uuid [%d]\n",
+                found_in_dn, found_in_id, found_in_uuid);
+#else
+            LOG("entry cache replace: cache index tables out of sync - found dn [%d] id [%d]\n",
+                found_in_dn, found_in_id, 0);
+#endif
             PR_Unlock(cache->c_mutex);
             return 1;
         }
@@ -1035,7 +1058,9 @@ static int entrycache_replace(struct cache *cache, struct backentry *olde,
     }
     if (!add_hash(cache->c_idtable, &(newe->ep_id), sizeof(ID), newe, NULL)) {
        LOG("entry cache replace: can't add id\n", 0, 0, 0);
-       remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn));
+       if(remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn)) == 0){
+    	   LOG("entry cache replace: failed to remove dn table\n", 0, 0, 0);
+       }
        PR_Unlock(cache->c_mutex);
        return 1;
     }
@@ -1043,14 +1068,18 @@ static int entrycache_replace(struct cache *cache, struct backentry *olde,
     if (newuuid && !add_hash(cache->c_uuidtable, (void *)newuuid, strlen(newuuid),
                        newe, NULL)) {
        LOG("entry cache replace: can't add uuid\n", 0, 0, 0);
-       remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn));
-       remove_hash(cache->c_idtable, &(newe->ep_id), sizeof(ID));
+       if(remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn)) == 0){
+    	   LOG("entry cache replace: failed to remove dn table(uuid cache)\n", 0, 0, 0);
+       }
+       if(remove_hash(cache->c_idtable, &(newe->ep_id), sizeof(ID)) == 0){
+    	   LOG("entry cache replace: failed to remove id table(uuid cache)\n", 0, 0, 0);
+       }
        PR_Unlock(cache->c_mutex);
        return 1;
     }
 #endif
     /* adjust cache meta info */
-    newe->ep_refcnt = 1;
+    newe->ep_refcnt++;
     newe->ep_size = cache_entry_size(newe);
     if (newe->ep_size > olde->ep_size) {
         slapi_counter_add(cache->c_cursize, newe->ep_size - olde->ep_size);
@@ -1334,7 +1363,9 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
                 PR_Unlock(cache->c_mutex);
                 return 0;
             }
-            remove_hash(cache->c_dntable, (void *)ndn, strlen(ndn));
+            if(remove_hash(cache->c_dntable, (void *)ndn, strlen(ndn)) == 0){
+            	LOG("entrycache_add_int: failed to remove dn table\n", 0, 0, 0);
+            }
             e->ep_state |= ENTRY_STATE_NOTINCACHE;
             PR_Unlock(cache->c_mutex);
             return -1;
@@ -1346,8 +1377,12 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
                    NULL)) {
                 LOG("entry %s already in uuid cache!\n", backentry_get_ndn(e),
                             0, 0);
-                remove_hash(cache->c_dntable, (void *)ndn, strlen(ndn));
-                remove_hash(cache->c_idtable, &(e->ep_id), sizeof(ID));
+                if(remove_hash(cache->c_dntable, (void *)ndn, strlen(ndn)) == 0){
+                	LOG("entrycache_add_int: failed to remove dn table(uuid cache)\n", 0, 0, 0);
+                }
+                if(remove_hash(cache->c_idtable, &(e->ep_id), sizeof(ID)) == 0){
+                	LOG("entrycache_add_int: failed to remove id table(uuid cache)\n", 0, 0, 0);
+                }
                 e->ep_state |= ENTRY_STATE_NOTINCACHE;
                 PR_Unlock(cache->c_mutex);
                 return -1;
@@ -1472,7 +1507,9 @@ int cache_lock_entry(struct cache *cache, struct backentry *e)
 void cache_unlock_entry(struct cache *cache, struct backentry *e)
 {
     LOG("=> cache_unlock_entry\n", 0, 0, 0);
-    PR_ExitMonitor(e->ep_mutexp);
+    if (PR_ExitMonitor(e->ep_mutexp)) {
+        LOG("=> cache_unlock_entry - monitor was not entered!!!\n", 0, 0, 0);
+    }
 }
 
 /* DN cache */
