@@ -412,7 +412,7 @@ index_addordel_entry(
         }
         result = index_addordel_string(be, SLAPI_ATTR_NSCP_ENTRYDN, slapi_sdn_get_ndn(&parent), e->ep_id, flags, txn);
         if ( result != 0 ) {
-            ldbm_nasty(errmsg, 1020, result);
+            ldbm_nasty(errmsg, 1021, result);
             return( result );
         }
         slapi_sdn_done(&parent);
@@ -423,6 +423,7 @@ index_addordel_entry(
              */ 
             result = entryrdn_index_entry(be, e, flags, txn);
             if ( result != 0 ) {
+                ldbm_nasty(errmsg, 1023, result);
                 return( result );
             }
             /* To maintain tombstonenumsubordinates,
@@ -433,7 +434,7 @@ index_addordel_entry(
                 result = index_addordel_values_sv(be, LDBM_PARENTID_STR, svals, NULL,
                                                   e->ep_id, flags, txn);
                 if ( result != 0 ) {
-                    ldbm_nasty(errmsg, 1020, result);
+                    ldbm_nasty(errmsg, 1022, result);
                     return( result );
                 }
             }
@@ -454,7 +455,9 @@ index_addordel_entry(
                     /* skip "entrydn" */
                     continue;
                 } else {
-                    slapi_values_set_flags(svals, SLAPI_ATTR_FLAG_NORMALIZED);
+                    /* entrydn is case-normalized */
+                    slapi_values_set_flags(svals,
+                                           SLAPI_ATTR_FLAG_NORMALIZED_CIS);
                 }
             }
             result = index_addordel_values_sv( be, type, svals, NULL,
@@ -481,6 +484,7 @@ index_addordel_entry(
         if (entryrdn_get_switch()) { /* subtree-rename: on */
             result = entryrdn_index_entry(be, e, flags, txn);
             if ( result != 0 ) {
+                ldbm_nasty(errmsg, 1031, result);
                 return( result );
             }
         }
@@ -550,8 +554,7 @@ index_add_mods(
         /* Get a list of all values specified in the operation.
          */
         if ( mods[i]->mod_bvalues != NULL ) {
-            valuearray_init_bervalarray(mods[i]->mod_bvalues,
-                                        &mods_valueArray);
+            valuearray_init_bervalarray(mods[i]->mod_bvalues, &mods_valueArray);
         }
 
         switch ( mods[i]->mod_op & ~LDAP_MOD_BVALUES ) {
@@ -609,42 +612,55 @@ index_add_mods(
             /* We need to first remove the old values from the 
              * index, if any. */
             if (deleted_valueArray) {
-                index_addordel_values_sv( be, mods[i]->mod_type,
-                                          deleted_valueArray, evals, id, 
-                                          flags, txn );
+                rc = index_addordel_values_sv( be, mods[i]->mod_type,
+                                               deleted_valueArray, evals, id, 
+                                               flags, txn );
+                if (rc) {
+                    ldbm_nasty(errmsg, 1041, rc);
+                    goto error;
+                }
             }
 
             /* Free valuearray */
             slapi_valueset_free(mod_vals);
+            mod_vals = NULL;
         case LDAP_MOD_ADD:
             if ( mods_valueArray == NULL ) {
                 rc = 0;
             } else {
                 /* Verify if the value is in newe.
                  * If it is in, we will add the attr value to the index file. */
-                slapi_entry_attr_find( newe->ep_entry, 
-                                       mods[i]->mod_type, &curr_attr );
+                curr_attr = NULL;
+                slapi_entry_attr_find(newe->ep_entry, 
+                                      mods[i]->mod_type, &curr_attr);
                 
-                for (j = 0; mods_valueArray[j] != NULL; j++) {
-                    /* mods_valueArray[j] is in curr_attr ==> return 0 */
-                    if (slapi_attr_value_find(curr_attr,
+                if (curr_attr) { /* found the type */
+                    for (j = 0; mods_valueArray[j] != NULL; j++) {
+                        /* mods_valueArray[j] is in curr_attr ==> return 0 */
+                        if (slapi_attr_value_find(curr_attr,
                                 slapi_value_get_berval(mods_valueArray[j]))) {
-                        /* The value is NOT in newe, remove it. */
-                        Slapi_Value *rval = valuearray_remove_value(curr_attr,
-                                                            mods_valueArray,
-                                                            mods_valueArray[j]);
-                        slapi_value_free( &rval );
-                        /* indicates there was some conflict */
-                        mods[i]->mod_op |= LDAP_MOD_IGNORE;
+                            /* The value is NOT in newe, remove it. */
+                            Slapi_Value *rval;
+                            rval = valuearray_remove_value(curr_attr,
+                                                           mods_valueArray,
+                                                           mods_valueArray[j]);
+                            slapi_value_free( &rval );
+                            /* indicates there was some conflict */
+                            mods[i]->mod_op |= LDAP_MOD_IGNORE;
+                        }
+                    }
+                    if (mods_valueArray) {
+                        rc = index_addordel_values_sv( be,
+                                                   mods[i]->mod_type, 
+                                                   mods_valueArray, NULL,
+                                                   id, BE_INDEX_ADD, txn );
+                        if (rc) {
+                            ldbm_nasty(errmsg, 1042, rc);
+                            goto error;
+                        }
+                    } else {
+                        rc = 0;
                     }
-                }
-                if (mods_valueArray) {
-                    rc = index_addordel_values_sv( be,
-                                            mods[i]->mod_type, 
-                                            mods_valueArray, NULL,
-                                            id, BE_INDEX_ADD, txn );
-                } else {
-                    rc = 0;
                 }
             }
             break;
@@ -699,12 +715,17 @@ index_add_mods(
 
                 /* Update the index, if necessary */
                 if (deleted_valueArray) {
-                    index_addordel_values_sv( be, mods[i]->mod_type,
-                                              deleted_valueArray, evals, id, 
-                                              flags, txn );
+                    rc = index_addordel_values_sv( be, mods[i]->mod_type,
+                                                   deleted_valueArray, evals, id, 
+                                                   flags, txn );
+                    if (rc) {
+                        ldbm_nasty(errmsg, 1043, rc);
+                        goto error;
+                    }
                 }
 
                 slapi_valueset_free(mod_vals);
+                mod_vals = NULL;
             } else {
 
                 /* determine if the presence key should be
@@ -724,7 +745,9 @@ index_add_mods(
                      * BE_INDEX_EQUALITY flag so the equality index is
                      * removed.
                      */
-                    slapi_entry_attr_find( olde->ep_entry, mods[i]->mod_type, &curr_attr );
+                    curr_attr = NULL;
+                    slapi_entry_attr_find(olde->ep_entry,
+                                          mods[i]->mod_type, &curr_attr);
                     if (curr_attr) {
                         int found = 0;
                         for (j = 0; mods_valueArray[j] != NULL; j++ ) {
@@ -748,15 +771,25 @@ index_add_mods(
                 rc = index_addordel_values_sv( be, basetype,
                                                mods_valueArray,
                                                evals, id, flags, txn );
+                if (rc) {
+                    ldbm_nasty(errmsg, 1044, rc);
+                    goto error;
+                }
             }
             rc = 0;
             break;
         } /* switch ( mods[i]->mod_op & ~LDAP_MOD_BVALUES ) */
 
+error:
         /* free memory */
         slapi_ch_free((void **)&tmp);
+        tmp = NULL;
         valuearray_free(&mods_valueArray);
+        mods_valueArray = NULL;
         slapi_valueset_free(all_vals);
+        all_vals = NULL;
+        slapi_valueset_free(mod_vals);
+        mod_vals = NULL;
 
         if ( rc != 0 ) {
             ldbm_nasty(errmsg, 1040, rc);
@@ -991,6 +1024,9 @@ index_read_ext_allids(
 	  idl = idl_fetch_ext( be, db, &key, db_txn, ai, err, allidslimit );
 	  if(*err == DB_LOCK_DEADLOCK) {
 	    ldbm_nasty("index read retrying transaction", 1045, *err);
+#ifdef FIX_TXN_DEADLOCKS
+#error can only retry here if txn == NULL - otherwise, have to abort and retry txn
+#endif
 	    continue;
 	  } else {
 	    break;
@@ -1119,6 +1155,9 @@ retry:
 			cursor->c_close(cursor);
 			cursor = NULL;
 			key->data = saved_key;
+#ifdef FIX_TXN_DEADLOCKS
+#error if txn != NULL, have to abort and retry the transaction, not just the cursor
+#endif
 			goto retry;
 		} else
 		{
@@ -1141,6 +1180,9 @@ retry:
 		cursor->c_close(cursor);
 		cursor = NULL;
 		key->data = saved_key;
+#ifdef FIX_TXN_DEADLOCKS
+#error if txn != NULL, have to abort and retry the transaction, not just the cursor
+#endif
 		goto retry;
 	}
 error:
@@ -1488,6 +1530,9 @@ index_range_read_ext(
           tmp = idl_fetch_ext( be, db, &cur_key, NULL, ai, err, allidslimit );
           if(*err == DB_LOCK_DEADLOCK) {
             ldbm_nasty("index_range_read retrying transaction", 1090, *err);
+#ifdef FIX_TXN_DEADLOCKS
+#error if txn != NULL, have to abort and retry the transaction, not just the fetch
+#endif
             continue;
           } else {
             break;
@@ -1496,14 +1541,23 @@ index_range_read_ext(
         if(retry_count == IDL_FETCH_RETRY_COUNT) {
           ldbm_nasty("index_range_read retry count exceeded",1095,*err);
         }
-        tmp2 = idl_union( be, idl, tmp );
-        idl_free( idl );
-        idl_free( tmp );
-        idl = tmp2;
-        if (ALLIDS(idl)) {
-            LDAPDebug(LDAP_DEBUG_TRACE, "index_range_read hit an allids value\n",
-                      0, 0, 0);
-            break;
+        if (!tmp) {
+            if (slapi_is_loglevel_set(LDAP_DEBUG_TRACE)) {
+                char encbuf[BUFSIZ];
+                LDAPDebug2Args(LDAP_DEBUG_TRACE,
+                               "index_range_read_ext: cur_key=%s(%li bytes) was deleted - skipping\n",
+                               encoded(&cur_key, encbuf), (long)cur_key.dsize);
+            }
+        } else {
+            tmp2 = idl_union( be, idl, tmp );
+            idl_free( idl );
+            idl_free( tmp );
+            idl = tmp2;
+            if (ALLIDS(idl)) {
+                LDAPDebug(LDAP_DEBUG_TRACE, "index_range_read hit an allids value\n",
+                          0, 0, 0);
+                break;
+            }
         }
         if (DBT_EQ (&cur_key, &upperkey)) { /* this is the last key */
             break;
@@ -1627,7 +1681,7 @@ addordel_values(
 
 		if ( rc != 0)
 		{
-                        ldbm_nasty(errmsg, 1090, rc);
+                        ldbm_nasty(errmsg, 1096, rc);
 		}
 		index_free_prefix (prefix);
 		if (NULL != key.dptr && prefix != key.dptr)
