@@ -84,6 +84,7 @@ ldbm_back_add( Slapi_PBlock *pb )
 	struct backentry *tombstoneentry = NULL;
 	struct backentry *addingentry = NULL;
 	struct backentry *parententry = NULL;
+	struct backentry *originalentry = NULL;
 	ID pid;
 	int	isroot;
 	char *errbuf= NULL;
@@ -141,9 +142,14 @@ ldbm_back_add( Slapi_PBlock *pb )
 	slapi_entry_delete_values( e, numsubordinates, NULL );
 
 	dblayer_txn_init(li,&txn);
-	/* the calls to get_copy_of_entry require the parent txn if any
+	/* the calls to perform searches require the parent txn if any
 	   so set txn to the parent_txn until we begin the child transaction */
-	txn.back_txn_txn = parent_txn;
+	if (parent_txn) {
+		txn.back_txn_txn = parent_txn;
+	} else {
+		parent_txn = txn.back_txn_txn;
+		slapi_pblock_set( pb, SLAPI_TXN, parent_txn );
+	}
 
 	/* The dblock serializes writes to the database,
 	 * which reduces deadlocking in the db code,
@@ -207,14 +213,19 @@ ldbm_back_add( Slapi_PBlock *pb )
 				               "ldbm_back_add: Null target dn\n");
 				goto error_return;
 			}
-			dn = slapi_sdn_get_dn(sdn);
-			ldap_result_code = slapi_dn_syntax_check(pb, dn, 1);
-			if (ldap_result_code)
-			{
-				ldap_result_code = LDAP_INVALID_DN_SYNTAX;
-				slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
-				goto error_return;
+
+			/* not need to check the dn syntax as this is a replicated op */
+			if(!is_replicated_operation){
+				dn = slapi_sdn_get_dn(sdn);
+				ldap_result_code = slapi_dn_syntax_check(pb, dn, 1);
+				if (ldap_result_code)
+				{
+					ldap_result_code = LDAP_INVALID_DN_SYNTAX;
+					slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
+					goto error_return;
+				}
 			}
+
 			slapi_sdn_get_backend_parent(sdn, &parentsdn, pb->pb_backend);
 			/* Check if an entry with the intended DN already exists. */
 			done_with_pblock_entry(pb,SLAPI_ADD_EXISTING_DN_ENTRY); /* Could be through this multiple times */
@@ -222,11 +233,11 @@ ldbm_back_add( Slapi_PBlock *pb )
 			addr.udn = NULL;
 			addr.uniqueid = NULL;
 			ldap_result_code= get_copy_of_entry(pb, &addr, &txn, SLAPI_ADD_EXISTING_DN_ENTRY, !is_replicated_operation);
-            if(ldap_result_code==LDAP_OPERATIONS_ERROR ||
-               ldap_result_code==LDAP_INVALID_DN_SYNTAX)
-            {
-                goto error_return;
-            }
+			if(ldap_result_code==LDAP_OPERATIONS_ERROR ||
+			   ldap_result_code==LDAP_INVALID_DN_SYNTAX)
+			{
+			    goto error_return;
+			}
 		}
 		/* if we can find the parent by dn or uniqueid, and the operation has requested the parent
 		   then get it */
@@ -246,11 +257,25 @@ ldbm_back_add( Slapi_PBlock *pb )
 		rc= plugin_call_plugins(pb, SLAPI_PLUGIN_BE_PRE_ADD_FN);
 		if(rc==-1)
 		{
+			int opreturn = 0;
 			/* 
 			 * Plugin indicated some kind of failure,
 			 * or that this Operation became a No-Op.
 			 */
-			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			if (!ldap_result_code) {
+				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
+			if (!ldap_result_code) {
+				LDAPDebug0Args(LDAP_DEBUG_ANY,
+					       "ldbm_back_add: SLAPI_PLUGIN_BE_PRE_ADD_FN returned error but did not set SLAPI_RESULT_CODE\n");
+				ldap_result_code = LDAP_OPERATIONS_ERROR;
+			}
+			slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+			if (!opreturn) {
+				/* make sure opreturn is set for the postop plugins */
+				slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &rc);
+			}
+
 			goto error_return;
 		}
 		/*
@@ -331,22 +356,24 @@ ldbm_back_add( Slapi_PBlock *pb )
 		}
 	}
 
+	/* no need to check the schema as this is a replication add */
+	if(!is_replicated_operation){
+		if ((operation_is_flag_set(operation,OP_FLAG_ACTION_SCHEMA_CHECK)) && slapi_entry_schema_check(pb, e) != 0)
+		{
+			LDAPDebug(LDAP_DEBUG_TRACE, "entry failed schema check\n", 0, 0, 0);
+			ldap_result_code = LDAP_OBJECT_CLASS_VIOLATION;
+			slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
+			goto error_return;
+		}
 
-	if ((operation_is_flag_set(operation,OP_FLAG_ACTION_SCHEMA_CHECK)) && slapi_entry_schema_check(pb, e) != 0) 
-	{
-		LDAPDebug(LDAP_DEBUG_TRACE, "entry failed schema check\n", 0, 0, 0);
-		ldap_result_code = LDAP_OBJECT_CLASS_VIOLATION;
-		slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
-		goto error_return;
-	} 
-
-	/* Check attribute syntax */
-	if (slapi_entry_syntax_check(pb, e, 0) != 0)
-	{
-		LDAPDebug(LDAP_DEBUG_TRACE, "entry failed syntax check\n", 0, 0, 0);
-		ldap_result_code = LDAP_INVALID_SYNTAX;
-		slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
-		goto error_return;
+		/* Check attribute syntax */
+		if (slapi_entry_syntax_check(pb, e, 0) != 0)
+		{
+			LDAPDebug(LDAP_DEBUG_TRACE, "entry failed syntax check\n", 0, 0, 0);
+			ldap_result_code = LDAP_INVALID_SYNTAX;
+			slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
+			goto error_return;
+		}
 	}
 
 	opcsn = operation_get_csn (operation);
@@ -654,19 +681,46 @@ ldbm_back_add( Slapi_PBlock *pb )
 		}
 	}
 
+	if ( (originalentry = backentry_dup(addingentry )) == NULL ) {
+		ldap_result_code= LDAP_OPERATIONS_ERROR;
+		goto error_return;
+	}
 	/*
  	 * So, we believe that no code up till here actually added anything
 	 * to persistent store. From now on, we're transacted
 	 */
 	txn.back_txn_txn = NULL; /* ready to create the child transaction */
 	for (retry_count = 0; retry_count < RETRY_TIMES; retry_count++) {
-		if (retry_count > 0) {
+		if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) {
 			dblayer_txn_abort(li,&txn);
-			/* txn is no longer valid - reset slapi_txn to the parent */
-			txn.back_txn_txn = NULL;
 			slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
+
+			if (addingentry_in_cache) {
+				/* addingentry is in cache.  Remove it once. */
+				CACHE_REMOVE(&inst->inst_cache, addingentry);
+				CACHE_RETURN(&inst->inst_cache, &addingentry);
+			} else {
+				backentry_free(&addingentry);
+			}
+			slapi_pblock_set( pb, SLAPI_ADD_ENTRY, originalentry->ep_entry );
+			addingentry = originalentry;
+			if ( (originalentry = backentry_dup( addingentry )) == NULL ) {
+				ldap_result_code= LDAP_OPERATIONS_ERROR;
+				goto error_return;
+			}
+			if (addingentry_in_cache) {
+				/* Adding the resetted addingentry to the cache. */
+				if (cache_add_tentative(&inst->inst_cache,
+				                        addingentry, NULL) != 0) {
+					LDAPDebug0Args(LDAP_DEBUG_CACHE,
+					              "cache_add_tentative concurrency detected\n");
+					ldap_result_code = LDAP_ALREADY_EXISTS;
+					goto error_return;
+				}
+			}
+
 			/* We're re-trying */
-			LDAPDebug( LDAP_DEBUG_TRACE, "Add Retrying Transaction\n", 0, 0, 0 );
+			LDAPDebug0Args(LDAP_DEBUG_BACKLDBM, "Add Retrying Transaction\n");
 #ifndef LDBM_NO_BACKOFF_DELAY
 			{
 				PRIntervalTime interval;
@@ -691,9 +745,22 @@ ldbm_back_add( Slapi_PBlock *pb )
 
 		/* call the transaction pre add plugins just after creating the transaction */
 		if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_PRE_ADD_FN))) {
-			LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_PRE_ADD_FN plugin "
-						   "returned error code %d\n", retval );
-			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			int opreturn = 0;
+			LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_PRE_ADD_FN plugin "
+				       "returned error code %d\n", retval );
+			if (!ldap_result_code) {
+				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
+			if (!ldap_result_code) {
+				LDAPDebug0Args( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_PRE_ADD_FN plugin "
+						"returned error but did not setSLAPI_RESULT_CODE \n" );
+				ldap_result_code = LDAP_OPERATIONS_ERROR;
+				slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
+			slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+			if (!opreturn) {
+				slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval);
+			}
 			goto error_return;
 		}
 
@@ -867,7 +934,7 @@ ldbm_back_add( Slapi_PBlock *pb )
 	if (retry_count == RETRY_TIMES) {
 		/* Failed */
 		LDAPDebug( LDAP_DEBUG_ANY, "Retry count exceeded in add\n", 0, 0, 0 );
-   		ldap_result_code= LDAP_OPERATIONS_ERROR;
+   		ldap_result_code= LDAP_BUSY;
 		goto error_return;
 	}
 
@@ -915,15 +982,27 @@ ldbm_back_add( Slapi_PBlock *pb )
 
 	/* call the transaction post add plugins just before the commit */
 	if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_ADD_FN))) {
-		LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_ADD_FN plugin "
-					   "returned error code %d\n", retval );
-		slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		int opreturn = 0;
+		LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_ADD_FN plugin "
+			       "returned error code %d\n", retval );
+		if (!ldap_result_code) {
+			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		}
+		if (!ldap_result_code) {
+			LDAPDebug0Args( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_ADD_FN plugin "
+					"returned error but did not set SLAPI_RESULT_CODE\n" );
+			ldap_result_code = LDAP_OPERATIONS_ERROR;
+			slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		}
+		slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+		if (!opreturn) {
+			slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval);
+		}
 		goto error_return;
 	}
 
 	retval = dblayer_txn_commit(li,&txn);
 	/* after commit - txn is no longer valid - replace SLAPI_TXN with parent */
-	txn.back_txn_txn = NULL;
 	slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
 	if (0 != retval)
 	{
@@ -969,10 +1048,38 @@ diskfull_return:
 		rc= return_on_disk_full(li);
 	} else {
 		/* It is safer not to abort when the transaction is not started. */
-		if (retry_count > 0) {
+		if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) {
+			/* make sure SLAPI_RESULT_CODE and SLAPI_PLUGIN_OPRETURN are set */
+			int val = 0;
+			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &val);
+			if (!val) {
+				if (!ldap_result_code) {
+					ldap_result_code = LDAP_OPERATIONS_ERROR;
+				}
+				slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
+			slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &val );
+			if (!val) {
+				val = -1;
+				slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, &val );
+			}
+			/* call the transaction post add plugins just before the abort */
+			if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_ADD_FN))) {
+				int opreturn = 0;
+				LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_ADD_FN plugin "
+					       "returned error code %d\n", retval );
+				if (!ldap_result_code) {
+					slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+				}
+				slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+				if (!opreturn) {
+					opreturn = -1;
+					slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+				}
+			}
+
 			dblayer_txn_abort(li,&txn); /* abort crashes in case disk full */
 			/* txn is no longer valid - reset the txn pointer to the parent */
-			txn.back_txn_txn = NULL;
 			slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
 		}
 		rc= SLAPI_FAIL_GENERAL;
@@ -980,7 +1087,7 @@ diskfull_return:
 	
 common_return:
     if (addingentry_in_cache && addingentry)
-    {
+	{
         if (entryrdn_get_switch()) { /* subtree-rename: on */
             /* since adding the entry to the entry cache was successful,
              * let's add the dn to dncache, if not yet done. */
@@ -1023,6 +1130,7 @@ common_return:
 	{
 		slapi_send_ldap_result( pb, ldap_result_code, ldap_result_matcheddn, ldap_result_message, 0, NULL );
 	}
+	backentry_free(&originalentry);
 	slapi_sdn_done(&parentsdn);
 	slapi_ch_free( (void**)&ldap_result_matcheddn );
 	slapi_ch_free( (void**)&errbuf );
@@ -1081,7 +1189,7 @@ add_update_entrydn_operational_attributes(struct backentry *ep)
     bv.bv_val = (void*)backentry_get_ndn(ep);
     bv.bv_len = strlen( bv.bv_val );
     entry_replace_values_with_flags( ep->ep_entry, LDBM_ENTRYDN_STR, bvp,
-                                     SLAPI_ATTR_FLAG_NORMALIZED );
+                                     SLAPI_ATTR_FLAG_NORMALIZED_CIS );
 }
 
 /*
