@@ -69,6 +69,11 @@ ldbm_back_compare( Slapi_PBlock *pb )
 	slapi_pblock_get( pb, SLAPI_COMPARE_VALUE, &bval );
 	slapi_pblock_get( pb, SLAPI_TXN, &txn.back_txn_txn );
 	
+	if ( !txn.back_txn_txn ) {
+		dblayer_txn_init( li, &txn );
+		slapi_pblock_set( pb, SLAPI_TXN, txn.back_txn_txn );
+	}
+
 	inst = (ldbm_instance *) be->be_instance_info;
 	/* get the namespace dn */
 	namespace_dn = (Slapi_DN*)slapi_be_getsuffix(be, 0);
