@@ -60,7 +60,8 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	struct ldbminfo	*li = NULL;
 	struct backentry *e = NULL;
 	struct backentry *tombstone = NULL;
-	char *dn = NULL;
+	struct backentry *original_tombstone = NULL;
+	const char *dn = NULL;
 	back_txn txn;
 	back_txnid parent_txn;
 	int retval = -1;
@@ -75,10 +76,9 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	int rc = 0;
 	int ldap_result_code= LDAP_SUCCESS;
 	char *ldap_result_message= NULL;
-	Slapi_DN sdn;
 	Slapi_DN *sdnp = NULL;
 	char *e_uniqueid = NULL;
-	Slapi_DN *nscpEntrySDN = NULL;
+	Slapi_DN nscpEntrySDN;
 	int dblock_acquired= 0;
 	Slapi_Operation *operation;
 	CSN *opcsn = NULL;
@@ -90,12 +90,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	int create_tombstone_entry = 0;	/* We perform a "regular" LDAP delete but since we use	*/
 									/* replication, we must create a new tombstone entry	*/
 	int tombstone_in_cache = 0;
+	int e_in_cache = 0;
 	entry_address *addr;
 	int addordel_flags = 0; /* passed to index_addordel */
 	char *entryusn_str = NULL;
-	char *prev_entryusn_str = NULL;
 	Slapi_Entry *orig_entry = NULL;
 	Slapi_DN parentsdn;
+	int opreturn = 0;
+	int free_delete_existing_entry = 0;
 
 	slapi_pblock_get( pb, SLAPI_BACKEND, &be);
 	slapi_pblock_get( pb, SLAPI_PLUGIN_PRIVATE, &li );
@@ -105,26 +107,33 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	slapi_pblock_get( pb, SLAPI_OPERATION, &operation );
 	slapi_pblock_get( pb, SLAPI_IS_REPLICATED_OPERATION, &is_replicated_operation );
 	
-	/* sdn needs to be initialized before "goto *_return */
-	slapi_sdn_init(&sdn);
+	slapi_sdn_init(&nscpEntrySDN);
 
 	/* dblayer_txn_init needs to be called before "goto error_return" */
 	dblayer_txn_init(li,&txn);
 	/* the calls to perform searches require the parent txn if any
 	   so set txn to the parent_txn until we begin the child transaction */
-	txn.back_txn_txn = parent_txn;
+	if (parent_txn) {
+		txn.back_txn_txn = parent_txn;
+	} else {
+		parent_txn = txn.back_txn_txn;
+		slapi_pblock_set( pb, SLAPI_TXN, parent_txn );
+	}
 
 	if (pb->pb_conn)
 	{
 		slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_delete", "enter conn=%" NSPRIu64 " op=%d\n", pb->pb_conn->c_connid, operation->o_opid);
 	}
 
-	if (NULL == addr)
+	if ((NULL == addr) || (NULL == sdnp))
 	{
 		/* retval is -1 */
+		slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete",
+		            "Either of DELETE_TARGET_SDN or TARGET_ADDRESS is NULL.\n");
 		goto error_return;
 	}
-	ldap_result_code = slapi_dn_syntax_check(pb, slapi_sdn_get_dn(sdnp), 1);
+	dn = slapi_sdn_get_dn(sdnp);
+	ldap_result_code = slapi_dn_syntax_check(pb, dn, 1);
 	if (ldap_result_code)
 	{
 		ldap_result_code = LDAP_INVALID_DN_SYNTAX;
@@ -139,11 +148,6 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	
 	inst = (ldbm_instance *) be->be_instance_info;
 
-	if (NULL == sdnp) {
-		slapi_sdn_init_normdn_byref(&sdn, dn);
-		sdnp = &sdn;
-	}
-
 	/* The dblock serializes writes to the database,
 	 * which reduces deadlocking in the db code,
 	 * which means that we run faster.
@@ -179,6 +183,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		/* retval is -1 */
 		goto error_return; /* error result sent by find_entry2modify() */
 	}
+	e_in_cache = 1; /* e is cached */
 
 	if ( slapi_entry_has_children( e->ep_entry ) )
 	{
@@ -187,10 +192,6 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		goto error_return;
 	}
 
-	/* set entry in case be-preop plugins need to work on it (e.g., USN) */
-	slapi_pblock_get( pb, SLAPI_DELETE_BEPREOP_ENTRY, &orig_entry );
-	slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY, e->ep_entry );
-
 	/* Don't call pre-op for Tombstone entries */
 	if (!delete_tombstone_entry)
 	{
@@ -202,27 +203,31 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		 */
 		ldap_result_code= get_copy_of_entry(pb, addr, &txn,
 						SLAPI_DELETE_EXISTING_ENTRY, !is_replicated_operation);
+		free_delete_existing_entry = 1;
 		if(ldap_result_code==LDAP_OPERATIONS_ERROR ||
 		   ldap_result_code==LDAP_INVALID_DN_SYNTAX)
 		{
 			/* restore original entry so the front-end delete code can free it */
-			slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY, orig_entry );
 			/* retval is -1 */
 			goto error_return;
 		}
 		slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
 
 		rc = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_PRE_DELETE_FN);
-
 		if (rc == -1)
 		{
 			/* 
 			 * Plugin indicated some kind of failure,
 			 * or that this Operation became a No-Op.
 			 */
-			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			if (!ldap_result_code) {
+				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
 			/* restore original entry so the front-end delete code can free it */
-			slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY, orig_entry );
+			slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &opreturn );
+			if (!opreturn) {
+				slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &rc );
+			}
 			/* retval is -1 */
 			goto error_return;
 		}
@@ -231,8 +236,6 @@ ldbm_back_delete( Slapi_PBlock *pb )
 									OP_FLAG_TOMBSTONE_ENTRY);
 	}
 
-	slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY, orig_entry );
-
 	/*
 	 * Sanity check to avoid to delete a non-tombstone or to tombstone again
 	 * a tombstone entry. This should not happen (see bug 561003).
@@ -249,7 +252,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		PR_ASSERT(!is_tombstone_entry);
 		if (is_tombstone_entry) { 
 				slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete",
-						"Attempt to Tombstone again a tombstone entry %s\n", dn);
+					"Attempt to Tombstone again a tombstone entry %s\n", dn);
 			delete_tombstone_entry = 1;
 		}
 	}
@@ -261,12 +264,8 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	opcsn = operation_get_csn (operation);
 	if (!delete_tombstone_entry)
 	{
-		/* If both USN and replication is enabled, csn set by replication 
-		 * should be honored. */
-		if ((opcsn == NULL || ldbm_usn_enabled(be)) &&
-						!is_fixup_operation && operation->o_csngen_handler)
+		if ((opcsn == NULL) && !is_fixup_operation && operation->o_csngen_handler)
 		{
-			csn_free(&opcsn); /* free opcsn set by USN plugin, if any */
 			/*
 			 * Current op is a user request. Opcsn will be assigned
 			 * by entry_assign_operation_csn() if the dn is in an
@@ -280,14 +279,15 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			{
 				entry_set_maxcsn (e->ep_entry, opcsn);
 			}
-			/*
-			 * We are dealing with replication and if we haven't been called to
-			 * remove a tombstone, then it's because  we want to create a new one.
-			 */
-			if ( slapi_operation_get_replica_attr (pb, operation, "nsds5ReplicaTombstonePurgeInterval", &create_tombstone_entry) == 0)
-			{
-				create_tombstone_entry = (create_tombstone_entry < 0) ? 0 : 1;
-			}
+		}
+		/*
+		 * We are dealing with replication and if we haven't been called to
+		 * remove a tombstone, then it's because  we want to create a new one.
+		 */
+		if ( slapi_operation_get_replica_attr (pb, operation, "nsds5ReplicaTombstonePurgeInterval",
+											   &create_tombstone_entry) == 0 )
+		{
+			create_tombstone_entry = (create_tombstone_entry < 0) ? 0 : 1;
 		}
 	}
 
@@ -365,7 +365,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 				slapi_pblock_set ( pb, SLAPI_DELETE_GLUE_PARENT_ENTRY,
 					slapi_entry_dup (parent_modify_c.new_entry->ep_entry) );
 			}
-	    }		
+		}
 	}
 	slapi_sdn_done(&parentsdn);
     
@@ -382,7 +382,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			childuniqueid);
 		Slapi_Value *tomb_value;
 
-		nscpEntrySDN = slapi_entry_get_sdn(e->ep_entry);
+		slapi_sdn_set_ndn_byval(&nscpEntrySDN, slapi_sdn_get_ndn(slapi_entry_get_sdn(e->ep_entry)));
 
 		/* Copy the entry unique_id for URP conflict checking */
 		e_uniqueid = slapi_ch_strdup(childuniqueid);
@@ -412,32 +412,19 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			/* The suffix entry has no parent */
 			slapi_entry_add_string(tombstone->ep_entry, SLAPI_ATTR_VALUE_PARENT_UNIQUEID, parentuniqueid);
 		}
-		if(nscpEntrySDN!=NULL)
-		{
-			slapi_entry_add_string(tombstone->ep_entry, SLAPI_ATTR_NSCP_ENTRYDN, slapi_sdn_get_ndn(nscpEntrySDN));
-		}
+		slapi_entry_add_string(tombstone->ep_entry, SLAPI_ATTR_NSCP_ENTRYDN, slapi_sdn_get_ndn(&nscpEntrySDN));
 		tomb_value = slapi_value_new_string(SLAPI_ATTR_VALUE_TOMBSTONE);
 		value_update_csn(tomb_value, CSN_TYPE_VALUE_UPDATED,
 			operation_get_csn(operation));
 		slapi_entry_add_value(tombstone->ep_entry, SLAPI_ATTR_OBJECTCLASS, tomb_value);
 		slapi_value_free(&tomb_value);
-		
-		/* retrieve previous entry usn value, if any */
-		prev_entryusn_str = slapi_entry_attr_get_charptr(tombstone->ep_entry,
-												SLAPI_ATTR_ENTRYUSN_PREV);
-		if (prev_entryusn_str) {
-			/* discard the previous value from the tombstone entry */
-		    retval = slapi_entry_delete_string(tombstone->ep_entry,
-							SLAPI_ATTR_ENTRYUSN_PREV, prev_entryusn_str);
-			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_TRACE,
-							"delete (deleting %s) failed, err=%d\n",
-							SLAPI_ATTR_ENTRYUSN, retval, 0) ;
-			}
-		}
-
 		/* XXXggood above used to be: slapi_entry_add_string(tombstone->ep_entry, SLAPI_ATTR_OBJECTCLASS, SLAPI_ATTR_VALUE_TOMBSTONE); */
 		/* JCMREPL - Add a description of what's going on? */
+
+		if ( (original_tombstone = backentry_dup( tombstone )) == NULL ) {
+			ldap_result_code= LDAP_OPERATIONS_ERROR;
+			goto error_return;
+		}
 	}
 
 	if (!is_ruv && !is_fixup_operation && !delete_tombstone_entry) {
@@ -460,13 +447,52 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	
 	txn.back_txn_txn = NULL; /* ready to create the child transaction */
 	for (retry_count = 0; retry_count < RETRY_TIMES; retry_count++) {
-		if (retry_count > 0) {
+		if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) {
+			Slapi_Entry *ent = NULL;
+
 			dblayer_txn_abort(li,&txn);
+			slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
+
+			/* reset original entry */
+			slapi_pblock_get( pb, SLAPI_DELETE_EXISTING_ENTRY, &ent );
+			if (ent && free_delete_existing_entry) {
+				slapi_entry_free(ent);
+				slapi_pblock_set( pb, SLAPI_DELETE_EXISTING_ENTRY, NULL );
+			}
+			slapi_pblock_set( pb, SLAPI_DELETE_EXISTING_ENTRY, slapi_entry_dup( e->ep_entry ) );
+			free_delete_existing_entry = 1; /* must free the dup */
+			if (create_tombstone_entry) {
+				slapi_sdn_set_ndn_byval(&nscpEntrySDN, slapi_sdn_get_ndn(slapi_entry_get_sdn(e->ep_entry)));
+			}
+
+			/* reset tombstone entry */
+			if (original_tombstone) {
+				if (tombstone_in_cache) {
+					CACHE_REMOVE(&inst->inst_cache, tombstone);
+					CACHE_RETURN(&inst->inst_cache, &tombstone);
+					tombstone_in_cache = 0; 
+				} else {
+					backentry_free(&tombstone);
+				}
+				tombstone = original_tombstone;
+				if ( (original_tombstone = backentry_dup( tombstone )) == NULL ) {
+					ldap_result_code= LDAP_OPERATIONS_ERROR;
+					goto error_return;
+				}
+			}
+
+			/* reset original entry in cache */
+			if (!e_in_cache) {
+				CACHE_ADD(&inst->inst_cache, e, NULL);
+				e_in_cache = 1;
+			}
+
 			/* We're re-trying */
-			LDAPDebug( LDAP_DEBUG_TRACE, "Delete Retrying Transaction\n", 0, 0, 0 );
+			LDAPDebug0Args(LDAP_DEBUG_BACKLDBM,
+			               "Delete Retrying Transaction\n");
 #ifndef LDBM_NO_BACKOFF_DELAY
-            {
-	        PRIntervalTime interval;
+			{
+			PRIntervalTime interval;
 			interval = PR_MillisecondsToInterval(slapi_rand() % 100);
 			DS_Sleep(interval);
 			}
@@ -482,16 +508,62 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		/* stash the transaction */
 		slapi_pblock_set(pb, SLAPI_TXN, txn.back_txn_txn);
 
-		/* call the transaction pre delete plugins just after creating the transaction */
-		if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_PRE_DELETE_FN))) {
-			LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_PRE_DELETE_FN plugin "
-						   "returned error code %d\n", retval );
-			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		/* call the transaction pre delete plugins just after creating
+		 * the transaction */
+		/* these should not need to modify the entry to be deleted -
+		   if for some reason they ever do, do not use e->ep_entry since
+		   it could be in the cache and referenced by other search threads -
+		   instead, have them modify a copy of the entry */
+		retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_PRE_DELETE_FN);
+		if (retval) {
+			LDAPDebug1Arg( LDAP_DEBUG_TRACE,
+			               "SLAPI_PLUGIN_BE_TXN_PRE_DELETE_FN plugin "
+			               "returned error code %d\n", retval );
+			if (!ldap_result_code) {
+				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
+			if (!opreturn) {
+				slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &opreturn );
+			}
+			if (!opreturn) {
+				slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN,
+				                  ldap_result_code ?
+				                  &ldap_result_code : &retval );
+			}
 			goto error_return;
 		}
 
 		if(create_tombstone_entry)
 		{
+			slapi_pblock_get( pb, SLAPI_DELETE_BEPREOP_ENTRY, &orig_entry );
+			/* this is ok because no other threads should be accessing
+			   the tombstone entry */
+			slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY,
+			                  tombstone->ep_entry );
+			rc = plugin_call_plugins(pb,
+			                       SLAPI_PLUGIN_BE_TXN_PRE_DELETE_TOMBSTONE_FN);
+			if (rc == -1) {
+				/* 
+				* Plugin indicated some kind of failure,
+				 * or that this Operation became a No-Op.
+				 */
+				if (!ldap_result_code) {
+					slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+				}
+				/* restore original entry so the front-end delete code 
+				 * can free it */
+				slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &opreturn );
+				if (!opreturn) {
+					slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN,
+					                  ldap_result_code ?
+					                  &ldap_result_code : &rc );
+				}
+				/* retval is -1 */
+				goto error_return;
+			}
+			slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY, orig_entry );
+			orig_entry = NULL;
+
 			/*
 			 * The entry is not removed from the disk when we tombstone an
 			 * entry. We change the DN, add objectclass=tombstone, and record
@@ -502,6 +574,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			 * tentatively for now, then cache_add again when the original
 			 * entry is removed from the cache.
 			 */
+			if (cache_add_tentative( &inst->inst_cache, tombstone, NULL) == 0) {
+				tombstone_in_cache = 1;
+			} else if (!(tombstone->ep_state & ENTRY_STATE_NOTINCACHE)) {
+			    LDAPDebug1Arg(LDAP_DEBUG_CACHE,
+			                  "id2entry_add tombstone (%s) is in cache\n",
+			                  slapi_entry_get_dn(tombstone->ep_entry));
+			    tombstone_in_cache = 1;
+			}
 			retval = id2entry_add( be, tombstone, &txn );
 			if (DB_LOCK_DEADLOCK == retval) {
 				LDAPDebug( LDAP_DEBUG_ARGS, "delete 1 DB_LOCK_DEADLOCK\n", 0, 0, 0 );
@@ -516,14 +596,19 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							  LDAP_OPERATIONS_ERROR, retry_count);
 				goto error_return;
 			}
-			if (cache_add_tentative( &inst->inst_cache, tombstone, NULL) == 0) {
-				tombstone_in_cache = 1;
-			} else if (!(tombstone->ep_state & ENTRY_STATE_NOTINCACHE)) {
-			    LDAPDebug1Arg(LDAP_DEBUG_CACHE,
-			                  "id2entry_add tombstone (%s) is in cache\n",
-			                  slapi_entry_get_dn(tombstone->ep_entry));
-			    tombstone_in_cache = 1;
+			if (cache_replace( &inst->inst_cache, e, tombstone ) != 0 ) {
+				LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "ldbm_back_delete cache_replace failed\n");
+				DEL_SET_ERROR(ldap_result_code, 
+							  LDAP_OPERATIONS_ERROR, retry_count);
+				retval= -1;
+				goto error_return;
+			} else {
+				e_in_cache = 0; /* e un-cached */
 			}
+			/* tombstone was already added to the cache via cache_add_tentative (to reserve its spot in the cache)
+			   and/or id2entry_add - so it already had one refcount - cache_replace adds another refcount -
+			   drop the extra ref added by cache_replace */
+			CACHE_RETURN( &inst->inst_cache, &tombstone );
 		}
 		else
 		{
@@ -611,7 +696,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 				goto error_return;
 			}
 			retval = index_addordel_string(be, SLAPI_ATTR_NSCP_ENTRYDN,
-							slapi_sdn_get_ndn(nscpEntrySDN),
+							slapi_sdn_get_ndn(&nscpEntrySDN),
 							tombstone->ep_id, BE_INDEX_ADD, &txn);
 			if (DB_LOCK_DEADLOCK == retval) {
 				LDAPDebug( LDAP_DEBUG_ARGS,
@@ -655,30 +740,6 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					goto error_return;
 				}
 			}
-			/* delete a previous value (if it exists) from the entryusn index */
-			if (prev_entryusn_str) {
-				retval = index_addordel_string(be, SLAPI_ATTR_ENTRYUSN,
-								prev_entryusn_str, tombstone->ep_id,
-								BE_INDEX_DEL|BE_INDEX_EQUALITY, &txn);
-				slapi_ch_free_string(&prev_entryusn_str);
-				if (DB_LOCK_DEADLOCK == retval) {
-					LDAPDebug( LDAP_DEBUG_ARGS,
-								"delete (deleting %s) DB_LOCK_DEADLOCK\n",
-								SLAPI_ATTR_ENTRYUSN, 0, 0 );
-					/* Retry txn */
-					continue;
-				}
-				if (0 != retval) {
-					LDAPDebug( LDAP_DEBUG_TRACE, 
-								"delete (deleting %s) failed, err=%d %s\n",
-								SLAPI_ATTR_ENTRYUSN, retval,
-								(msg = dblayer_strerror( retval )) ? msg : "" );
-					if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
-					DEL_SET_ERROR(ldap_result_code, 
-								  LDAP_OPERATIONS_ERROR, retry_count);
-					goto error_return;
-				}
-			}
 			if (entryrdn_get_switch()) /* subtree-rename: on */
 			{
 				Slapi_Attr *attr;
@@ -692,6 +753,12 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					retval = index_addordel_values_sv(be, LDBM_PARENTID_STR, 
 					                                  svals, NULL, e->ep_id, 
 					                                  BE_INDEX_ADD, &txn);
+					if (DB_LOCK_DEADLOCK == retval) {
+						LDAPDebug0Args( LDAP_DEBUG_ARGS,
+										"delete (updating " LDBM_PARENTID_STR ") DB_LOCK_DEADLOCK\n");
+						/* Retry txn */
+						continue;
+					}
 					if ( retval ) {
 						LDAPDebug( LDAP_DEBUG_TRACE, 
 								"delete (deleting %s) failed, err=%d %s\n",
@@ -703,18 +770,33 @@ ldbm_back_delete( Slapi_PBlock *pb )
 						goto error_return;
 					}
 				}
-				entryrdn_index_entry(be, e, BE_INDEX_DEL, &txn);
-				retval =
-				        entryrdn_index_entry(be, tombstone, BE_INDEX_ADD, &txn);
+				retval = entryrdn_index_entry(be, e, BE_INDEX_DEL, &txn);
 				if (DB_LOCK_DEADLOCK == retval) {
 					LDAPDebug0Args( LDAP_DEBUG_ARGS,
-								"delete (adding entryrdn) DB_LOCK_DEADLOCK\n");
+								"delete (deleting entryrdn) DB_LOCK_DEADLOCK\n");
 					/* Retry txn */
 					continue;
 				}
 				if (0 != retval) {
 					LDAPDebug2Args( LDAP_DEBUG_TRACE, 
-								"delete (adding entryrdn) failed, err=%d %s\n",
+								"delete (deleting entryrdn) failed, err=%d %s\n",
+								retval,
+								(msg = dblayer_strerror( retval )) ? msg : "" );
+					if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
+					DEL_SET_ERROR(ldap_result_code, 
+								  LDAP_OPERATIONS_ERROR, retry_count);
+					goto error_return;
+				}
+				retval = entryrdn_index_entry(be, tombstone, BE_INDEX_ADD, &txn);
+				if (DB_LOCK_DEADLOCK == retval) {
+					LDAPDebug0Args( LDAP_DEBUG_ARGS,
+								"adding (adding tombstone entryrdn) DB_LOCK_DEADLOCK\n");
+					/* Retry txn */
+					continue;
+				}
+				if (0 != retval) {
+					LDAPDebug2Args( LDAP_DEBUG_TRACE, 
+								"adding (adding tombstone entryrdn) failed, err=%d %s\n",
 								retval,
 								(msg = dblayer_strerror( retval )) ? msg : "" );
 					if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
@@ -909,22 +991,38 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	if (retry_count == RETRY_TIMES) {
 		/* Failed */
 		LDAPDebug( LDAP_DEBUG_ANY, "Retry count exceeded in delete\n", 0, 0, 0 );
-		ldap_result_code= LDAP_OPERATIONS_ERROR;
+		ldap_result_code= LDAP_BUSY;
 		retval = -1;
 		goto error_return;
 	}
 
 	/* call the transaction post delete plugins just before the commit */
-	if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN))) {
-		LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
-					   "returned error code %d\n", retval );
-		slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+	if (plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN)) {
+		LDAPDebug0Args( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
+						"returned error code\n" );
+		if (!ldap_result_code) {
+			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		}
+		if (!ldap_result_code) {
+			LDAPDebug0Args( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
+							"returned error code but did not set SLAPI_RESULT_CODE\n" );
+			ldap_result_code = LDAP_OPERATIONS_ERROR;
+			slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		}
+		if (!opreturn) {
+			slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &opreturn );
+		}
+		if (!retval) {
+			retval = -1;
+		}
+		if (!opreturn) {
+			slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, &retval );
+		}
 		goto error_return;
 	}
 
 	retval = dblayer_txn_commit(li,&txn);
 	/* after commit - txn is no longer valid - replace SLAPI_TXN with parent */
-	txn.back_txn_txn = NULL;
 	slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
 	if (0 != retval)
 	{
@@ -934,16 +1032,15 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	}
 
 	/* delete from cache and clean up */
-	CACHE_REMOVE(&inst->inst_cache, e);
-	cache_unlock_entry( &inst->inst_cache, e );
-	CACHE_RETURN( &inst->inst_cache, &e );
-	if (tombstone_in_cache) {
-		if (CACHE_ADD( &inst->inst_cache, tombstone, NULL ) == 0) {
-			tombstone_in_cache = 1;
-		} else {
-			tombstone_in_cache = 0;
+	if (e) {
+		if (e_in_cache) {
+			CACHE_REMOVE(&inst->inst_cache, e);
 		}
+		cache_unlock_entry(&inst->inst_cache, e);
+		CACHE_RETURN(&inst->inst_cache, &e);
+		e = NULL;
 	}
+
 	if (parent_found)
 	{
 		 /* Replace the old parent entry with the newly modified one */
@@ -967,6 +1064,9 @@ error_return:
 	if (tombstone_in_cache)
 	{
 		CACHE_REMOVE( &inst->inst_cache, tombstone );
+		CACHE_RETURN( &inst->inst_cache, &tombstone );
+		tombstone = NULL;
+		tombstone_in_cache = 0;
 	}
 	else
 	{
@@ -987,17 +1087,54 @@ error_return:
 	    rc= SLAPI_FAIL_GENERAL;
 
 	/* It is safer not to abort when the transaction is not started. */
-	if (retry_count > 0) {
+	if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) {
+		/* make sure SLAPI_RESULT_CODE and SLAPI_PLUGIN_OPRETURN are set */
+		int val = 0;
+		slapi_pblock_get(pb, SLAPI_RESULT_CODE, &val);
+		if (!val) {
+			if (!ldap_result_code) {
+				ldap_result_code = LDAP_OPERATIONS_ERROR;
+			}
+			slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		}
+		slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &val );
+		if (!val) {
+			opreturn = retval;
+			slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, &retval );
+		}
+
+		/* call the transaction post delete plugins just before the commit */
+		if (plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN)) {
+			LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
+						   "returned error code %d\n", retval );
+			if (!ldap_result_code) {
+				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
+			if (!opreturn) {
+				slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &opreturn );
+			}
+			if (!opreturn) {
+				slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval );
+			}
+		}
+
 		dblayer_txn_abort(li,&txn); /* abort crashes in case disk full */
 		/* txn is no longer valid - reset the txn pointer to the parent */
-		txn.back_txn_txn = NULL;
 		slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
 	}
 	
 common_return:
+	if (orig_entry) {
+		/* NOTE: #define SLAPI_DELETE_BEPREOP_ENTRY SLAPI_ENTRY_PRE_OP */
+		/* so if orig_entry is NULL, we will wipe out SLAPI_ENTRY_PRE_OP
+		   for the post op plugins */
+		slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY, orig_entry );
+	}
 	if (tombstone_in_cache)
 	{
 		CACHE_RETURN( &inst->inst_cache, &tombstone );
+		tombstone = NULL;
+		tombstone_in_cache = 0;
 	}
 	else
 	{
@@ -1011,17 +1148,7 @@ common_return:
 	 * but not if the operation is purging tombstones.
 	 */
 	if (!delete_tombstone_entry) {
-		if (e) {
-			/* set entry in case be-postop plugins need to work on it
-			 * (e.g., USN) */
-			slapi_pblock_get( pb, SLAPI_DELETE_BEPOSTOP_ENTRY, &orig_entry );
-			slapi_pblock_set( pb, SLAPI_DELETE_BEPOSTOP_ENTRY, e->ep_entry );
-		}
 		plugin_call_plugins (pb, SLAPI_PLUGIN_BE_POST_DELETE_FN);
-		/* set original entry back */
-		if (e) {
-			slapi_pblock_set( pb, SLAPI_DELETE_BEPOSTOP_ENTRY, orig_entry );
-		}
 	}
 
 	/* Need to return to cache after post op plugins are called */
@@ -1054,11 +1181,16 @@ diskfull_return:
 		 * with this entry. Now that this name has been given up, one
 		 * of those entries can take over the name. 
 		 */
-		slapi_pblock_set(pb, SLAPI_URP_NAMING_COLLISION_DN, slapi_ch_strdup (dn));
+		slapi_pblock_set(pb, SLAPI_URP_NAMING_COLLISION_DN, slapi_ch_strdup(dn));
 	}
-	done_with_pblock_entry(pb, SLAPI_DELETE_EXISTING_ENTRY);
+	if (free_delete_existing_entry) {
+		done_with_pblock_entry(pb, SLAPI_DELETE_EXISTING_ENTRY);
+	} else { /* owned by someone else */
+		slapi_pblock_set(pb, SLAPI_DELETE_EXISTING_ENTRY, NULL);
+	}
+	backentry_free(&original_tombstone);
 	slapi_ch_free((void**)&errbuf);
-	slapi_sdn_done(&sdn);
+	slapi_sdn_done(&nscpEntrySDN);
 	slapi_ch_free_string(&e_uniqueid);
 	if (pb->pb_conn)
 	{
