@@ -151,7 +151,7 @@ int modify_update_all(backend *be, Slapi_PBlock *pb,
 	 * Update the ID to Entry index. 
 	 * Note that id2entry_add replaces the entry, so the Entry ID stays the same.
 	 */
-	retval = id2entry_add_ext( be, mc->new_entry, txn, mc->attr_encrypt );
+	retval = id2entry_add_ext( be, mc->new_entry, txn, mc->attr_encrypt, NULL );
 	if ( 0 != retval ) {
 		if (DB_LOCK_DEADLOCK != retval)
 		{
@@ -187,15 +187,132 @@ error:
 	return retval;
 }
 
+/**
+   Apply the mods to the ec entry.  Check for syntax, schema problems.
+   Check for abandon.
+
+   Return code:
+   -1 - error - result code and message are set appropriately
+   0 - successfully applied and checked
+   1 - not an error - no mods to apply or op abandoned
+ */
+static int
+modify_apply_check_expand(
+	Slapi_PBlock *pb,
+	Slapi_Operation *operation,
+	LDAPMod **mods, /* list of mods to apply */
+	struct backentry *e, /* original "before" entry */
+	struct backentry *ec, /* "after" entry with mods applied */
+	Slapi_Entry **postentry,
+	int *ldap_result_code,
+	char **ldap_result_message
+)
+{
+	int rc = 0;
+	int i;
+	int repl_op;
+	int change_entry = 0;
+	Slapi_Mods smods = {0};
+	CSN *csn = operation_get_csn(operation);
+
+	slapi_pblock_get (pb, SLAPI_IS_REPLICATED_OPERATION, &repl_op);
+	slapi_mods_init_byref( &smods, mods );
+
+	if ( (change_entry = mods_have_effect (ec->ep_entry, &smods)) ) {
+		*ldap_result_code = entry_apply_mods_wsi(ec->ep_entry, &smods, csn,
+												 operation_is_flag_set(operation, OP_FLAG_REPLICATED));
+		/*
+		 * XXXmcs: it would be nice to get back an error message from
+		 * the above call so we could pass it along to the client, e.g.,
+		 * "duplicate value for attribute givenName."
+		 */
+	} else {
+		Slapi_Entry *epostop = NULL;
+		/* If the entry was not actually changed, we still need to
+		 * set the SLAPI_ENTRY_POST_OP field in the pblock (post-op
+		 * plugins expect that field to be present for all modify
+		 * operations that return LDAP_SUCCESS).
+		 */
+		slapi_pblock_get ( pb, SLAPI_ENTRY_POST_OP, &epostop );
+		slapi_entry_free ( epostop ); /* free existing one, if any */
+		slapi_pblock_set ( pb, SLAPI_ENTRY_POST_OP, slapi_entry_dup( e->ep_entry ) );
+		*postentry = NULL; /* to avoid free in main error cleanup code */
+	}
+	if ( !change_entry || *ldap_result_code != 0 ) {
+		/* change_entry == 0 is not an error just a no-op */
+		rc = change_entry ? -1 : 1;
+		goto done;
+	}
+
+	/*
+	 * If the objectClass attribute type was modified in any way, expand
+	 * the objectClass values to reflect the inheritance hierarchy.
+	 */
+	for ( i = 0; mods[i] != NULL && !repl_op; ++i ) {
+		if ( 0 == strcasecmp( SLAPI_ATTR_OBJECTCLASS, mods[i]->mod_type )) {
+			slapi_schema_expand_objectclasses( ec->ep_entry );
+			break;
+		}
+	}
+
+	/*
+	 * We are about to pass the last abandon test, so from now on we are
+	 * committed to finish this operation. Set status to "will complete"
+	 * before we make our last abandon check to avoid race conditions in
+	 * the code that processes abandon operations.
+	 */
+	operation->o_status = SLAPI_OP_STATUS_WILL_COMPLETE;
+	if ( slapi_op_abandoned( pb ) ) {
+		rc = 1;
+		goto done;
+	}
+
+	/* if this is a replicated op, we don't need to perform these checks */
+	if(!repl_op){
+		/* check that the entry still obeys the schema */
+		if ((operation_is_flag_set(operation,OP_FLAG_ACTION_SCHEMA_CHECK)) &&
+			slapi_entry_schema_check( pb, ec->ep_entry ) != 0 ) {
+			*ldap_result_code = LDAP_OBJECT_CLASS_VIOLATION;
+			slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, ldap_result_message);
+			rc = -1;
+			goto done;
+		}
+
+		/* check attribute syntax for the new values */
+		if (slapi_mods_syntax_check(pb, mods, 0) != 0) {
+			*ldap_result_code = LDAP_INVALID_SYNTAX;
+			slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, ldap_result_message);
+			rc = -1;
+			goto done;
+		}
+
+		/*
+		 * make sure the entry contains all values in the RDN.
+		 * if not, the modification must have removed them.
+		 */
+		if ( ! slapi_entry_rdn_values_present( ec->ep_entry ) ) {
+			*ldap_result_code= LDAP_NOT_ALLOWED_ON_RDN;
+			rc = -1;
+			goto done;
+		}
+	}
+
+done:
+	slapi_mods_done( &smods );
+
+	return rc;
+}
+
 int
 ldbm_back_modify( Slapi_PBlock *pb )
 {
 	backend *be;
 	ldbm_instance *inst = NULL;
 	struct ldbminfo		*li;
-	struct backentry	*e = NULL, *ec = NULL;
+	struct backentry	*e = NULL, *ec = NULL, *original_entry = NULL;
 	Slapi_Entry		*postentry = NULL;
-	LDAPMod			**mods;
+	LDAPMod			**mods = NULL;
+	LDAPMod			**mods_original = NULL;
 	Slapi_Mods smods = {0};
 	back_txn txn;
 	back_txnid		parent_txn;
@@ -212,20 +329,32 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	Slapi_Operation *operation;
 	int dblock_acquired= 0;
 	entry_address *addr;
-	int change_entry = 0;
 	int ec_in_cache = 0;
 	int is_fixup_operation= 0;
 	int is_ruv = 0;                 /* True if the current entry is RUV */
 	CSN *opcsn = NULL;
-	int i = 0;
+	int repl_op;
+	int opreturn = 0;
+	int mod_count = 0;
 
 	slapi_pblock_get( pb, SLAPI_BACKEND, &be);
 	slapi_pblock_get( pb, SLAPI_PLUGIN_PRIVATE, &li );
 	slapi_pblock_get( pb, SLAPI_TARGET_ADDRESS, &addr );
 	slapi_pblock_get( pb, SLAPI_MODIFY_MODS, &mods );
 	slapi_pblock_get( pb, SLAPI_TXN, (void**)&parent_txn );
+	slapi_pblock_get (pb, SLAPI_IS_REPLICATED_OPERATION, &repl_op);
 
 	slapi_pblock_get( pb, SLAPI_OPERATION, &operation );
+	dblayer_txn_init(li,&txn); /* must do this before first goto error_return */
+	/* the calls to perform searches require the parent txn if any
+	   so set txn to the parent_txn until we begin the child transaction */
+	if (parent_txn) {
+		txn.back_txn_txn = parent_txn;
+	} else {
+		parent_txn = txn.back_txn_txn;
+		slapi_pblock_set( pb, SLAPI_TXN, parent_txn );
+	}
+
 	if (NULL == operation)
 	{
 		ldap_result_code = LDAP_OPERATIONS_ERROR;
@@ -236,20 +365,20 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	is_ruv = operation_is_flag_set(operation, OP_FLAG_REPL_RUV);
 	inst = (ldbm_instance *) be->be_instance_info;
 
-	dblayer_txn_init(li,&txn);
-	/* the calls to search for entries require the parent txn if any
-	   so set txn to the parent_txn until we begin the child transaction */
-	txn.back_txn_txn = parent_txn;
 	if (NULL == addr)
 	{
 		goto error_return;
 	}
-	ldap_result_code = slapi_dn_syntax_check(pb, slapi_sdn_get_dn(addr->sdn), 1);
-	if (ldap_result_code)
-	{
-		ldap_result_code = LDAP_INVALID_DN_SYNTAX;
-		slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
-		goto error_return;
+
+	/* no need to check the dn syntax as this is a replicated op */
+	if(!repl_op){
+		ldap_result_code = slapi_dn_syntax_check(pb, slapi_sdn_get_dn(addr->sdn), 1);
+		if (ldap_result_code)
+		{
+			ldap_result_code = LDAP_INVALID_DN_SYNTAX;
+			slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
+			goto error_return;
+		}
 	}
 
 	/* The dblock serializes writes to the database,
@@ -302,88 +431,37 @@ ldbm_back_modify( Slapi_PBlock *pb )
 		goto error_return;
 	}
 
-	remove_illegal_mods(mods);
+	if(!repl_op){
+	    remove_illegal_mods(mods);
+	}
 
 	/* ec is the entry that our bepreop should get to mess with */
 	slapi_pblock_set( pb, SLAPI_MODIFY_EXISTING_ENTRY, ec->ep_entry );
 	slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
 
-	plugin_call_plugins(pb, SLAPI_PLUGIN_BE_PRE_MODIFY_FN);
-
-	slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+	if ((opreturn = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_PRE_MODIFY_FN)) ||
+		(slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code) && ldap_result_code) ||
+		(slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn) && opreturn)) {
+		slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+		if (!ldap_result_code) {
+			LDAPDebug0Args(LDAP_DEBUG_ANY, "ldbm_back_modify: SLAPI_PLUGIN_BE_PRE_MODIFY_FN "
+				       "returned error but did not set SLAPI_RESULT_CODE\n");
+			ldap_result_code = LDAP_OPERATIONS_ERROR;
+		}
+		if (!opreturn) {
+			opreturn = -1;
+			slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+		}
+	}
 	/* The Plugin may have messed about with some of the PBlock parameters... ie. mods */
 	slapi_pblock_get( pb, SLAPI_MODIFY_MODS, &mods );
 	slapi_mods_init_byref(&smods,mods);
+	mod_count = slapi_mods_get_num_mods(&smods);
 
-	{
-		CSN *csn = operation_get_csn(operation);
-		if ( (change_entry = mods_have_effect (ec->ep_entry, &smods)) ) {
-			ldap_result_code = entry_apply_mods_wsi(ec->ep_entry, &smods, csn, operation_is_flag_set(operation,OP_FLAG_REPLICATED));
-			/*
-			 * XXXmcs: it would be nice to get back an error message from
-			 * the above call so we could pass it along to the client, e.g.,
-			 * "duplicate value for attribute givenName."
-			 */
-		} else {
-			 /* If the entry was not actually changed, we still need to
-			 * set the SLAPI_ENTRY_POST_OP field in the pblock (post-op
-			 * plugins expect that field to be present for all modify
-			 * operations that return LDAP_SUCCESS).
-			 */
-			postentry = slapi_entry_dup( e->ep_entry );
-			slapi_pblock_set ( pb, SLAPI_ENTRY_POST_OP, postentry );
-			postentry = NULL;	/* avoid removal/free in error_return code */
-		}
-		if ( !change_entry || ldap_result_code != 0 ) {
-			/* change_entry == 0 is not an error, but we need to free lock etc */
-			goto error_return;
-		}
-	}
-
-	/*
-	 * If the objectClass attribute type was modified in any way, expand
-	 * the objectClass values to reflect the inheritance hierarchy.
-	 */
-	for ( i = 0; mods[i] != NULL; ++i ) {
-		if ( 0 == strcasecmp( SLAPI_ATTR_OBJECTCLASS, mods[i]->mod_type )) {
-			slapi_schema_expand_objectclasses( ec->ep_entry );
-			break;
-		}
-	}
-
-	/*
-	 * We are about to pass the last abandon test, so from now on we are
-	 * committed to finish this operation. Set status to "will complete"
-	 * before we make our last abandon check to avoid race conditions in
-	 * the code that processes abandon operations.
-	 */
-	operation->o_status = SLAPI_OP_STATUS_WILL_COMPLETE;
-	if ( slapi_op_abandoned( pb ) ) {
-		goto error_return;
-	}
-
-	/* check that the entry still obeys the schema */
-	if ( (operation_is_flag_set(operation,OP_FLAG_ACTION_SCHEMA_CHECK)) && 
-		  slapi_entry_schema_check( pb, ec->ep_entry ) != 0 ) {
-		ldap_result_code= LDAP_OBJECT_CLASS_VIOLATION;
-		slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
-		goto error_return;
-	}
-
-	/* check attribute syntax for the new values */
-	if (slapi_mods_syntax_check(pb, mods, 0) != 0)
-	{
-		ldap_result_code = LDAP_INVALID_SYNTAX;
-		slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
-		goto error_return;
-	}
-
-	/*
-	 * make sure the entry contains all values in the RDN.
-	 * if not, the modification must have removed them.
-	 */
-	if ( ! slapi_entry_rdn_values_present( ec->ep_entry ) ) {
-		ldap_result_code= LDAP_NOT_ALLOWED_ON_RDN;
+	/* apply the mods, check for syntax, schema problems, etc. */
+	if (modify_apply_check_expand(pb, operation, mods, e, ec, &postentry,
+								  &ldap_result_code, &ldap_result_message)) {
 		goto error_return;
 	}
 
@@ -400,15 +478,45 @@ ldbm_back_modify( Slapi_PBlock *pb )
 		}
 	}
 
+	/*
+	 * Grab a copy of the mods and the entry in case the be_txn_preop changes
+	 * the them.  If we have a failure, then we need to reset the mods to their
+	 * their original state;
+	 */
+	mods_original = copy_mods(mods);
+	if ( (original_entry = backentry_dup( ec )) == NULL ) {
+		ldap_result_code= LDAP_OPERATIONS_ERROR;
+		goto error_return;
+	}
+
 	txn.back_txn_txn = NULL; /* ready to create the child transaction */
 	for (retry_count = 0; retry_count < RETRY_TIMES; retry_count++) {
-
-		if (retry_count > 0) {
+		int cache_rc = 0;
+		int new_mod_count = 0;
+		if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) {
 			dblayer_txn_abort(li,&txn);
-			/* txn is no longer valid - reset slapi_txn to the parent */
-			txn.back_txn_txn = NULL;
 			slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
-			LDAPDebug( LDAP_DEBUG_TRACE, "Modify Retrying Transaction\n", 0, 0, 0 );
+			/*
+			 * Since be_txn_preop functions could have modified the entry/mods,
+			 * We need to grab the current mods, free them, and restore the
+			 * originals.  Same thing for the entry.
+			 */
+			
+			slapi_pblock_get(pb, SLAPI_MODIFY_MODS, &mods);
+			ldap_mods_free(mods, 1);
+			slapi_pblock_set(pb, SLAPI_MODIFY_MODS, copy_mods(mods_original));
+			/* ec is not really added to the cache until cache_replace, so we
+			   don't have to worry about the cache here */
+			backentry_free(&ec);
+			slapi_pblock_set( pb, SLAPI_MODIFY_EXISTING_ENTRY, original_entry->ep_entry );
+			ec = original_entry;
+			if ( (original_entry = backentry_dup( ec )) == NULL ) {
+				ldap_result_code= LDAP_OPERATIONS_ERROR;
+				goto error_return;
+			}
+
+			LDAPDebug0Args(LDAP_DEBUG_BACKLDBM,
+			               "Modify Retrying Transaction\n");
 #ifndef LDBM_NO_BACKOFF_DELAY
 			{
 			PRIntervalTime interval;
@@ -432,17 +540,44 @@ ldbm_back_modify( Slapi_PBlock *pb )
 
 		/* call the transaction pre modify plugins just after creating the transaction */
 		if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_PRE_MODIFY_FN))) {
-			LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_PRE_MODIFY_FN plugin "
+			LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_PRE_MODIFY_FN plugin "
 						   "returned error code %d\n", retval );
 			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+			if (!opreturn) {
+				slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval);
+			}
 			goto error_return;
 		}
 
+		/* the mods might have been changed, so get the latest */
+		slapi_pblock_get( pb, SLAPI_MODIFY_MODS, &mods );
+
+		/* make sure the betxnpreop did not alter any of the mods that
+		   had already previously been applied */
+		slapi_mods_done(&smods);
+		slapi_mods_init_byref(&smods,mods);
+		new_mod_count = slapi_mods_get_num_mods(&smods);
+		if (new_mod_count < mod_count) {
+			LDAPDebug2Args( LDAP_DEBUG_ANY, "Error: BE_TXN_PRE_MODIFY plugin has removed "
+							"mods from the original list - mod count was [%d] now [%d] "
+							"mods will not be applied - mods list changes must be done "
+							"in the BE_PRE_MODIFY plugin, not the BE_TXN_PRE_MODIFY\n",
+							mod_count, new_mod_count );
+		} else if (new_mod_count > mod_count) { /* apply the new betxnpremod mods */
+			/* apply the mods, check for syntax, schema problems, etc. */
+			if (modify_apply_check_expand(pb, operation, &mods[mod_count], e, ec, &postentry,
+										  &ldap_result_code, &ldap_result_message)) {
+				goto error_return;
+			}
+		} /* else if new_mod_count == mod_count then betxnpremod plugin did nothing */
+			
 		/*
 		 * Update the ID to Entry index. 
-		 * Note that id2entry_add replaces the entry, so the Entry ID stays the same.
+		 * Note that id2entry_add replaces the entry, so the Entry ID 
+		 * stays the same.
 		 */
-		retval = id2entry_add( be, ec, &txn ); 
+		retval = id2entry_add_ext( be, ec, &txn, 1, &cache_rc ); 
 		if (DB_LOCK_DEADLOCK == retval)
 		{
 			/* Abort and re-try */
@@ -455,7 +590,6 @@ ldbm_back_modify( Slapi_PBlock *pb )
 			MOD_SET_ERROR(ldap_result_code, LDAP_OPERATIONS_ERROR, retry_count);
 			goto error_return;
 		}
-		ec_in_cache = 1;
 		retval = index_add_mods( be, mods, e, ec, &txn );
 		if (DB_LOCK_DEADLOCK == retval)
 		{
@@ -516,7 +650,7 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	}
 	if (retry_count == RETRY_TIMES) {
 		LDAPDebug( LDAP_DEBUG_ANY, "Retry count exceeded in modify\n", 0, 0, 0 );
-	   	ldap_result_code= LDAP_OPERATIONS_ERROR;
+	   	ldap_result_code= LDAP_BUSY;
 		goto error_return;
 	}
 
@@ -533,6 +667,9 @@ ldbm_back_modify( Slapi_PBlock *pb )
 		MOD_SET_ERROR(ldap_result_code, LDAP_OPERATIONS_ERROR, retry_count);
 		goto error_return;
 	}
+	/* lock new entry in cache to prevent usage until we are complete */
+	cache_lock_entry( &inst->inst_cache, ec );
+	ec_in_cache = 1;
 
 	postentry = slapi_entry_dup( ec->ep_entry );
 	slapi_pblock_set( pb, SLAPI_ENTRY_POST_OP, postentry );
@@ -541,6 +678,7 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	ec->ep_entry->e_virtual_watermark = 0;
 
 	/* we must return both e (which has been deleted) and new entry ec */
+	/* cache_replace removes e from the caches */
 	cache_unlock_entry( &inst->inst_cache, e );
 	CACHE_RETURN( &inst->inst_cache, &e );
 	/* 
@@ -555,15 +693,22 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	
 	/* call the transaction post modify plugins just before the commit */
 	if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN))) {
-		LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN plugin "
+		LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN plugin "
 					   "returned error code %d\n", retval );
-		slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		if (!ldap_result_code) {
+			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+		}
+		if (!opreturn) {
+			slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+		}
+		if (!opreturn) {
+			slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval);
+		}
 		goto error_return;
 	}
 
 	retval = dblayer_txn_commit(li,&txn);
 	/* after commit - txn is no longer valid - replace SLAPI_TXN with parent */
-	txn.back_txn_txn = NULL;
 	slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
 	if (0 != retval) {
 		if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
@@ -575,26 +720,12 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	goto common_return;
 
 error_return:
-	if (ec_in_cache)
-	{
-		CACHE_REMOVE( &inst->inst_cache, ec );
-	}
-	else
-	{
-		backentry_free(&ec);
-	}
 	if ( postentry != NULL ) 
 	{
 		slapi_entry_free( postentry );
 		postentry = NULL;
 		slapi_pblock_set( pb, SLAPI_ENTRY_POST_OP, NULL );
 	}
-	
-	if (e!=NULL) {
-	    cache_unlock_entry( &inst->inst_cache, e);
-	    CACHE_RETURN( &inst->inst_cache, &e);
-	}
-
 	if (retval == DB_RUNRECOVERY) {
 	  dblayer_remember_disk_filled(li);
 	  ldbm_nasty("Modify",81,retval);
@@ -603,25 +734,72 @@ error_return:
 
 	if (disk_full) {
 	    rc= return_on_disk_full(li);
-	} else if (ldap_result_code != LDAP_SUCCESS) {
-		if (retry_count > 0) {
+	} else {
+		if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) {
+			/* make sure SLAPI_RESULT_CODE and SLAPI_PLUGIN_OPRETURN are set */
+			int val = 0;
+			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &val);
+			if (!val) {
+				if (!ldap_result_code) {
+					ldap_result_code = LDAP_OPERATIONS_ERROR;
+				}
+				slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+			}
+			slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &val );
+			if (!val) {
+				opreturn = -1;
+				slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, &opreturn );
+			}
+			/* call the transaction post modify plugins just before the abort */
+			/* plugins called before abort should check for the OPRETURN or RESULT_CODE
+			   and skip processing if they don't want do anything - some plugins that
+			   keep track of a counter (usn, dna) may want to "rollback" the counter
+			   in this case */
+			if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN))) {
+				LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN plugin "
+							   "returned error code %d\n", retval );
+				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+				slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+				if (!opreturn) {
+					slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval);
+				}
+			}
+
 			/* It is safer not to abort when the transaction is not started. */
 			dblayer_txn_abort(li,&txn); /* abort crashes in case disk full */
 			/* txn is no longer valid - reset the txn pointer to the parent */
-			txn.back_txn_txn = NULL;
 			slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
 		}
 	    rc= SLAPI_FAIL_GENERAL;
 	}
 
-	
+	/* if ec is in cache, remove it, then add back e if we still have it */
+	if (ec_in_cache) {
+		CACHE_REMOVE( &inst->inst_cache, ec );
+		/* if ec was in cache, e was not - add back e */
+		if (e) {
+			CACHE_ADD( &inst->inst_cache, e, NULL );
+			cache_lock_entry( &inst->inst_cache, e );
+		}
+	}
+
 common_return:
 	slapi_mods_done(&smods);
 	
 	if (ec_in_cache)
 	{
+		cache_unlock_entry( &inst->inst_cache, ec);
 		CACHE_RETURN( &inst->inst_cache, &ec );
 	}
+	else
+	{
+		backentry_free(&ec);
+	}
+	
+	if (e!=NULL) {
+		cache_unlock_entry( &inst->inst_cache, e);
+		CACHE_RETURN( &inst->inst_cache, &e);
+	}
 
 	/* result code could be used in the bepost plugin functions. */
 	slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
@@ -642,8 +820,12 @@ common_return:
 	{
 		slapi_send_ldap_result( pb, ldap_result_code, NULL, ldap_result_message, 0, NULL );
 	}
-	
+
+	/* free our backups */
+	ldap_mods_free(mods_original, 1);
+	backentry_free(&original_entry);
 	slapi_ch_free_string(&errbuf);
+
 	return rc;
 }
 
