@@ -46,11 +46,11 @@
 #include "back-ldbm.h"
 
 static const char *moddn_get_newdn(Slapi_PBlock *pb, Slapi_DN *dn_olddn, Slapi_DN *dn_newrdn, Slapi_DN *dn_newsuperiordn);
-static void moddn_unlock_and_return_entries(backend *be,struct backentry **targetentry, struct backentry **existingentry);
+static void moddn_unlock_and_return_entry(backend *be,struct backentry **targetentry);
 static int moddn_newrdn_mods(Slapi_PBlock *pb, const char *olddn, struct backentry *ec, Slapi_Mods *smods_wsi, int is_repl_op);
 static IDList *moddn_get_children(back_txn *ptxn, Slapi_PBlock *pb, backend *be, struct backentry *parententry, Slapi_DN *parentdn, struct backentry ***child_entries, struct backdn ***child_dns);
 static int moddn_rename_children(back_txn *ptxn, Slapi_PBlock *pb, backend *be, IDList *children, Slapi_DN *dn_parentdn, Slapi_DN *dn_newsuperiordn, struct backentry *child_entries[]);
-static int modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbminfo *li, struct backentry *e, struct backentry *ec, Slapi_Mods *smods1, Slapi_Mods *smods2, Slapi_Mods *smods3);
+static int modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbminfo *li, struct backentry *e, struct backentry **ec, Slapi_Mods *smods1, Slapi_Mods *smods2, Slapi_Mods *smods3, int *e_in_cache, int *ec_in_cache);
 static void mods_remove_nsuniqueid(Slapi_Mods *smods);
 
 #define MOD_SET_ERROR(rc, error, count)                                        \
@@ -68,6 +68,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     struct backentry *e= NULL;
     struct backentry *ec= NULL;
     int ec_in_cache= 0;
+    int e_in_cache= 0;
     back_txn txn;
     back_txnid parent_txn;
     int retval = -1;
@@ -81,7 +82,9 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     char *ldap_result_matcheddn= NULL;
     struct backentry *parententry= NULL;
     struct backentry *newparententry= NULL;
-    struct backentry *existingentry= NULL;
+    struct backentry *original_entry = NULL;
+    struct backentry *original_parent = NULL;
+    struct backentry *original_newparent = NULL;
     modify_context parent_modify_context = {0};
     modify_context newparent_modify_context = {0};
     modify_context ruv_c = {0};
@@ -95,6 +98,9 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     Slapi_DN dn_newrdn;
     Slapi_DN *dn_newsuperiordn = NULL;
     Slapi_DN dn_parentdn;
+    Slapi_DN *orig_dn_newsuperiordn = NULL;
+    Slapi_Entry *target_entry = NULL;
+    Slapi_Entry *original_targetentry = NULL;
     int rc;
     int isroot;
     LDAPMod **mods;
@@ -109,8 +115,12 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     entry_address *old_addr;
     entry_address oldparent_addr;
     entry_address *newsuperior_addr;
-    char ebuf[BUFSIZ];
+    char *original_newrdn = NULL;
     CSN *opcsn = NULL;
+    const char *newdn = NULL;
+    char *newrdn = NULL;
+    int opreturn = 0;
+    int free_modrdn_existing_entry = 0;
 
     /* sdn & parentsdn need to be initialized before "goto *_return" */
     slapi_sdn_init(&dn_newdn);
@@ -135,9 +145,14 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
 
     /* dblayer_txn_init needs to be called before "goto error_return" */
     dblayer_txn_init(li,&txn);
-    /* the calls to search for entries require the parent txn if any
+    /* the calls to perform searches require the parent txn if any
        so set txn to the parent_txn until we begin the child transaction */
-    txn.back_txn_txn = parent_txn;
+    if (parent_txn) {
+        txn.back_txn_txn = parent_txn;
+    } else {
+        parent_txn = txn.back_txn_txn;
+        slapi_pblock_set( pb, SLAPI_TXN, parent_txn );
+    }
 
     if (pb->pb_conn)
     {
@@ -147,6 +162,8 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     inst = (ldbm_instance *) be->be_instance_info;
     {
         char *newrdn/* , *newsuperiordn */;
+        /* newrdn is normalized, bu tno need to be case-ignored as 
+         * it's passed to slapi_sdn_init_normdn_byref */
         slapi_pblock_get( pb, SLAPI_MODRDN_NEWRDN, &newrdn );
         slapi_pblock_get( pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &dn_newsuperiordn );
         slapi_sdn_init_normdn_byref(&dn_newrdn, newrdn);
@@ -212,11 +229,10 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
         /* <new rdn>,<new superior> */
         if(slapi_isbitset_int(rc,SLAPI_RTN_BIT_FETCH_EXISTING_DN_ENTRY))
         {
-            const char *newdn = NULL;
-            char * newrdn = NULL;
-
             /* see if an entry with the new name already exists */
             done_with_pblock_entry(pb,SLAPI_MODRDN_EXISTING_ENTRY); /* Could be through this multiple times */
+            /* newrdn is normalized, bu tno need to be case-ignored as 
+             * it's passed to slapi_sdn_init_normdn_byref */
             slapi_pblock_get(pb, SLAPI_MODRDN_NEWRDN, &newrdn);
             slapi_sdn_init_normdn_byref(&dn_newrdn, newrdn);
             newdn= moddn_get_newdn(pb,sdn, &dn_newrdn, dn_newsuperiordn);
@@ -239,6 +255,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
             {
                 goto error_return;
             }
+            free_modrdn_existing_entry = 1; /* need to free it */
         }
 
         /* <old superior> */
@@ -307,7 +324,15 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
              * Plugin indicated some kind of failure,
              * or that this Operation became a No-Op.
              */
-            slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+            if (!ldap_result_code) {
+                slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+            }
+            if (!opreturn) {
+                slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+            }
+            if (!opreturn) {
+                slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &rc );
+            }
             goto error_return;
         }
         /*
@@ -326,6 +351,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
         ldap_result_code= -1;
         goto error_return; /* error result sent by find_entry2modify() */
     }
+    e_in_cache = 1; /* e is in the cache and locked */
     /* Check that an entry with the same DN doesn't already exist. */
     {
         Slapi_Entry *entry;
@@ -495,17 +521,21 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     }
 
     /* create it in the cache - prevents others from creating it */
-    if (( cache_add_tentative( &inst->inst_cache, ec, NULL ) != 0 ) &&
+    if (( cache_add_tentative( &inst->inst_cache, ec, NULL ) != 0 ) ) {
+        ec_in_cache = 0; /* not in cache */
         /* allow modrdn even if the src dn and dest dn are identical */
-        ( 0 != slapi_sdn_compare((const Slapi_DN *)&dn_newdn,
-                                 (const Slapi_DN *)sdn)) )
-    {
-        /* somebody must've created it between dn2entry() and here */
-        /* JCMREPL - Hmm... we can't permit this to happen...? */
-        ldap_result_code= LDAP_ALREADY_EXISTS;
-        goto error_return;
+        if ( 0 != slapi_sdn_compare((const Slapi_DN *)&dn_newdn,
+                                    (const Slapi_DN *)sdn) ) {
+            /* somebody must've created it between dn2entry() and here */
+            /* JCMREPL - Hmm... we can't permit this to happen...? */
+            ldap_result_code= LDAP_ALREADY_EXISTS;
+            goto error_return;
+        }
+        /* so if the old dn is the same as the new dn, the entry will not be cached
+           until it is replaced with cache_replace */
+    } else {
+        ec_in_cache = 1;
     }
-    ec_in_cache= 1;
 
     /* Build the list of modifications required to the existing entry */
     {
@@ -569,7 +599,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     {
         ldap_result_code= LDAP_OPERATIONS_ERROR;
         LDAPDebug( LDAP_DEBUG_TRACE, "ldbm_modrdn: entry_apply_mods failed for entry %s\n",
-                   escape_string(slapi_entry_get_dn_const(ec->ep_entry), ebuf), 0, 0);
+                   slapi_entry_get_dn_const(ec->ep_entry), 0, 0);
         goto error_return;
     }
 
@@ -582,7 +612,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
         {
             ldap_result_code= LDAP_OPERATIONS_ERROR;
             LDAPDebug( LDAP_DEBUG_TRACE, "ldbm_modrdn: entry_apply_mods_wsi failed for entry %s\n",
-                       escape_string(slapi_entry_get_dn_const(ec->ep_entry), ebuf), 0, 0);
+                       slapi_entry_get_dn_const(ec->ep_entry), 0, 0);
             goto error_return;
         }
     }
@@ -600,7 +630,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
         {
             ldap_result_code= LDAP_OPERATIONS_ERROR;
             LDAPDebug( LDAP_DEBUG_TRACE, "ldbm_modrdn: entry_apply_mods_wsi (operational attributes) failed for entry %s\n",
-                       escape_string(slapi_entry_get_dn_const(ec->ep_entry), ebuf), 0, 0);
+                       slapi_entry_get_dn_const(ec->ep_entry), 0, 0);
             goto error_return;
         }
     }
@@ -694,6 +724,25 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
         }
     }
 
+    /*
+     * make copies of the originals, no need to copy the mods because
+     * we have already copied them
+     */
+    if ( (original_entry = backentry_dup( ec )) == NULL ) {
+        ldap_result_code= LDAP_OPERATIONS_ERROR;
+        goto error_return;
+    }
+    slapi_pblock_get(pb, SLAPI_MODRDN_TARGET_ENTRY, &target_entry);
+    if ( (original_targetentry = slapi_entry_dup(target_entry)) == NULL ) {
+        ldap_result_code= LDAP_OPERATIONS_ERROR;
+        goto error_return;
+    }
+
+    slapi_pblock_get(pb, SLAPI_MODRDN_NEWRDN, &newrdn);
+    original_newrdn = slapi_ch_strdup(newrdn);
+    slapi_pblock_get(pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &dn_newsuperiordn);
+    orig_dn_newsuperiordn = slapi_sdn_dup(dn_newsuperiordn);
+
     /* 
      * So, we believe that no code up till here actually added anything
      * to persistent store. From now on, we're transacted
@@ -701,14 +750,91 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     txn.back_txn_txn = NULL; /* ready to create the child transaction */
     for (retry_count = 0; retry_count < RETRY_TIMES; retry_count++)
     {
-        if (retry_count > 0)
+        if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn))
         {
+            Slapi_Entry *ent = NULL;
+
             dblayer_txn_abort(li,&txn);
             /* txn is no longer valid - reset slapi_txn to the parent */
-            txn.back_txn_txn = NULL;
             slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
+
+            slapi_pblock_get(pb, SLAPI_MODRDN_NEWRDN, &newrdn);
+            slapi_ch_free_string(&newrdn);
+            slapi_pblock_set(pb, SLAPI_MODRDN_NEWRDN, original_newrdn);
+            slapi_sdn_set_normdn_byref(&dn_newrdn, original_newrdn);
+            original_newrdn = slapi_ch_strdup(original_newrdn);
+
+            slapi_pblock_get(pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &dn_newsuperiordn);
+            slapi_sdn_free(&dn_newsuperiordn);
+            slapi_pblock_set(pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, orig_dn_newsuperiordn);
+            orig_dn_newsuperiordn = slapi_sdn_dup(orig_dn_newsuperiordn);
+            if (ec_in_cache) {
+                /* New entry 'ec' is in the entry cache.
+                 * Remove it from the cache . */
+                CACHE_REMOVE(&inst->inst_cache, ec);
+                CACHE_RETURN(&inst->inst_cache, &ec);
+#ifdef DEBUG_CACHE
+                PR_ASSERT(ec == NULL);
+#endif
+                ec_in_cache = 0;
+            } else {
+                backentry_free(&ec);
+            }
+            /* make sure the original entry is back in the cache if it was removed */
+            if (!e_in_cache) {
+                CACHE_ADD(&inst->inst_cache, e, NULL);
+                e_in_cache = 1;
+            }
+            slapi_pblock_get( pb, SLAPI_MODRDN_EXISTING_ENTRY, &ent );
+            if (ent && (ent != original_entry->ep_entry)) {
+                slapi_entry_free(ent);
+                slapi_pblock_set( pb, SLAPI_MODRDN_EXISTING_ENTRY, NULL );
+            }
+            ec = original_entry;
+            if ( (original_entry = backentry_dup( ec )) == NULL ) {
+                ldap_result_code= LDAP_OPERATIONS_ERROR;
+                goto error_return;
+            }
+            slapi_pblock_set( pb, SLAPI_MODRDN_EXISTING_ENTRY, original_entry->ep_entry );
+            free_modrdn_existing_entry = 0; /* owned by original_entry now */
+            if (!ec_in_cache) {
+                /* Put the resetted entry 'ec' into the cache again. */
+                if (cache_add_tentative( &inst->inst_cache, ec, NULL ) != 0) {
+                    ec_in_cache = 0; /* not in cache */
+                    /* allow modrdn even if the src dn and dest dn are identical */
+                    if ( 0 != slapi_sdn_compare((const Slapi_DN *)&dn_newdn,
+                                                (const Slapi_DN *)sdn) ) {
+                        LDAPDebug1Arg(LDAP_DEBUG_ANY, 
+                                      "ldbm_back_modrdn: adding %s to cache failed\n",
+                                      slapi_entry_get_dn_const(ec->ep_entry));
+                        ldap_result_code = LDAP_OPERATIONS_ERROR;
+                        goto error_return;
+                    }
+                    /* so if the old dn is the same as the new dn, the entry will not be cached
+                       until it is replaced with cache_replace */
+                } else {
+                    ec_in_cache = 1;
+                }
+            }
+
+            slapi_pblock_get(pb, SLAPI_MODRDN_TARGET_ENTRY, &target_entry );
+            slapi_entry_free(target_entry);
+            slapi_pblock_set( pb, SLAPI_MODRDN_TARGET_ENTRY, original_targetentry );
+            if ( (original_targetentry = slapi_entry_dup( original_targetentry )) == NULL ) {
+                ldap_result_code= LDAP_OPERATIONS_ERROR;
+                goto error_return;
+            }
+
             /* We're re-trying */
-            LDAPDebug( LDAP_DEBUG_TRACE, "Modrdn Retrying Transaction\n", 0, 0, 0 );
+            LDAPDebug0Args(LDAP_DEBUG_BACKLDBM,
+                           "Modrdn Retrying Transaction\n");
+#ifndef LDBM_NO_BACKOFF_DELAY
+            {
+            PRIntervalTime interval;
+            interval = PR_MillisecondsToInterval(slapi_rand() % 100);
+            DS_Sleep(interval);
+            }
+#endif
         }
         retval = dblayer_txn_begin(li,parent_txn,&txn);
         if (0 != retval) {
@@ -717,21 +843,29 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
             goto error_return;
         }
 
-		/* stash the transaction */
-		slapi_pblock_set(pb, SLAPI_TXN, (void *)txn.back_txn_txn);
+        /* stash the transaction */
+        slapi_pblock_set(pb, SLAPI_TXN, (void *)txn.back_txn_txn);
 
-		/* call the transaction pre modrdn plugins just after creating the transaction */
-		if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_PRE_MODRDN_FN))) {
-			LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_PRE_MODRDN_FN plugin "
-						   "returned error code %d\n", retval );
-			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
-			goto error_return;
-		}
+        /* call the transaction pre modrdn plugins just after creating the transaction */
+        if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_PRE_MODRDN_FN))) {
+            LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_PRE_MODRDN_FN plugin "
+                           "returned error code %d\n", retval );
+            if (!ldap_result_code) {
+                slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+            }
+            if (!opreturn) {
+                slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+            }
+            if (!opreturn) {
+                slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval );
+            }
+            goto error_return;
+        }
 
         /*
          * Update the indexes for the entry.
          */
-        retval = modrdn_rename_entry_update_indexes(&txn, pb, li, e, ec, &smods_generated, &smods_generated_wsi, &smods_operation_wsi);
+        retval = modrdn_rename_entry_update_indexes(&txn, pb, li, e, &ec, &smods_generated, &smods_generated_wsi, &smods_operation_wsi, &e_in_cache, &ec_in_cache);
         if (DB_LOCK_DEADLOCK == retval)
         {
             /* Retry txn */
@@ -762,9 +896,8 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
                     memset(&sv,0,sizeof(Slapi_Value));
                     if ( slapi_rdn2typeval( rdns[i], &type, &sv.bv ) != 0 ) 
                     {
-                        char ebuf[ BUFSIZ ];
                         LDAPDebug( LDAP_DEBUG_ANY, "modrdn: rdn2typeval (%s) failed\n",
-                                   escape_string( rdns[i], ebuf ), 0, 0 );
+                                    rdns[i], 0, 0 );
                         if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
                         MOD_SET_ERROR(ldap_result_code, 
                                       LDAP_OPERATIONS_ERROR, retry_count);
@@ -859,11 +992,14 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
         {
             Slapi_RDN newsrdn;
             slapi_rdn_init_sdn(&newsrdn, (const Slapi_DN *)&dn_newdn);
-            rc = entryrdn_rename_subtree(be, (const Slapi_DN *)sdn, &newsrdn,
-                                         (const Slapi_DN *)dn_newsuperiordn,
-                                         e->ep_id, &txn);
+            retval = entryrdn_rename_subtree(be, (const Slapi_DN *)sdn, &newsrdn,
+                                             (const Slapi_DN *)dn_newsuperiordn,
+                                             e->ep_id, &txn);
             slapi_rdn_done(&newsrdn);
-            if (rc) {
+            if (retval != 0) {
+                if (retval == DB_LOCK_DEADLOCK) continue;
+                if (retval == DB_RUNRECOVERY || LDBM_OS_ERR_IS_DISKFULL(retval))
+                    disk_full = 1;
                 MOD_SET_ERROR(ldap_result_code, 
                               LDAP_OPERATIONS_ERROR, retry_count);
                 goto error_return;
@@ -915,7 +1051,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     {
         /* Failed */
         LDAPDebug( LDAP_DEBUG_ANY, "Retry count exceeded in modrdn\n", 0, 0, 0 );
-        ldap_result_code= LDAP_OPERATIONS_ERROR;
+        ldap_result_code= LDAP_BUSY;
         goto error_return;
     }
 
@@ -933,15 +1069,28 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     slapi_pblock_set( pb, SLAPI_ENTRY_POST_OP, postentry );
     /* call the transaction post modrdn plugins just before the commit */
     if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN))) {
-        LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN plugin "
+        LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN plugin "
                        "returned error code %d\n", retval );
-        slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+        if (!ldap_result_code) {
+            slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+        }
+        if (!ldap_result_code) {
+            LDAPDebug0Args( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN plugin "
+                            "returned error but did not set SLAPI_RESULT_CODE\n" );
+            ldap_result_code = LDAP_OPERATIONS_ERROR;
+            slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+        }
+        if (!opreturn) {
+            slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+        }
+        if (!opreturn) {
+            slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval );
+        }
         goto error_return;
     }
 
     retval = dblayer_txn_commit(li,&txn);
     /* after commit - txn is no longer valid - replace SLAPI_TXN with parent */
-    txn.back_txn_txn = NULL;
     slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
     if (0 != retval)
     {
@@ -995,12 +1144,6 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
     }
 
     retval= 0;
-#if 0 /* this new entry in the cache can be used for future; don't remove it */
-    /* remove from cache so that memory can be freed by cache_return */
-    if (ec_in_cache) {
-        CACHE_REMOVE(&inst->inst_cache, ec);
-    }
-#endif
     goto common_return;
 
 error_return:
@@ -1018,13 +1161,6 @@ error_return:
         CACHE_REMOVE(&inst->inst_dncache, bdn);
         CACHE_RETURN(&inst->inst_dncache, &bdn);
     }
-    if( ec!=NULL ) {
-        if (ec_in_cache) {
-            CACHE_REMOVE(&inst->inst_cache, ec);
-        } else {
-            backentry_free( &ec );
-        }
-    }
     if(children)
     {
         int i = 0;
@@ -1072,10 +1208,39 @@ error_return:
     else 
     {
         /* It is safer not to abort when the transaction is not started. */
-        if (retry_count > 0) {
+        if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn))
+        {
+            /* make sure SLAPI_RESULT_CODE and SLAPI_PLUGIN_OPRETURN are set */
+            int val = 0;
+            slapi_pblock_get(pb, SLAPI_RESULT_CODE, &val);
+            if (!val) {
+                if (!ldap_result_code) {
+                    ldap_result_code = LDAP_OPERATIONS_ERROR;
+                }
+                slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+            }
+            slapi_pblock_get( pb, SLAPI_PLUGIN_OPRETURN, &val );
+            if (!val) {
+                opreturn = -1;
+                slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, &opreturn );
+            }
+            /* call the transaction post modrdn plugins just before the commit */
+            if ((retval = plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN))) {
+                LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN plugin "
+                               "returned error code %d\n", retval );
+                if (!ldap_result_code) {
+                    slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+                }
+                if (!opreturn) {
+                    slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
+                }
+                if (!opreturn) {
+                    slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval );
+                }
+            }
+
             dblayer_txn_abort(li,&txn); /* abort crashes in case disk full */
             /* txn is no longer valid - reset the txn pointer to the parent */
-            txn.back_txn_txn = NULL;
             slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
         }
         retval= SLAPI_FAIL_GENERAL;
@@ -1083,13 +1248,6 @@ error_return:
 
 common_return:
 
-    /* Free up the resource we don't need any more */
-    if(ec_in_cache) {
-        CACHE_RETURN( &inst->inst_cache, &ec );
-    }
-
-    moddn_unlock_and_return_entries(be,&e,&existingentry);
-
     /* result code could be used in the bepost plugin functions. */
     slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
     /*
@@ -1097,6 +1255,32 @@ common_return:
      */
     plugin_call_plugins (pb, SLAPI_PLUGIN_BE_POST_MODRDN_FN);
 
+    /* Free up the resource we don't need any more */
+    if (ec) {
+        /* remove the new entry from the cache if the op failed -
+           otherwise, leave it in */
+        if (ec_in_cache && retval) {
+            CACHE_REMOVE( &inst->inst_cache, ec );
+        }
+        if (ec_in_cache) {
+            CACHE_RETURN( &inst->inst_cache, &ec );
+        } else {
+            backentry_free( &ec );
+        }
+        ec = NULL;
+        ec_in_cache = 0;
+    }
+
+    /* put e back in the cache if the modrdn failed */
+    if (e) {
+        if (!e_in_cache && retval) {
+            CACHE_ADD(&inst->inst_cache, e, NULL);
+            e_in_cache = 1;
+        }
+    }
+
+    moddn_unlock_and_return_entry(be,&e);
+
     if (ruv_c_init) {
         modify_term(&ruv_c, be);
     }
@@ -1119,10 +1303,21 @@ common_return:
     slapi_sdn_done(&dn_parentdn);
     modify_term(&parent_modify_context,be);
     modify_term(&newparent_modify_context,be);
-    done_with_pblock_entry(pb,SLAPI_MODRDN_EXISTING_ENTRY);
+    if (free_modrdn_existing_entry) {
+        done_with_pblock_entry(pb,SLAPI_MODRDN_EXISTING_ENTRY);
+    } else { /* owned by original_entry */
+        slapi_pblock_set(pb, SLAPI_MODRDN_EXISTING_ENTRY, NULL);
+    }
     done_with_pblock_entry(pb,SLAPI_MODRDN_PARENT_ENTRY);
     done_with_pblock_entry(pb,SLAPI_MODRDN_NEWPARENT_ENTRY);
     done_with_pblock_entry(pb,SLAPI_MODRDN_TARGET_ENTRY);
+    slapi_ch_free_string(&original_newrdn);
+    slapi_sdn_free(&orig_dn_newsuperiordn);
+    backentry_free(&original_entry);
+    backentry_free(&original_parent);
+    backentry_free(&original_newparent);
+    slapi_entry_free(original_targetentry);
+
     if(dblock_acquired)
     {
         dblayer_unlock_backend(be);
@@ -1185,10 +1380,9 @@ moddn_get_newdn(Slapi_PBlock *pb, Slapi_DN *dn_olddn, Slapi_DN *dn_newrdn, Slapi
  * Return the entries to the cache.
  */
 static void
-moddn_unlock_and_return_entries(
+moddn_unlock_and_return_entry(
     backend *be,
-    struct backentry **targetentry, 
-    struct backentry **existingentry)
+    struct backentry **targetentry)
     {
         ldbm_instance *inst = (ldbm_instance *) be->be_instance_info;
 
@@ -1198,10 +1392,6 @@ moddn_unlock_and_return_entries(
             CACHE_RETURN( &inst->inst_cache, targetentry );
             *targetentry= NULL;
         }
-        if ( *existingentry!=NULL ) {
-            CACHE_RETURN( &inst->inst_cache, existingentry );
-            *existingentry= NULL;
-        }
     }
 
 
@@ -1272,7 +1462,6 @@ moddn_rdn_add_needed (
 static int
 moddn_newrdn_mods(Slapi_PBlock *pb, const char *olddn, struct backentry *ec, Slapi_Mods *smods_wsi, int is_repl_op)
 {
-    char ebuf[BUFSIZ];
     char **rdns = NULL;
     char **dns = NULL;
     int deleteoldrdn;
@@ -1343,7 +1532,7 @@ moddn_newrdn_mods(Slapi_PBlock *pb, const char *olddn, struct backentry *ec, Sla
     if ( baddn || badrdn )
     {
         LDAPDebug( LDAP_DEBUG_TRACE, "moddn_newrdn_mods failed: olddn=%s baddn=%d badrdn=%d\n",
-                   escape_string(olddn, ebuf), baddn, badrdn);
+                   olddn, baddn, badrdn);
         return LDAP_OPERATIONS_ERROR;
     }
     }
@@ -1372,8 +1561,8 @@ moddn_newrdn_mods(Slapi_PBlock *pb, const char *olddn, struct backentry *ec, Sla
     }
     else
     {
-    LDAPDebug( LDAP_DEBUG_TRACE, "moddn_newrdn_mods failed: could not parse new rdn %s\n",
-           escape_string(newrdn, ebuf), 0, 0);
+        LDAPDebug( LDAP_DEBUG_TRACE, "moddn_newrdn_mods failed: could not parse new rdn %s\n",
+                   newrdn, 0, 0);
         return LDAP_OPERATIONS_ERROR;
     }
     
@@ -1400,7 +1589,7 @@ mods_remove_nsuniqueid(Slapi_Mods *smods)
  * mods contains the list of attribute change made.
  */
 static int
-modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbminfo *li, struct backentry *e, struct backentry *ec, Slapi_Mods *smods1, Slapi_Mods *smods2, Slapi_Mods *smods3)
+modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbminfo *li, struct backentry *e, struct backentry **ec, Slapi_Mods *smods1, Slapi_Mods *smods2, Slapi_Mods *smods3, int *e_in_cache, int *ec_in_cache)
 {
     backend *be;
     ldbm_instance *inst;
@@ -1408,36 +1597,41 @@ modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbm
     char *msg;
     Slapi_Operation *operation;
     int is_ruv = 0;                 /* True if the current entry is RUV */
+    int orig_ec_in_cache = 0;
 
     slapi_pblock_get( pb, SLAPI_BACKEND, &be );
     slapi_pblock_get( pb, SLAPI_OPERATION, &operation );
     is_ruv = operation_is_flag_set(operation, OP_FLAG_REPL_RUV);
     inst = (ldbm_instance *) be->be_instance_info;
 
+    orig_ec_in_cache = *ec_in_cache;
     /*
      * Update the ID to Entry index. 
      * Note that id2entry_add replaces the entry, so the Entry ID stays the same.
      */
-    retval = id2entry_add( be, ec, ptxn );
+    retval = id2entry_add( be, *ec, ptxn );
     if (DB_LOCK_DEADLOCK == retval)
     {
         /* Retry txn */
+        LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "modrdn_rename_entry_update_indexes: id2entry_add deadlock\n" );
         goto error_return;
     }
     if (retval != 0)
     {
-        LDAPDebug( LDAP_DEBUG_ANY, "id2entry_add failed, err=%d %s\n", retval, (msg = dblayer_strerror( retval )) ? msg : "", 0 );
+        LDAPDebug( LDAP_DEBUG_ANY, "modrdn_rename_entry_update_indexes: id2entry_add failed, err=%d %s\n", retval, (msg = dblayer_strerror( retval )) ? msg : "", 0 );
         goto error_return;
     }
+    *ec_in_cache = 1; /* id2entry_add adds to cache if not already in */
     if(smods1!=NULL && slapi_mods_get_num_mods(smods1)>0)
     {
         /*
          * update the indexes: lastmod, rdn, etc.
          */
-        retval = index_add_mods( be, slapi_mods_get_ldapmods_byref(smods1), e, ec, ptxn );
+        retval = index_add_mods( be, slapi_mods_get_ldapmods_byref(smods1), e, *ec, ptxn );
         if (DB_LOCK_DEADLOCK == retval)
         {
             /* Retry txn */
+            LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "modrdn_rename_entry_update_indexes: index_add_mods1 deadlock\n" );
             goto error_return;
         }
         if (retval != 0)
@@ -1457,10 +1651,11 @@ modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbm
         /*
          * update the indexes: lastmod, rdn, etc.
          */
-        retval = index_add_mods( be, slapi_mods_get_ldapmods_byref(smods2), e, ec, ptxn );
+        retval = index_add_mods( be, slapi_mods_get_ldapmods_byref(smods2), e, *ec, ptxn );
         if (DB_LOCK_DEADLOCK == retval)
         {
             /* Retry txn */
+            LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "modrdn_rename_entry_update_indexes: index_add_mods2 deadlock\n" );
             goto error_return;
         }
         if (retval != 0)
@@ -1474,10 +1669,11 @@ modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbm
         /*
          * update the indexes: lastmod, rdn, etc.
          */
-        retval = index_add_mods( be, slapi_mods_get_ldapmods_byref(smods3), e, ec, ptxn );
+        retval = index_add_mods( be, slapi_mods_get_ldapmods_byref(smods3), e, *ec, ptxn );
         if (DB_LOCK_DEADLOCK == retval)
         {
             /* Retry txn */
+            LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "modrdn_rename_entry_update_indexes: index_add_mods3 deadlock\n" );
             goto error_return;
         }
         if (retval != 0)
@@ -1493,10 +1689,11 @@ modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbm
      */
     if (!is_ruv)
     {
-        retval= vlv_update_all_indexes(ptxn, be, pb, e, ec);
+        retval= vlv_update_all_indexes(ptxn, be, pb, e, *ec);
         if (DB_LOCK_DEADLOCK == retval)
         {
             /* Abort and re-try */
+            LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "modrdn_rename_entry_update_indexes: vlv_update_all_indexes deadlock\n" );
             goto error_return;
         }
         if (retval != 0)
@@ -1505,9 +1702,18 @@ modrdn_rename_entry_update_indexes(back_txn *ptxn, Slapi_PBlock *pb, struct ldbm
             goto error_return;
         }
     }
-    if (cache_replace( &inst->inst_cache, e, ec ) != 0 ) {
+    if (cache_replace( &inst->inst_cache, e, *ec ) != 0 ) {
+        LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "modrdn_rename_entry_update_indexes cache_replace failed\n");
         retval= -1;
         goto error_return;
+    } else {
+        *e_in_cache = 0; /* e un-cached */
+    }
+    if (orig_ec_in_cache) {
+        /* ec was already added to the cache via cache_add_tentative (to reserve its spot in the cache)
+           and/or id2entry_add - so it already had one refcount - cache_replace adds another refcount -
+           drop the extra ref added by cache_replace */
+        CACHE_RETURN( &inst->inst_cache, ec );
     }
 error_return:
     return retval;
@@ -1519,7 +1725,7 @@ moddn_rename_child_entry(
     Slapi_PBlock *pb, 
     struct ldbminfo *li, 
     struct backentry *e, 
-    struct backentry *ec, 
+    struct backentry **ec, 
     int parentdncomps, 
     char **newsuperiordns, 
     int newsuperiordncomps,
@@ -1543,8 +1749,10 @@ moddn_rename_child_entry(
     int olddncomps= 0;
     int need= 1; /* For the '\0' */
     int i;
+    int e_in_cache = 1;
+    int ec_in_cache = 0;
 
-    olddn = slapi_entry_get_dn(ec->ep_entry);
+    olddn = slapi_entry_get_dn((*ec)->ep_entry);
     if (NULL == olddn) {
         return retval;
     }
@@ -1579,9 +1787,9 @@ moddn_rename_child_entry(
         }
     }
     slapi_ldap_value_free( olddns );
-    slapi_entry_set_dn( ec->ep_entry, newdn );
+    slapi_entry_set_dn( (*ec)->ep_entry, newdn );
     /* add the entrydn operational attributes */
-    add_update_entrydn_operational_attributes (ec);
+    add_update_entrydn_operational_attributes (*ec);
 
     /*
      * Update the DN CSN of the entry.
@@ -1599,13 +1807,14 @@ moddn_rename_child_entry(
         slapi_mods_add( &smods, LDAP_MOD_DELETE, LDBM_ENTRYDN_STR, 
                     strlen( backentry_get_ndn(e) ), backentry_get_ndn(e) );
         slapi_mods_add( &smods, LDAP_MOD_REPLACE, LDBM_ENTRYDN_STR, 
-                    strlen( backentry_get_ndn(ec) ), backentry_get_ndn(ec) );
+                    strlen( backentry_get_ndn(*ec) ), backentry_get_ndn(*ec) );
         smodsp = &smods;
         /*
          * Update all the indexes.
          */
         retval = modrdn_rename_entry_update_indexes(ptxn, pb, li, e, ec,
-                                                    smodsp, NULL, NULL);
+                                                    smodsp, NULL, NULL,
+                                                    &e_in_cache, &ec_in_cache);
         /* JCMREPL - Should the children get updated modifiersname and lastmodifiedtime? */
         slapi_mods_done(&smods);
     }
@@ -1682,7 +1891,7 @@ moddn_rename_children(
     opcsn = operation_get_csn (operation);
     for (i=0,retval=0; retval == 0 && child_entries[i] && child_entry_copies[i]; i++) {
         retval = moddn_rename_child_entry(ptxn, pb, li, child_entries[i],
-                                         child_entry_copies[i], parentdncomps,
+                                         &child_entry_copies[i], parentdncomps,
                                          newsuperiordns, newsuperiordncomps,
                                          opcsn );
     }
