@@ -95,7 +95,7 @@ compute_lookthrough_limit( Slapi_PBlock *pb, struct ldbminfo *li )
         }
     }
 
-    if (op && (op->o_flags & OP_FLAG_PAGED_RESULTS)) {
+    if (op_is_pagedresults(op)) {
         if ( slapi_reslimit_get_integer_limit( conn,
                 li->li_reslimit_pagedlookthrough_handle, &limit )
                 != SLAPI_RESLIMIT_STATUS_SUCCESS ) {
@@ -127,7 +127,7 @@ compute_allids_limit( Slapi_PBlock *pb, struct ldbminfo *li )
         limit = li->li_allidsthreshold;
         PR_Unlock(li->li_config_mutex);
     }
-    if (op && (op->o_flags & OP_FLAG_PAGED_RESULTS)) {
+    if (op_is_pagedresults(op)) {
         if ( slapi_reslimit_get_integer_limit( conn,
                 li->li_reslimit_pagedallids_handle, &limit )
              != SLAPI_RESLIMIT_STATUS_SUCCESS ) {
@@ -184,8 +184,10 @@ ldbm_back_search_cleanup(Slapi_PBlock *pb,
         back_search_result_set *sr = NULL;
         slapi_pblock_get( pb, SLAPI_SEARCH_RESULT_SET, &sr );
         if ( (NULL != sr) && (function_result != 0) ) {
+            int pr_idx = -1;
+            slapi_pblock_get( pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx );
             /* in case paged results, clean up the conn */
-            pagedresults_set_search_result(pb->pb_conn, NULL, 0);
+            pagedresults_set_search_result( pb->pb_conn, NULL, 0, pr_idx );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, NULL );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET_SIZE_ESTIMATE, &estimate );
             delete_search_result_set(&sr);
@@ -204,12 +206,11 @@ ldbm_search_compile_filter(Slapi_Filter *f, void *arg)
     int rc = SLAPI_FILTER_SCAN_CONTINUE;
     if (f->f_choice == LDAP_FILTER_SUBSTRINGS) {
         char pat[BUFSIZ];
-        char *p, *end, *tmpbuf, *bigpat = NULL;
+        char *p, *end, *bigpat = NULL;
         size_t size = 0;
         Slapi_Regex *re = NULL;
         const char *re_result = NULL;
         int i = 0;
-        char ebuf[BUFSIZ];
 
         PR_ASSERT(NULL == f->f_un.f_un_sub.sf_private);
         /*
@@ -258,13 +259,13 @@ ldbm_search_compile_filter(Slapi_Filter *f, void *arg)
 
         /* compile the regex */
         p = bigpat ? bigpat : pat;
-        tmpbuf = NULL;
         re = slapi_re_comp(p, &re_result);
         if (NULL == re) {
             LDAPDebug(LDAP_DEBUG_ANY, "ldbm_search_compile_filter: re_comp (%s) failed (%s): %s\n",
                       pat, p, re_result?re_result:"unknown" );
             rc = SLAPI_FILTER_SCAN_ERROR;
         } else {
+            char ebuf[BUFSIZ];
             LDAPDebug(LDAP_DEBUG_TRACE, "ldbm_search_compile_filter: re_comp (%s)\n",
                       escape_string(p, ebuf), 0, 0);
             f->f_un.f_un_sub.sf_private = (void *)re;
@@ -353,6 +354,11 @@ ldbm_back_search( Slapi_PBlock *pb )
     slapi_pblock_get( pb, SLAPI_BACKEND_COUNT, &backend_count );
     slapi_pblock_get( pb, SLAPI_TXN, &txn.back_txn_txn );
 
+    if ( !txn.back_txn_txn ) {
+        dblayer_txn_init( li, &txn );
+        slapi_pblock_set( pb, SLAPI_TXN, txn.back_txn_txn );
+    }
+
     inst = (ldbm_instance *) be->be_instance_info;
 
     if (NULL == basesdn) {
@@ -824,6 +830,7 @@ ldbm_back_search( Slapi_PBlock *pb )
     if ( NULL != candidates && ALLIDS( candidates )) {
         unsigned int opnote = SLAPI_OP_NOTE_UNINDEXED;
         int ri = 0;
+        int pr_idx = -1;
 
         /*
          * Return error if nsslapd-require-index is set and
@@ -845,7 +852,8 @@ ldbm_back_search( Slapi_PBlock *pb )
         }
 
         slapi_pblock_set( pb, SLAPI_OPERATION_NOTES, &opnote );
-        pagedresults_set_unindexed( pb->pb_conn );
+        slapi_pblock_get( pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx );
+        pagedresults_set_unindexed( pb->pb_conn, pr_idx );
     }
 
     sr->sr_candidates = candidates;
@@ -1319,7 +1327,6 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
     int                    managedsait;
     Slapi_Attr             *attr;
     Slapi_Filter           *filter;
-    const char             *base;
     back_search_result_set *sr;
     ID                     id;
     struct backentry       *e;
@@ -1333,6 +1340,7 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
     int                    rc = 0; 
     int                    estimate = 0; /* estimated search result count */
     back_txn               txn = {NULL};
+    int                    pr_idx = -1;
 
     slapi_pblock_get( pb, SLAPI_BACKEND, &be );
     slapi_pblock_get( pb, SLAPI_PLUGIN_PRIVATE, &li );
@@ -1349,6 +1357,12 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
     slapi_pblock_get( pb, SLAPI_TARGET_UNIQUEID, &target_uniqueid );
     slapi_pblock_get( pb, SLAPI_SEARCH_RESULT_SET, &sr );
     slapi_pblock_get( pb, SLAPI_TXN, &txn.back_txn_txn );
+    slapi_pblock_get( pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx );
+
+    if ( !txn.back_txn_txn ) {
+        dblayer_txn_init( li, &txn );
+        slapi_pblock_set( pb, SLAPI_TXN, txn.back_txn_txn );
+    }
 
     if (NULL == sr) {
         goto bail;
@@ -1365,7 +1379,6 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
                                "Null target DN", 0, NULL );
         return( -1 );
     }
-    base = slapi_sdn_get_dn(basesdn);
 
     if (sr->sr_current_sizelimit >= 0) {
         /* 
@@ -1381,10 +1394,6 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
     
     inst = (ldbm_instance *) be->be_instance_info;
 
-    if (NULL == basesdn) {
-        basesdn = slapi_sdn_new_normdn_byref(base);
-        slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, basesdn);
-    }
     /* Return to the cache the entry we handed out last time */
     /* If we are using the extension, the front end will tell
      * us when to do this so we don't do it now */
@@ -1414,7 +1423,7 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
         if ( slapi_op_abandoned( pb ) || (NULL == sr) )
         {
             /* in case paged results, clean up the conn */
-            pagedresults_set_search_result(pb->pb_conn, NULL, 0);
+            pagedresults_set_search_result( pb->pb_conn, NULL, 0, pr_idx );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, NULL );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET_SIZE_ESTIMATE, &estimate );
             if ( use_extension ) {
@@ -1431,7 +1440,7 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
         if ( tlimit != -1 && curtime > stoptime )
         {
             /* in case paged results, clean up the conn */
-            pagedresults_set_search_result(pb->pb_conn, NULL, 0);
+            pagedresults_set_search_result( pb->pb_conn, NULL, 0, pr_idx );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, NULL );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET_SIZE_ESTIMATE, &estimate );
             if ( use_extension ) {
@@ -1448,7 +1457,7 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
         if ( llimit != -1 && sr->sr_lookthroughcount >= llimit )
         {
             /* in case paged results, clean up the conn */
-            pagedresults_set_search_result(pb->pb_conn, NULL, 0);
+            pagedresults_set_search_result( pb->pb_conn, NULL, 0, pr_idx );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, NULL );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET_SIZE_ESTIMATE, &estimate );
             if ( use_extension ) {
@@ -1468,7 +1477,7 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
             /* No more entries */
             /* destroy back_search_result_set */
             /* in case paged results, clean up the conn */
-            pagedresults_set_search_result(pb->pb_conn, NULL, 0);
+            pagedresults_set_search_result( pb->pb_conn, NULL, 0, pr_idx );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, NULL );
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET_SIZE_ESTIMATE, &estimate );
             if ( use_extension ) {
@@ -1526,8 +1535,7 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
             Slapi_Value **refs= attr_get_present_values(attr);
             if ( refs == NULL || refs[0] == NULL )
             {
-                char ebuf[ BUFSIZ ];
-                LDAPDebug( LDAP_DEBUG_ANY, "null ref in (%s)\n", escape_string( backentry_get_ndn(e), ebuf ), 0, 0 );
+                LDAPDebug( LDAP_DEBUG_ANY, "null ref in (%s)\n", backentry_get_ndn(e), 0, 0 );
             }
             else if ( slapi_sdn_scope_test( backentry_get_sdn(e), basesdn, scope ))
             {
@@ -1588,8 +1596,7 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
                               ACL_CHECK_FLAG );
                       if (filter_test != ft_rc) {
                           /* Oops ! This means that we thought we could bypass the filter test, but noooo... */
-                          char ebuf[ BUFSIZ ];
-                          LDAPDebug( LDAP_DEBUG_ANY, "Filter bypass ERROR on entry %s\n", escape_string( backentry_get_ndn(e), ebuf ), 0, 0 );
+                          LDAPDebug( LDAP_DEBUG_ANY, "Filter bypass ERROR on entry %s\n", backentry_get_ndn(e), 0, 0 );
                           filter_test = ft_rc; /* Fix the error */
                       }
                   }
@@ -1611,7 +1618,8 @@ ldbm_back_next_search_entry_ext( Slapi_PBlock *pb, int use_extension )
                      if ( --slimit < 0 ) {
                          CACHE_RETURN( &inst->inst_cache, &e );
                          /* in case paged results, clean up the conn */
-                         pagedresults_set_search_result(pb->pb_conn, NULL, 0);
+                         pagedresults_set_search_result( pb->pb_conn, NULL,
+                                                         0, pr_idx);
                          slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, NULL );
                          slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET_SIZE_ESTIMATE, &estimate );
                          delete_search_result_set( &sr );
