@@ -53,7 +53,7 @@ void ldbm_nasty(const char* str, int c, int err)
     char buffer[200];
     if (err == DB_LOCK_DEADLOCK) {
         PR_snprintf(buffer,200,"%s WARNING %d",str,c);
-        LDAPDebug(LDAP_DEBUG_TRACE,"%s, err=%d %s\n",
+        LDAPDebug(LDAP_DEBUG_BACKLDBM,"%s, err=%d %s\n",
                   buffer,err,(msg = dblayer_strerror( err )) ? msg : "");
    } else if (err == DB_RUNRECOVERY) {
         LDAPDebug2Args(LDAP_DEBUG_ANY, "FATAL ERROR at %s (%d); "
