@@ -260,7 +260,6 @@ vlv_init_index_entry(Slapi_PBlock *pb, Slapi_Entry* entryBefore, Slapi_Entry* en
     struct vlvIndex* newVlvIndex;
     struct vlvSearch* pSearch;
     Slapi_Backend *be= ((ldbm_instance*)arg)->inst_be;
-    char ebuf[BUFSIZ];
 	
     if(be!=NULL)
     {
@@ -272,7 +271,7 @@ vlv_init_index_entry(Slapi_PBlock *pb, Slapi_Entry* entryBefore, Slapi_Entry* en
         pSearch= vlvSearch_finddn((struct vlvSearch *)be->vlvSearchList, &parentdn);
 		if (pSearch == NULL) { 
 			LDAPDebug( LDAP_DEBUG_ANY, "Parent doesn't exist for entry %s.\n",
-				escape_string(slapi_entry_get_dn(entryBefore), ebuf), 0, 0); 
+				slapi_entry_get_dn(entryBefore), 0, 0); 
 			vlvIndex_delete(&newVlvIndex);
 		} 
 		else { 
@@ -1159,9 +1158,11 @@ vlv_search_build_candidate_list(Slapi_PBlock *pb, const Slapi_DN *base, int *vlv
 	slapi_rwlock_rdlock(be->vlvSearchList_lock);
 	if((pi=vlv_find_search(be, base, scope, fstr, sort_control)) == NULL) {
 	    unsigned int opnote = SLAPI_OP_NOTE_UNINDEXED;
+	    int pr_idx = -1;
+	    slapi_pblock_get( pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx );
 	    slapi_rwlock_unlock(be->vlvSearchList_lock);
 	    slapi_pblock_set( pb, SLAPI_OPERATION_NOTES, &opnote );
-	    pagedresults_set_unindexed( pb->pb_conn );
+	    pagedresults_set_unindexed( pb->pb_conn, pr_idx );
 	    rc = VLV_FIND_SEARCH_FAILED;
 	} else if((*vlv_rc=vlvIndex_accessallowed(pi, pb)) != LDAP_SUCCESS) {
 	    slapi_rwlock_unlock(be->vlvSearchList_lock);
