@@ -241,13 +241,14 @@ slapd_bootstrap_config(const char *configdir)
 			char schemacheck[BUFSIZ];
 			char syntaxcheck[BUFSIZ];
 			char syntaxlogging[BUFSIZ];
+			char plugintracking[BUFSIZ];
 			char dn_validate_strict[BUFSIZ];
 			Slapi_DN plug_dn;
 
 			workpath[0] = loglevel[0] = maxdescriptors[0] = '\0';
 			val[0] = logenabled[0] = schemacheck[0] = syntaxcheck[0] = '\0';
 			syntaxlogging[0] = _localuser[0] = '\0';
-			dn_validate_strict[0] = '\0';
+			plugintracking [0] = dn_validate_strict[0] = '\0';
 
 			/* Convert LDIF to entry structures */
 			slapi_sdn_init_ndn_byref(&plug_dn, PLUGIN_BASE_DN);
@@ -464,6 +465,20 @@ slapd_bootstrap_config(const char *configdir)
 					}
 				}
 
+				/* see if we need to enable plugin binddn tracking */
+				if (!plugintracking[0] &&
+					entry_has_attr_and_value(e, CONFIG_PLUGIN_BINDDN_TRACKING_ATTRIBUTE,
+											 plugintracking, sizeof(plugintracking)))
+				{
+					if (config_set_plugin_tracking(CONFIG_PLUGIN_BINDDN_TRACKING_ATTRIBUTE,
+							plugintracking, errorbuf, CONFIG_APPLY)
+								!= LDAP_SUCCESS)
+					{
+						LDAPDebug(LDAP_DEBUG_ANY, "%s: %s: %s\n", configfile,
+								CONFIG_PLUGIN_BINDDN_TRACKING_ATTRIBUTE, errorbuf);
+					}
+				}
+
 				/* see if we need to enable syntax checking */
 				if (!syntaxcheck[0] &&
 				    entry_has_attr_and_value(e, CONFIG_SYNTAXCHECK_ATTRIBUTE,
