@@ -115,6 +115,7 @@ ignore_attr_type(const char *attr_type)
 		 (strcasecmp (attr_type, "aci") == 0) ||
 		 (strcasecmp (attr_type, "objectclass") == 0) ||
 		 (strcasecmp (attr_type, "numsubordinates") == 0) ||
+		 (strcasecmp (attr_type, "internalModifiersname") == 0) ||
 		 (strcasecmp (attr_type, "modifytimestamp") == 0) ||
 		 (strcasecmp (attr_type, "modifiersname") == 0)) {
 		return 1;
@@ -359,11 +360,9 @@ load_plugin_entry(Slapi_PBlock *pb, Slapi_Entry* e, Slapi_Entry* ignored, int *r
 		 */
 		if(retval)
 		{
-			char dnbuf[ BUFSIZ ];
-
 			slapi_log_error( SLAPI_LOG_FATAL, NULL,
 					"Unable to load plugin \"%s\"\n",
-					escape_string( slapi_entry_get_dn_const( e ), dnbuf ));
+					slapi_entry_get_dn_const( e ));
 			exit(1);
 		}
 	}
