@@ -117,6 +117,8 @@ connection_done(Connection *conn)
 	{
 		PR_DestroyLock(conn->c_pdumutex);
 	}
+	/* PAGED_RESULTS */
+	pagedresults_cleanup_all(conn, 0);
 }
 
 /*
@@ -2092,7 +2094,7 @@ void connection_enter_leave_turbo(Connection *conn, int current_turbo_flag, int
 	PR_Lock(conn->c_mutex);
 	/* We can already be in turbo mode, or not */
 	current_mode = current_turbo_flag;
-	if (conn->c_search_result_set) {
+	if (pagedresults_in_use(conn)) {
 		/* PAGED_RESULTS does not need turbo mode */
 		new_mode = 0;
 	} else if (conn->c_private->operation_rate == 0) {
@@ -2634,6 +2636,8 @@ op_copy_identity(Connection *conn, Operation *op)
     } else {
 	    slapi_sdn_set_dn_byval(&op->o_sdn,conn->c_dn);
         op->o_authtype = slapi_ch_strdup(conn->c_authtype);
+        /* set the thread data bind dn index */
+        slapi_td_set_dn(slapi_ch_strdup(conn->c_dn));
     }
     /* XXX We should also copy c_client_cert into *op here; it's
      * part of the authorization identity.  The operation's copy
@@ -2774,7 +2778,9 @@ disconnect_server_nomutex( Connection *conn, PRUint64 opconnid, int opid, PRErro
 
 	conn->c_gettingber = 0;
 	connection_abandon_operations( conn );
-	conn->c_timelimit = 0; /* needed here to ensure simple paged results timeout properly and don't impact subsequent ops */
+	/* needed here to ensure simple paged results timeout properly and 
+	 * don't impact subsequent ops */
+	pagedresults_reset_timedout(conn);
 
 	if (! config_check_referral_mode()) {
 	    /*
