@@ -99,6 +99,24 @@
 #if !defined(USE_OPENLDAP)
 #include <ldap_ssl.h>
 #include <ldappr.h>
+#else
+/* need mutex around ldap_initialize - see https://fedorahosted.org/389/ticket/348 */
+static PRCallOnceType ol_init_callOnce = {0,0};
+static PRLock *ol_init_lock = NULL;
+
+static PRStatus
+internal_ol_init_init(void)
+{
+    PR_ASSERT(NULL == ol_init_lock);
+    if ((ol_init_lock = PR_NewLock()) == NULL) {
+        PRErrorCode errorCode = PR_GetError();
+        slapi_log_error(SLAPI_LOG_FATAL, "internal_ol_init_init", "PR_NewLock failed %d:%s\n",
+                        errorCode, slapd_pr_strerror(errorCode));
+        return PR_FAILURE;
+    }
+
+    return PR_SUCCESS;
+}
 #endif
 
 /* the server depends on the old, deprecated ldap_explode behavior which openldap
@@ -108,6 +126,10 @@ static char **mozldap_ldap_explode( const char *dn, const int notypes, const int
 static char **mozldap_ldap_explode_dn( const char *dn, const int notypes );
 static char **mozldap_ldap_explode_rdn( const char *rdn, const int notypes );
 
+#ifdef HAVE_KRB5
+static void clear_krb5_ccache();
+#endif
+
 #ifdef MEMPOOL_EXPERIMENTAL
 void _free_wrapper(void *ptr)
 {
@@ -737,7 +759,16 @@ slapi_ldap_init_ext(
 
 #if defined(USE_OPENLDAP)
     if (ldapurl) {
+	if (PR_SUCCESS != PR_CallOnce(&ol_init_callOnce, internal_ol_init_init)) {
+	    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_init_ext",
+			    "Could not perform internal ol_init init\n");
+	    rc = -1;
+	    goto done;
+	}
+
+	PR_Lock(ol_init_lock);
 	rc = ldap_initialize(&ld, ldapurl);
+	PR_Unlock(ol_init_lock);
 	if (rc) {
 	    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_init_ext",
 			    "Could not initialize LDAP connection to [%s]: %d:%s\n",
@@ -751,7 +782,16 @@ slapi_ldap_init_ext(
 	} else { /* host port */
 	    makeurl = convert_to_openldap_uri(hostname, port, (secure == 1 ? "ldaps" : "ldap"));
 	}
+	if (PR_SUCCESS != PR_CallOnce(&ol_init_callOnce, internal_ol_init_init)) {
+	    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_init_ext",
+			    "Could not perform internal ol_init init\n");
+	    rc = -1;
+	    goto done;
+	}
+
+	PR_Lock(ol_init_lock);
 	rc = ldap_initialize(&ld, makeurl);
+	PR_Unlock(ol_init_lock);
 	if (rc) {
 	    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_init_ext",
 			    "Could not initialize LDAP connection to [%s]: %d:%s\n",
@@ -951,6 +991,7 @@ slapi_ldap_bind(
 )
 {
     int rc = LDAP_SUCCESS;
+    int err;
     LDAPControl **clientctrls = NULL;
     int secure = 0;
     struct berval bvcreds = {0, NULL};
@@ -1071,21 +1112,27 @@ slapi_ldap_bind(
 				mech ? mech : "SIMPLE");
 		goto done;
 	    }
-	    /* if we got here, we were able to read success result */
-	    /* Get the controls sent by the server if requested */
-	    if (returnedctrls) {
-                if ((rc = ldap_parse_result(ld, result, &rc, NULL, NULL,
-					    NULL, returnedctrls,
-					    0)) != LDAP_SUCCESS) {
-		    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
-				    "Error: could not bind id "
-				    "[%s] mech [%s]: error %d (%s) errno %d (%s)\n",
-				    bindid ? bindid : "(anon)",
-				    mech ? mech : "SIMPLE",
-				    rc, ldap_err2string(rc), errno, slapd_system_strerror(errno));
-		    goto done;
-		}
-	    }
+        /* if we got here, we were able to read success result */
+        /* Get the controls sent by the server if requested */
+        if ((rc = ldap_parse_result(ld, result, &err, NULL, NULL,
+                      NULL, returnedctrls, 0)) != LDAP_SUCCESS) {
+            slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
+                "Error: could not parse bind result: error %d (%s) errno %d (%s)\n",
+                rc, ldap_err2string(rc), errno, slapd_system_strerror(errno));
+            goto done;
+        }
+
+        /* check the result code from the bind operation */
+        if(err){
+            rc = err;
+            slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
+                            "Error: could not bind id "
+                            "[%s] mech [%s]: error %d (%s) errno %d (%s)\n",
+                            bindid ? bindid : "(anon)",
+                            mech ? mech : "SIMPLE",
+                            rc, ldap_err2string(rc), errno, slapd_system_strerror(errno));
+            goto done;
+        }
 
 	    /* parse the bind result and get the ldap error code */
 	    if ((rc = ldap_parse_sasl_bind_result(ld, result, &servercredp,
@@ -1119,6 +1166,12 @@ slapi_ldap_bind(
 			    bindid ? bindid : "(anon)",
 			    mech, /* mech cannot be SIMPLE here */
 			    rc, ldap_err2string(rc));
+#ifdef HAVE_KRB5
+        if(mech && !strcmp(mech, "GSSAPI") && rc == 49){
+            /* only on err 49 should we clear out the credential cache */
+            clear_krb5_ccache();
+        }
+#endif
 	}
     }
 
@@ -2022,6 +2075,43 @@ cleanup:
     return;
 }
 
+static void
+clear_krb5_ccache()
+{
+    krb5_context ctx = NULL;
+    krb5_ccache cc = NULL;
+    int rc = 0;
+
+    PR_Lock(krb5_lock);
+
+    /* initialize the kerberos context */
+    if ((rc = krb5_init_context(&ctx))) {
+        slapi_log_error(SLAPI_LOG_FATAL, "clear_krb5_ccache", "Could not initialize kerberos context: %d (%s)\n",
+                        rc, error_message(rc));
+        goto done;
+    }
+    /* get the default ccache */
+    if ((rc = krb5_cc_default(ctx, &cc))) {
+        slapi_log_error(SLAPI_LOG_FATAL, "clear_krb5_ccache", "Could not get default kerberos ccache: %d (%s)\n",
+                        rc, error_message(rc));
+        goto done;
+    }
+    /* destroy the ccache */
+    if((rc = krb5_cc_destroy(ctx, cc))){
+        slapi_log_error(SLAPI_LOG_FATAL, "clear_krb5_ccache", "Could not destroy kerberos ccache: %d (%s)\n",
+                        rc, error_message(rc));
+    } else {
+        slapi_log_error(SLAPI_LOG_TRACE,"clear_krb5_ccache", "Successfully cleared kerberos ccache\n");
+    }
+
+done:
+    if(ctx){
+        krb5_free_context(ctx);
+    }
+
+    PR_Unlock(krb5_lock);
+}
+
 #endif /* HAVE_KRB5 */
 
 #define LDAP_DN		1
