@@ -40,7 +40,7 @@
 #  include <config.h>
 #endif
 
-
+#include <malloc.h>
 #include <ldap.h>
 #undef OFF
 #undef LITTLE_ENDIAN
@@ -661,6 +661,17 @@ main( int argc, char **argv)
 		memalloc_fns.ldapmem_free = (LDAP_FREE_CALLBACK *)_free_wrapper;
 		ldap_set_option( 0x1, LDAP_OPT_MEMALLOC_FN_PTRS, &memalloc_fns );
 	}
+#endif
+#ifdef LINUX
+	char *m = getenv( "SLAPD_MXFAST" );
+	if(m){
+		int val = atoi(m);
+		int max = 80 * (sizeof(size_t) / 4);
+
+		if(val >= 0 && val <= max){
+			mallopt(M_MXFAST, val);
+		}
+	}
 #endif
 	/*
 	 * Initialize NSPR very early. NSPR supports implicit initialization,
@@ -1150,6 +1161,9 @@ main( int argc, char **argv)
 		 */
 		task_cleanup();
 
+		/* init the thread data index for bind dn's */
+		slapi_td_dn_init();
+
 		plugin_print_lists();
 		plugin_startall(argc, argv, 1 /* Start Backends */, 1 /* Start Globals */); 
 		if (housekeeping_start((time_t)0, NULL) == NULL) {
