@@ -299,9 +299,8 @@ do_modify( Slapi_PBlock *pb )
             continue;
 		}
 		
-		/* check for password change */
-		if ( mod->mod_bvalues != NULL && 
-			 strcasecmp( mod->mod_type, SLAPI_USERPWD_ATTR ) == 0 ){
+		/* check for password change (including deletion) */
+		if ( strcasecmp( mod->mod_type, SLAPI_USERPWD_ATTR ) == 0 ){
 			has_password_mod++;
 		}
 
@@ -357,11 +356,11 @@ do_modify( Slapi_PBlock *pb )
 		for (mod = slapi_mods_get_first_mod(&smods);
 			 mod;
 			 mod = slapi_mods_get_next_mod(&smods)) {
-			if ( mod->mod_bvalues != NULL && 
-				 strcasecmp( mod->mod_type, SLAPI_USERPWD_ATTR ) == 0 ) {
+			/* check for password change (including deletion) */
+			if ( strcasecmp( mod->mod_type, SLAPI_USERPWD_ATTR ) == 0 ) {
 				/* assumes controls have already been decoded and placed
 				   in the pblock */
-				pw_change = op_shared_allow_pw_change (pb, mod, &old_pw, &smods);
+				pw_change = op_shared_allow_pw_change(pb, mod, &old_pw, &smods);
 				if (pw_change == -1) {
 					goto free_and_return;
 				}
@@ -637,7 +636,6 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 	Slapi_Entry	*pse;
 	Slapi_Entry *referral;
 	Slapi_Entry	*e = NULL;
-	char ebuf[BUFSIZ];
 	char *dn = NULL;
 	char *normdn = NULL;
 	Slapi_DN *sdn = NULL;
@@ -705,7 +703,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 			slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s\n",
 							 pb->pb_conn->c_connid, 
 							 pb->pb_op->o_opid,
-							 escape_string(slapi_sdn_get_dn(sdn), ebuf),
+							 slapi_sdn_get_dn(sdn),
 							 proxystr ? proxystr : "");
 		}
 		else
@@ -713,7 +711,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 			slapi_log_access(LDAP_DEBUG_ARGS, "conn=%s op=%d MOD dn=\"%s\"%s\n",
 							 LOG_INTERNAL_OP_CON_ID,
 							 LOG_INTERNAL_OP_OP_ID,
-							 escape_string(slapi_sdn_get_dn(sdn), ebuf),
+							 slapi_sdn_get_dn(sdn),
 							 proxystr ? proxystr : "");
 		}
 
@@ -821,11 +819,12 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 	}
 			
 	/*
-	 * Add the unhashed password pseudo-attribute before
-	 * calling the preop plugins
+	 * Add the unhashed password pseudo-attribute (for add) OR
+	 * Delete the unhashed password pseudo-attribute (for delete)
+	 * before calling the preop plugins
 	 */
 
-	if (pw_change)
+	if (pw_change && !repl_op)
 	{
 		Slapi_Value **va= NULL;
 
@@ -837,10 +836,20 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 			if (strcasecmp (pw_mod->mod_type, SLAPI_USERPWD_ATTR) != 0)
 				continue;
 
-			/* add pseudo password attribute */
-			valuearray_init_bervalarray(pw_mod->mod_bvalues, &va);
-			slapi_mods_add_mod_values(&smods, pw_mod->mod_op, unhashed_pw_attr, va);
-			valuearray_free(&va);
+			if (LDAP_MOD_DELETE == pw_mod->mod_op) {
+				Slapi_Attr *a = NULL;
+				/* delete pseudo password attribute if it exists in the entry */
+				if (!slapi_entry_attr_find(e, unhashed_pw_attr, &a)) {
+					slapi_mods_add_mod_values(&smods, pw_mod->mod_op,
+					                          unhashed_pw_attr, va);
+				}
+			} else {
+				/* add pseudo password attribute */
+				valuearray_init_bervalarray(pw_mod->mod_bvalues, &va);
+				slapi_mods_add_mod_values(&smods, pw_mod->mod_op,
+				                          unhashed_pw_attr, va);
+				valuearray_free(&va);
+			}
 
 			/* Init new value array for hashed value */
 			valuearray_init_bervalarray(pw_mod->mod_bvalues, &va);
@@ -856,7 +865,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 			valuearray_free(&va);
 		}
 	}
-	for ( p = get_plugin_list(PLUGIN_LIST_REVER_PWD_STORAGE_SCHEME); p != NULL; p = p->plg_next )
+	for ( p = get_plugin_list(PLUGIN_LIST_REVER_PWD_STORAGE_SCHEME); p != NULL && !repl_op; p = p->plg_next )
     {
         char *L_attr = NULL;
         int i = 0;
@@ -1039,14 +1048,12 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 	Slapi_Entry *e = NULL;
 	passwdPolicy *pwpolicy;
 	int rc = 0;
-	char ebuf[BUFSIZ];
 	Slapi_Value **values= NULL;
 	Slapi_Operation *operation;
 	int proxy_err = LDAP_SUCCESS;
 	char *proxydn = NULL;
 	char *proxystr = NULL;
 	char *errtext = NULL;
-	void *txn = NULL;
 
 	slapi_pblock_get (pb, SLAPI_IS_REPLICATED_OPERATION, &repl_op);
 	if (repl_op) {
@@ -1061,7 +1068,6 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 	slapi_pblock_get (pb, SLAPI_OPERATION, &operation);
 	slapi_pblock_get (pb, SLAPI_PWPOLICY, &pwresponse_req);
 	internal_op= operation_is_flag_set(operation, OP_FLAG_INTERNAL);
-	slapi_pblock_get (pb, SLAPI_TXN, &txn);
 
 	slapi_sdn_init_dn_byref (&sdn, dn);
 	pwpolicy = new_passwdPolicy(pb, (char *)slapi_sdn_get_ndn(&sdn));
@@ -1073,7 +1079,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 		{
 			slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"\n",
 					pb->pb_conn->c_connid, pb->pb_op->o_opid,
-					escape_string(slapi_sdn_get_dn(&sdn), ebuf));
+					slapi_sdn_get_dn(&sdn));
 		}
 
 		send_ldap_result(pb, proxy_err, NULL, errtext, 0, NULL);
@@ -1091,7 +1097,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 		mods[1] = NULL;
 
 		/* We need to actually fetch the target here to use for ACI checking. */
-		slapi_search_internal_get_entry_ext(&sdn, NULL, &e, (void *)plugin_get_default_component_id(), txn);
+		slapi_search_internal_get_entry(&sdn, NULL, &e, (void *)plugin_get_default_component_id());
 
 		/* Create a bogus entry with just the target dn if we were unable to 
 		 * find the actual entry.  This will only be used for checking the ACIs. */
@@ -1116,7 +1122,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 
 				slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s\n",
 						pb->pb_conn->c_connid, pb->pb_op->o_opid,
-						escape_string(slapi_sdn_get_dn(&sdn), ebuf),
+						slapi_sdn_get_dn(&sdn),
 						proxystr ? proxystr : "");
 			}
 
@@ -1150,8 +1156,8 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 				}
 
 				slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s, %s\n",
-	    						 pb->pb_conn->c_connid, pb->pb_op->o_opid,
-	    						 escape_string(slapi_sdn_get_dn(&sdn), ebuf), 
+							 pb->pb_conn->c_connid, pb->pb_op->o_opid,
+							 slapi_sdn_get_dn(&sdn),
 							 proxystr ? proxystr : "",
 							 "user is not allowed to change password");
 			}
@@ -1178,7 +1184,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 				slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s, %s\n",
 								 pb->pb_conn->c_connid, 
 								 pb->pb_op->o_opid,
-								 escape_string(slapi_sdn_get_dn(&sdn), ebuf), 
+								 slapi_sdn_get_dn(&sdn),
 								 proxystr ? proxystr : "",
 								 "within password minimum age");
 			}
@@ -1187,7 +1193,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 				slapi_log_access(LDAP_DEBUG_ARGS, "conn=%s op=%d MOD dn=\"%s\"%s, %s\n",
 								 LOG_INTERNAL_OP_CON_ID,
 								 LOG_INTERNAL_OP_OP_ID,
-								 escape_string(slapi_sdn_get_dn(&sdn), ebuf), 
+								 slapi_sdn_get_dn(&sdn),
 								 proxystr ? proxystr : "",
 								 "within password minimum age");
 			}
@@ -1201,7 +1207,8 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 	/* check password syntax; remember the old password;
 	   error sent directly from check_pw_syntax function */
 	valuearray_init_bervalarray(mod->mod_bvalues, &values);
-	switch (check_pw_syntax_ext (pb, &sdn, values, old_pw, NULL, 1, smods)) 
+	switch (check_pw_syntax_ext (pb, &sdn, values, old_pw, NULL, 
+	                             mod->mod_op, smods)) 
 	{
 		case 0: /* success */
 				rc = 1;
@@ -1220,7 +1227,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 						slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s, %s\n",
 										 pb->pb_conn->c_connid, 
 										 pb->pb_op->o_opid,
-										 escape_string(slapi_sdn_get_dn(&sdn), ebuf),
+										 slapi_sdn_get_dn(&sdn),
 										 proxystr ? proxystr : "",
 										"invalid password syntax");
 					}
@@ -1229,7 +1236,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 						slapi_log_access(LDAP_DEBUG_ARGS, "conn=%s op=%d MOD dn=\"%s\"%s, %s\n",
 										LOG_INTERNAL_OP_CON_ID,
 										LOG_INTERNAL_OP_OP_ID,
-										escape_string(slapi_sdn_get_dn(&sdn), ebuf),
+										slapi_sdn_get_dn(&sdn),
 										proxystr ? proxystr : "",
 										"invalid password syntax");
 					}
