@@ -62,6 +62,15 @@ slapi_op_abandoned( Slapi_PBlock *pb )
 	return 0;
 }
 
+int
+slapi_op_internal( Slapi_PBlock *pb )
+{
+	if (pb && pb->pb_op) {
+		return operation_is_flag_set(pb->pb_op, OP_FLAG_INTERNAL);
+	}
+	return 0;
+}
+
 void
 operation_out_of_disk_space()
 {
@@ -374,7 +383,7 @@ struct slapi_operation_parameters *operation_parameters_new()
 	return (slapi_operation_parameters *)slapi_ch_calloc (1, sizeof (slapi_operation_parameters));
 }
 
-static LDAPMod **
+LDAPMod **
 copy_mods(LDAPMod **orig_mods)
 {
 	LDAPMod **new_mods = NULL;
