@@ -136,30 +136,69 @@ do_ps_service(Slapi_Entry *e, Slapi_Entry *eprev, ber_int_t chgtype, ber_int_t c
 void modify_update_last_modified_attr(Slapi_PBlock *pb, Slapi_Mods *smods)
 {
     char        buf[20];
+    char        *plugin_dn = NULL;
+    char        *binddn = NULL;
     struct berval    bv;
     struct berval    *bvals[2];
     time_t        curtime;
     struct tm    utm;
     Operation    *op;
+    struct slapdplugin *plugin = NULL;
+    struct slapi_componentid *cid = NULL;
+    slapdFrontendConfig_t *slapdFrontendConfig = getFrontendConfig();
 
     LDAPDebug(LDAP_DEBUG_TRACE, "modify_update_last_modified_attr\n", 0, 0, 0);
 
     slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-
     bvals[0] = &bv;
     bvals[1] = NULL;
 
-    /* fill in modifiersname */
-    if (slapi_sdn_isempty(&op->o_sdn)) {
-        bv.bv_val = "";
-        bv.bv_len = strlen(bv.bv_val);
+    if(slapdFrontendConfig->plugin_track){
+        /* plugin bindDN tracking is enabled, grab the bind dn from thread local storage */
+        if(slapi_sdn_isempty(&op->o_sdn)){
+            bv.bv_val = "";
+            bv.bv_len = strlen(bv.bv_val);
+        } else {
+            slapi_pblock_get (pb, SLAPI_PLUGIN_IDENTITY, &cid);
+            if (cid)
+                plugin=(struct slapdplugin *) cid->sci_plugin;
+            if(plugin)
+                plugin_dn = plugin_get_dn (plugin);
+            if(plugin_dn){
+                bv.bv_val = plugin_dn;
+                bv.bv_len = strlen(bv.bv_val);
+            } else {
+                bv.bv_val = (char*)slapi_sdn_get_dn(&op->o_sdn);
+                bv.bv_len = strlen(bv.bv_val);
+            }
+        }
+        slapi_mods_add_modbvps(smods, LDAP_MOD_REPLACE | LDAP_MOD_BVALUES,
+                                  "internalModifiersName", bvals);
+
+        /* Grab the thread data(binddn) */
+        slapi_td_get_dn(&binddn);
+
+        if(binddn == NULL){
+            /* anonymous bind */
+            bv.bv_val = "";
+            bv.bv_len = strlen(bv.bv_val);
+   	    } else {
+            bv.bv_val = binddn;
+            bv.bv_len = strlen(bv.bv_val);
+        }
     } else {
-        bv.bv_val = (char*)slapi_sdn_get_dn(&op->o_sdn);
-        bv.bv_len = strlen(bv.bv_val);
+        /* fill in modifiersname */
+        if (slapi_sdn_isempty(&op->o_sdn)) {
+            bv.bv_val = "";
+            bv.bv_len = strlen(bv.bv_val);
+        } else {
+            bv.bv_val = (char*)slapi_sdn_get_dn(&op->o_sdn);
+            bv.bv_len = strlen(bv.bv_val);
+        }
     }
 
-    slapi_mods_add_modbvps(smods, LDAP_MOD_REPLACE | LDAP_MOD_BVALUES, 
-                           "modifiersname", bvals);    
+   	slapi_mods_add_modbvps(smods, LDAP_MOD_REPLACE | LDAP_MOD_BVALUES,
+   						   "modifiersname", bvals);
 
     /* fill in modifytimestamp */
     curtime = current_time();
@@ -196,7 +235,6 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
   Slapi_Backend   *be_single = NULL;
   Slapi_Backend   *be_list[BE_LIST_SIZE+1];
   Slapi_Entry     *referral_list[BE_LIST_SIZE+1];
-  char            ebuf[ BUFSIZ ];
   char            attrlistbuf[ 1024 ], *attrliststr, **attrs = NULL;
   int             rc = 0;
   int             internal_op;
@@ -229,6 +267,9 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
   Slapi_Backend *pr_be = NULL;
   void *pr_search_result = NULL;
   int pr_reset_processing = 0;
+  int pr_idx = -1;
+  Slapi_DN *orig_sdn = NULL;
+  int free_sdn = 0;
 
   be_list[0] = NULL;
   referral_list[0] = NULL;
@@ -240,6 +281,11 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
   if (NULL == sdn) {
     sdn = slapi_sdn_new_dn_byval(base);
     slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, sdn);
+    free_sdn = 1;
+  } else {
+    /* save it so we can restore it later - may have to replace it internally
+       e.g. for onelevel and subtree searches, but need to restore it */
+    orig_sdn = sdn; 
   }
   normbase = slapi_sdn_get_dn(sdn);
 
@@ -315,7 +361,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
           slapi_log_access(LDAP_DEBUG_STATS, fmtstr,
                            pb->pb_conn->c_connid, 
                            pb->pb_op->o_opid, 
-                           escape_string(normbase, ebuf),
+                           normbase,
                            scope, fstr, attrliststr,
                            flag_psearch ? " options=persistent" : "",
                            proxystr ? proxystr : "");
@@ -325,7 +371,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
           slapi_log_access(LDAP_DEBUG_ARGS, fmtstr,
                            LOG_INTERNAL_OP_CON_ID,
                            LOG_INTERNAL_OP_OP_ID,
-                           escape_string(normbase, ebuf),
+                           normbase,
                            scope, fstr, attrliststr,
                            flag_psearch ? " options=persistent" : "",
                            proxystr ? proxystr : "");
@@ -411,28 +457,38 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
       if ( slapi_control_present (ctrlp, LDAP_CONTROL_PAGEDRESULTS,
                                   &ctl_value, &iscritical) )
       {
-          rc = pagedresults_parse_control_value(ctl_value,
-                                               &pagesize, &curr_search_count);
+          rc = pagedresults_parse_control_value(pb, ctl_value,
+                                                &pagesize, &pr_idx);
+          /* Let's set pr_idx even if it fails; in case, pr_idx == -1. */
+          slapi_pblock_set(pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx);
           if (LDAP_SUCCESS == rc) {
               unsigned int opnote = SLAPI_OP_NOTE_SIMPLEPAGED;
-              if (pagedresults_check_or_set_processing(pb->pb_conn)) {
+              if (pagedresults_check_or_set_processing(pb->pb_conn, pr_idx)) {
                   send_ldap_result(pb, LDAP_UNWILLING_TO_PERFORM,
-                                   NULL, "Simple Paged Results Search already in progress on this connection", 0, NULL);
+                                   NULL, "Simple Paged Results Search "
+                                   "already in progress on this connection",
+                                   0, NULL);
                   goto free_and_return_nolock;
               }
-              pr_reset_processing = 1; /* need to reset after we are done with this op */
-              operation->o_flags |= OP_FLAG_PAGED_RESULTS;
-              pr_be = pagedresults_get_current_be(pb->pb_conn);
-              pr_search_result = pagedresults_get_search_result(pb->pb_conn);
+              /* need to reset after we are done with this op */
+              pr_reset_processing = 1;
+              op_set_pagedresults(operation);
+              pr_be = pagedresults_get_current_be(pb->pb_conn, pr_idx);
+              pr_search_result = pagedresults_get_search_result(pb->pb_conn,
+                                                                pr_idx);
               estimate = 
-                 pagedresults_get_search_result_set_size_estimate(pb->pb_conn);
-              if (pagedresults_get_unindexed(pb->pb_conn)) {
+                 pagedresults_get_search_result_set_size_estimate(pb->pb_conn,
+                                                                  pr_idx);
+              if (pagedresults_get_unindexed(pb->pb_conn, pr_idx)) {
                   opnote |= SLAPI_OP_NOTE_UNINDEXED;
               }
               slapi_pblock_set( pb, SLAPI_OPERATION_NOTES, &opnote );
           } else {
               /* parse paged-results-control failed */
               if (iscritical) { /* return an error since it's critical */
+                  send_ldap_result(pb, rc, NULL,
+                                   "Simple Paged Results Search failed",
+                                   0, NULL);
                   goto free_and_return;
               }
           }
@@ -528,7 +584,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
           }
           if (slapi_sdn_compare(basesdn, sdn)) {
               slapi_sdn_free(&basesdn);
-			  basesdn = operation_get_target_spec(pb->pb_op);
+              basesdn = operation_get_target_spec(pb->pb_op);
               slapi_sdn_free(&basesdn);
               basesdn = slapi_sdn_dup(sdn);
               operation_set_target_spec (pb->pb_op, basesdn);
@@ -562,13 +618,13 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
   }
 
   /* set the timelimit to clean up the too-long-lived-paged results requests */
-  if (operation->o_flags & OP_FLAG_PAGED_RESULTS) {
+  if (op_is_pagedresults(operation)) {
     time_t optime, time_up;
     int tlimit;
     slapi_pblock_get( pb, SLAPI_SEARCH_TIMELIMIT, &tlimit );
     slapi_pblock_get( pb, SLAPI_OPINITIATED_TIME, &optime );
     time_up = (tlimit==-1 ? -1 : optime + tlimit); /* -1: no time limit */
-    pagedresults_set_timelimit(pb->pb_conn, time_up);
+    pagedresults_set_timelimit(pb->pb_conn, time_up, pr_idx);
   }
 
   /* PAR: now filters have been rewritten, we can assign plugins to work on them */
@@ -619,7 +675,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
         next_be = NULL;
     }
         
-    if ((operation->o_flags & OP_FLAG_PAGED_RESULTS) && pr_search_result) {
+    if (op_is_pagedresults(operation) && pr_search_result) {
       void *sr = NULL;
       /* PAGED RESULTS and already have the search results from the prev op */
       slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, pr_search_result );
@@ -627,7 +683,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
 
       /* search result could be reset in the backend/dse */
       slapi_pblock_get(pb, SLAPI_SEARCH_RESULT_SET, &sr);
-      pagedresults_set_search_result(pb->pb_conn, sr, 0);
+      pagedresults_set_search_result(pb->pb_conn, sr, 0, pr_idx);
 
       if (PAGEDRESULTS_SEARCH_END == pr_stat) {
         /* no more entries to send in the backend */
@@ -637,7 +693,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
         } else {
           curr_search_count = pnentries;
           /* no more entries, but at least another backend */
-          if (pagedresults_set_current_be(pb->pb_conn, next_be) < 0) {
+          if (pagedresults_set_current_be(pb->pb_conn, next_be, pr_idx) < 0) {
               goto free_and_return;
           }
         }
@@ -646,14 +702,16 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
         curr_search_count = pnentries;
         estimate -= estimate?curr_search_count:0;
       }
-      pagedresults_set_response_control(pb, 0, estimate, curr_search_count);
-      if (pagedresults_get_with_sort(pb->pb_conn)) {
+      pagedresults_set_response_control(pb, 0, estimate,
+                                        curr_search_count, pr_idx);
+      if (pagedresults_get_with_sort(pb->pb_conn, pr_idx)) {
         sort_make_sort_response_control(pb, CONN_GET_SORT_RESULT_CODE, NULL);
       }
-      pagedresults_set_search_result_set_size_estimate(pb->pb_conn, estimate);
+      pagedresults_set_search_result_set_size_estimate(pb->pb_conn, 
+                                                       estimate, pr_idx);
       next_be = NULL; /* to break the loop */
       if (curr_search_count == -1) {
-        pagedresults_cleanup(pb->pb_conn, 1 /* need to lock */);
+        pagedresults_free_one(pb->pb_conn, pr_idx);
       }
     } else {
       /* be_suffix null means that we are searching the default backend
@@ -674,10 +732,13 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
             int tmp_scope = LDAP_SCOPE_BASE;
             slapi_pblock_set(pb, SLAPI_SEARCH_SCOPE, &tmp_scope);
 
-            slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
-            slapi_sdn_free(&sdn);
+            if (free_sdn) {
+              slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
+              slapi_sdn_free(&sdn);
+            }
             sdn = slapi_sdn_dup(be_suffix);
             slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, (void *)sdn);
+            free_sdn = 1;
           }
           else if (slapi_sdn_issuffix(basesdn, be_suffix))
           {
@@ -698,10 +759,13 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
         {
           if (slapi_sdn_issuffix(be_suffix, basesdn))
           {
-            slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
-            slapi_sdn_free(&sdn);
+            if (free_sdn) {
+              slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
+              slapi_sdn_free(&sdn);
+            }
             sdn = slapi_sdn_dup(be_suffix);
             slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, (void *)sdn);
+            free_sdn = 1;
           }
         }
       }
@@ -764,7 +828,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
         rc = send_results_ext (pb, 1, &pnentries, pagesize, &pr_stat);
   
         /* PAGED RESULTS */
-        if (operation->o_flags & OP_FLAG_PAGED_RESULTS) {
+        if (op_is_pagedresults(operation)) {
             void *sr = NULL;
             int with_sort = operation->o_flags & OP_FLAG_SERVER_SIDE_SORTING;
   
@@ -775,7 +839,7 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
                 curr_search_count = -1;
               } else {
                 /* no more entries, but at least another backend */
-                if (pagedresults_set_current_be(pb->pb_conn, next_be) < 0) {
+                if (pagedresults_set_current_be(pb->pb_conn, next_be, pr_idx) < 0) {
                   goto free_and_return;
                 }
               }
@@ -783,22 +847,22 @@ op_shared_search (Slapi_PBlock *pb, int send_result)
               curr_search_count = pnentries;
               slapi_pblock_get(pb, SLAPI_SEARCH_RESULT_SET, &sr);
               slapi_pblock_get(pb, SLAPI_SEARCH_RESULT_SET_SIZE_ESTIMATE, &estimate);
-              if (pagedresults_set_current_be(pb->pb_conn, be) < 0 ||
-                  pagedresults_set_search_result(pb->pb_conn, sr, 0) < 0 ||
+              if (pagedresults_set_current_be(pb->pb_conn, be, pr_idx) < 0 ||
+                  pagedresults_set_search_result(pb->pb_conn, sr, 0, pr_idx) < 0 ||
                   pagedresults_set_search_result_count(pb->pb_conn,
-                                                   curr_search_count) < 0 ||
+                                                   curr_search_count, pr_idx) < 0 ||
                   pagedresults_set_search_result_set_size_estimate(pb->pb_conn,
-                                                   estimate) < 0 ||
-                  pagedresults_set_with_sort(pb->pb_conn, with_sort) < 0) {
+                                                   estimate, pr_idx) < 0 ||
+                  pagedresults_set_with_sort(pb->pb_conn, with_sort, pr_idx) < 0) {
                 goto free_and_return;
               }
             }
-            pagedresults_set_response_control(pb, 0,
-                                              estimate, curr_search_count);
+            pagedresults_set_response_control(pb, 0, estimate, 
+                                              curr_search_count, pr_idx);
             slapi_pblock_set( pb, SLAPI_SEARCH_RESULT_SET, NULL );
             next_be = NULL; /* to break the loop */
             if (curr_search_count == -1) {
-                pagedresults_cleanup(pb->pb_conn, 1 /* need to lock */);
+                pagedresults_free_one(pb->pb_conn, pr_idx);
             }
         }
   
@@ -917,16 +981,18 @@ free_and_return:
 free_and_return_nolock:
   slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, &rc);
   index_subsys_filter_decoders_done(pb);
-  
-  slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
-  slapi_sdn_free(&sdn);
+
+  if (free_sdn) {
+    slapi_pblock_get(pb, SLAPI_SEARCH_TARGET_SDN, &sdn);
+    slapi_sdn_free(&sdn);
+  }
   slapi_sdn_free(&basesdn);
-  slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, NULL);
+  slapi_pblock_set(pb, SLAPI_SEARCH_TARGET_SDN, orig_sdn);
 
   slapi_ch_free_string(&proxydn);
   slapi_ch_free_string(&proxystr);
   if (pr_reset_processing) {
-    pagedresults_reset_processing(pb->pb_conn);
+    pagedresults_reset_processing(pb->pb_conn, pr_idx);
   }
 }
 
@@ -968,9 +1034,8 @@ process_entry(Slapi_PBlock *pb, Slapi_Entry *e, int send_result)
         slapi_attr_get_numvalues(a, &numValues );
         if (numValues == 0) 
         {
-            char ebuf[ BUFSIZ ];
-            LDAPDebug(LDAP_DEBUG_ANY, "null ref in (%s)\n",
-                      escape_string(slapi_entry_get_dn_const(e), ebuf), 0, 0);
+            LDAPDebug1Arg(LDAP_DEBUG_ANY, "null ref in (%s)\n",
+                          slapi_entry_get_dn_const(e));
         }
         else 
         {
@@ -1165,12 +1230,14 @@ iterate(Slapi_PBlock *pb, Slapi_Backend *be, int send_result,
     Slapi_Entry *e = NULL;
     char **attrs = NULL;
     unsigned int pr_stat = 0;
+    int pr_idx = -1;
 
     if ( NULL == pb ) {
         return rval;
     }
     slapi_pblock_get(pb, SLAPI_SEARCH_ATTRS, &attrs);
     slapi_pblock_get(pb, SLAPI_SEARCH_ATTRSONLY, &attrsonly);
+    slapi_pblock_get(pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx);
 
     *pnentries = 0;
     
@@ -1192,7 +1259,7 @@ iterate(Slapi_PBlock *pb, Slapi_Backend *be, int send_result,
                 operation_out_of_disk_space();
             }
             pr_stat = PAGEDRESULTS_SEARCH_END;
-            pagedresults_set_timelimit(pb->pb_conn, 0);
+            pagedresults_set_timelimit(pb->pb_conn, 0, pr_idx);
             rval = -1;
             done = 1;
             continue;
@@ -1503,7 +1570,7 @@ compute_limits (Slapi_PBlock *pb)
         }
     }
 
-    if (op && (op->o_flags & OP_FLAG_PAGED_RESULTS)) {
+    if (op_is_pagedresults(op)) {
         if ( slapi_reslimit_get_integer_limit( pb->pb_conn,
                 pagedsizelimit_reslimit_handle, &max_sizelimit )
                 != SLAPI_RESLIMIT_STATUS_SUCCESS ) {
@@ -1594,7 +1661,7 @@ send_results_ext(Slapi_PBlock *pb, int send_result, int *nentries, int pagesize,
                      */
                     break;    
 
-        case 1:        /* everything is ok - don't send the result */
+        case 1:     /* everything is ok - don't send the result */
                     rc = 0;
     }
     
@@ -1603,7 +1670,6 @@ send_results_ext(Slapi_PBlock *pb, int send_result, int *nentries, int pagesize,
 
 void op_shared_log_error_access (Slapi_PBlock *pb, const char *type, const char *dn, const char *msg)
 {
-    char ebuf[BUFSIZ];
     char *proxydn = NULL;
     char *proxystr = NULL;
 
@@ -1615,7 +1681,7 @@ void op_shared_log_error_access (Slapi_PBlock *pb, const char *type, const char
                       ( pb->pb_conn ? pb->pb_conn->c_connid : 0), 
                       ( pb->pb_op ? pb->pb_op->o_opid : 0), 
                       type, 
-                      escape_string( dn, ebuf ), 
+                      dn, 
                       proxystr ? proxystr : "",
                       msg ? msg : "" );
 
