@@ -153,7 +153,6 @@ passwd_apply_mods(Slapi_PBlock *pb_orig, const Slapi_DN *sdn, Slapi_Mods *mods,
 	LDAPControl **req_controls_copy = NULL;
 	LDAPControl **pb_resp_controls = NULL;
 	int ret=0;
-	void *txn = NULL;
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "=> passwd_apply_mods\n", 0, 0, 0 );
 
@@ -180,9 +179,6 @@ passwd_apply_mods(Slapi_PBlock *pb_orig, const Slapi_DN *sdn, Slapi_Mods *mods,
 		 * that it was done by the root DN. */
 		pb.pb_conn = pb_orig->pb_conn;
 
-		slapi_pblock_get(pb_orig, SLAPI_TXN, &txn);
-		slapi_pblock_set(&pb, SLAPI_TXN, txn);
-
 		ret =slapi_modify_internal_pb (&pb);
 
 		/* We now clean up the connection that we copied into the
@@ -489,7 +485,7 @@ passwd_modify_extop( Slapi_PBlock *pb )
 		errMesg = "Could not get OID value from request.\n";
 		rc = LDAP_OPERATIONS_ERROR;
 		slapi_log_error( SLAPI_LOG_PLUGIN, "passwd_modify_extop", 
-				 errMesg );
+				"%s", errMesg );
 		goto free_and_return;
 	} else {
 	        slapi_log_error( SLAPI_LOG_PLUGIN, "passwd_modify_extop", 
@@ -515,7 +511,7 @@ passwd_modify_extop( Slapi_PBlock *pb )
 		errMesg = "Could not get SASL SSF from connection\n";
 		rc = LDAP_OPERATIONS_ERROR;
 		slapi_log_error( SLAPI_LOG_PLUGIN, "passwd_modify_extop",
-				 errMesg );
+				"%s", errMesg );
 		goto free_and_return;
 	}
 
@@ -523,7 +519,7 @@ passwd_modify_extop( Slapi_PBlock *pb )
 		errMesg = "Could not get local SSF from connection\n";
 		rc = LDAP_OPERATIONS_ERROR;
 		slapi_log_error( SLAPI_LOG_PLUGIN, "passwd_modify_extop",
-				 errMesg );
+				"%s", errMesg );
 		goto free_and_return;
 	}
 
@@ -846,7 +842,7 @@ parse_req_done:
 	/* Free anything that we allocated above */
 free_and_return:
 	slapi_log_error( SLAPI_LOG_PLUGIN, "passwd_modify_extop",
-		errMesg ? errMesg : "success" );
+		"%s", errMesg ? errMesg : "success" );
 
 	if ((rc == LDAP_REFERRAL) && (referrals)) {
 		send_referrals_from_entry(pb, referrals);
