@@ -289,6 +289,7 @@ int config_set_readonly( const char *attrname, char *value, 	char *errorbuf, int
 int config_set_schemacheck( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_syntaxcheck( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_syntaxlogging( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_plugin_tracking( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_dn_validate_strict( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_ds4_compatible_schema( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_schema_ignore_trailing_spaces( const char *attrname, char *value, char *errorbuf, int apply );
@@ -339,6 +340,8 @@ int config_set_pw_unlock(const char *attrname,  char *value, char *errorbuf, int
 int config_set_pw_lockduration(const char *attrname,  char *value, char *errorbuf, int apply );
 int config_set_pw_resetfailurecount(const char *attrname,  char *value, char *errorbuf, int apply );
 int config_set_pw_is_global_policy(const char *attrname,  char *value, char *errorbuf, int apply );
+int config_set_pw_is_legacy_policy(const char *attrname,  char *value, char *errorbuf, int apply );
+int config_set_pw_track_last_update_time(const char *attrname,  char *value, char *errorbuf, int apply );
 int config_set_pw_gracelimit(const char *attrname,  char *value, char *errorbuf, int apply );
 int config_set_useroc(const char *attrname,  char *value, char *errorbuf, int apply );
 int config_set_return_exact_case(const char *attrname,  char *value, char *errorbuf, int apply );
@@ -379,6 +382,12 @@ int config_set_entryusn_global( const char *attrname, char *value, char *errorbu
 int config_set_allowed_to_delete_attrs( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_entryusn_import_init( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_default_naming_context( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_disk_monitoring( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_disk_threshold( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_disk_grace_period( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_disk_preserve_logging( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_disk_logging_critical( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_auditlog_unhashed_pw(const char *attrname, char *value, char *errorbuf, int apply);
 
 #if !defined(_WIN32) && !defined(AIX)
 int config_set_maxdescriptors( const char *attrname, char *value, char *errorbuf, int apply );
@@ -525,8 +534,15 @@ int config_get_entryusn_global(void);
 char *config_get_allowed_to_delete_attrs(void);
 char *config_get_entryusn_import_init(void);
 char *config_get_default_naming_context(void);
-
 int config_allowed_to_delete_attrs(const char *attr_type);
+void config_set_accesslog_enabled(int value);
+void config_set_auditlog_enabled(int value);
+int config_get_accesslog_logging_enabled();
+int config_get_disk_monitoring();
+long config_get_disk_threshold();
+int config_get_disk_grace_period();
+int config_get_disk_preserve_logging();
+int config_get_disk_logging_critical();
 
 int is_abspath(const char *);
 char* rel2abspath( char * );
@@ -623,7 +639,7 @@ int is_rootdse( const char *dn );
 int get_entry_object_type();
 int entry_computed_attr_init();
 void send_referrals_from_entry(Slapi_PBlock *pb, Slapi_Entry *referral);
-
+int is_type_forbidden(const char *type);
 
 /*
  * dse.c
@@ -736,7 +752,7 @@ int check_log_max_size(
  
 
 void g_set_accesslog_level(int val);
-
+void log__delete_rotated_logs();
 
 /*
  * util.c
@@ -867,7 +883,6 @@ void pw_add_allowchange_aci(Slapi_Entry *e, int pw_prohibit_change);
  */
 int update_pw_retry ( Slapi_PBlock *pb );
 void pw_apply_mods(const Slapi_DN *sdn, Slapi_Mods *mods);
-void pw_apply_mods_ext(const Slapi_DN *sdn, Slapi_Mods *mods, void *txn);
 void pw_set_componentID(struct slapi_componentid * cid);
 struct slapi_componentid * pw_get_componentID();
 
@@ -1176,6 +1191,8 @@ void factory_destroy_extension(int type,void *object,void *parent,void **extensi
  */
 
 void write_audit_log_entry( Slapi_PBlock *pb);
+void auditlog_hide_unhashed_pw();
+void auditlog_expose_unhashed_pw();
 
 /*
  * eventq.c
@@ -1378,26 +1395,41 @@ int slapd_do_all_nss_ssl_init(int slapd_exemode, int importexport_encrypt,
 /*
  * pagedresults.c
  */
-int pagedresults_parse_control_value(struct berval *psbvp, ber_int_t *pagesize, int *curr_search_count);
-void pagedresults_set_response_control(Slapi_PBlock *pb, int iscritical, ber_int_t estimate, int curr_search_count);
-Slapi_Backend *pagedresults_get_current_be(Connection *conn);
-int pagedresults_set_current_be(Connection *conn, Slapi_Backend *be);
-void *pagedresults_get_search_result(Connection *conn);
-int pagedresults_set_search_result(Connection *conn, void *sr, int locked);
-int pagedresults_get_search_result_count(Connection *conn);
-int pagedresults_set_search_result_count(Connection *conn, int cnt);
-int pagedresults_get_search_result_set_size_estimate(Connection *conn);
-int pagedresults_set_search_result_set_size_estimate(Connection *conn, int cnt);
-int pagedresults_get_with_sort(Connection *conn);
-int pagedresults_set_with_sort(Connection *conn, int flags);
-int pagedresults_get_unindexed(Connection *conn);
-int pagedresults_set_unindexed(Connection *conn);
-int pagedresults_get_sort_result_code(Connection *conn);
-int pagedresults_set_sort_result_code(Connection *conn, int code);
-int pagedresults_set_timelimit(Connection *conn, time_t timelimit);
+int pagedresults_parse_control_value(Slapi_PBlock *pb, struct berval *psbvp,
+                                     ber_int_t *pagesize, int *index);
+void pagedresults_set_response_control(Slapi_PBlock *pb, int iscritical, 
+                                       ber_int_t estimate,
+                                       int curr_search_count, int index);
+Slapi_Backend *pagedresults_get_current_be(Connection *conn, int index);
+int pagedresults_set_current_be(Connection *conn, Slapi_Backend *be, int index);
+void *pagedresults_get_search_result(Connection *conn, int index);
+int pagedresults_set_search_result(Connection *conn, void *sr, 
+                                   int locked, int index);
+int pagedresults_get_search_result_count(Connection *conn, int index);
+int pagedresults_set_search_result_count(Connection *conn, int cnt, int index);
+int pagedresults_get_search_result_set_size_estimate(Connection *conn, 
+                                                     int index);
+int pagedresults_set_search_result_set_size_estimate(Connection *conn, int cnt, 
+                                                     int index);
+int pagedresults_get_with_sort(Connection *conn, int index);
+int pagedresults_set_with_sort(Connection *conn, int flags, int index);
+int pagedresults_get_unindexed(Connection *conn, int index);
+int pagedresults_set_unindexed(Connection *conn, int index);
+int pagedresults_get_sort_result_code(Connection *conn, int index);
+int pagedresults_set_sort_result_code(Connection *conn, int code, int index);
+int pagedresults_set_timelimit(Connection *conn, time_t timelimit, int index);
 int pagedresults_cleanup(Connection *conn, int needlock);
-int pagedresults_check_or_set_processing(Connection *conn);
-int pagedresults_reset_processing(Connection *conn);
+int pagedresults_check_or_set_processing(Connection *conn, int index);
+int pagedresults_reset_processing(Connection *conn, int index);
+int pagedresults_is_timedout(Connection *conn);
+int pagedresults_reset_timedout(Connection *conn);
+int pagedresults_in_use(Connection *conn);
+int pagedresults_free_one(Connection *conn, int index);
+int pagedresults_free_one_msgid( Connection *conn, ber_int_t msgid );
+int op_is_pagedresults(Operation *op);
+int pagedresults_cleanup_all(Connection *conn, int needlock);
+void op_set_pagedresults(Operation *op);
+
 
 /*
  * sort.c
