@@ -593,9 +593,7 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw) {
 	passwdPolicy *pwpolicy = NULL;
 	int internal_op = 0;
 	Slapi_Operation *operation = NULL;
-	void *txn = NULL;
 
-	slapi_pblock_get(pb, SLAPI_TXN, &txn);
 	slapi_pblock_get(pb, SLAPI_OPERATION, &operation);
 	internal_op = slapi_operation_is_flag_set(operation, SLAPI_OP_FLAG_INTERNAL);
 
@@ -613,6 +611,13 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw) {
 
 	slapi_mods_init(&smods, 0);
 	
+	/* Update the "pwdUpdateTime" attribute */
+	if ( pwpolicy->pw_track_update_time ){
+		timestr = format_genTime(cur_time);
+		slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "pwdUpdateTime",timestr);
+		slapi_ch_free((void **)&timestr);
+	}
+
 	/* update password allow change time */
 	if ( pwpolicy->pw_minage != 0) {
 		timestr = format_genTime( time_plus_sec( cur_time, 
@@ -668,7 +673,7 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw) {
 			}  else if (prev_exp_date == SLAPD_END_TIME) {
 			    /* Special entries' passwords never expire */
 			  slapi_ch_free((void**)&prev_exp_date_str);
-			  pw_apply_mods_ext(sdn, &smods, txn);
+			  pw_apply_mods(sdn, &smods);
 			  slapi_mods_done(&smods);
 			  delete_passwdPolicy(&pwpolicy);
 			  return 0;
@@ -685,7 +690,7 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw) {
 		 */
 		pw_exp_date = NOT_FIRST_TIME;
 	} else {
-		pw_apply_mods_ext(sdn, &smods, txn);
+		pw_apply_mods(sdn, &smods);
 		slapi_mods_done(&smods);
 		delete_passwdPolicy(&pwpolicy);
 		return 0;
@@ -699,7 +704,7 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw) {
 	
 	slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "passwordExpWarned", "0");
 
-	pw_apply_mods_ext(sdn, &smods, txn);
+	pw_apply_mods(sdn, &smods);
 	slapi_mods_done(&smods);
     if (pb->pb_conn) { /* no conn for internal op */
         /* reset c_needpw to 0 */
@@ -790,6 +795,20 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 	passwdPolicy	*pwpolicy = NULL;
 	Slapi_Operation *operation = NULL;
 
+	/*
+	 * check_pw_syntax_ext could be called with mod_op == LDAP_MOD_DELETE.
+	 * In that case, no need to check the password syntax, but just returns 
+	 * PASS == 0.
+	 */
+	if (LDAP_MOD_DELETE == (mod_op & LDAP_MOD_OP)) {
+		return 0;
+	}
+	if (NULL == vals) {
+		slapi_log_error( SLAPI_LOG_FATAL, NULL, 
+		                 "check_pw_syntax_ext: no passwords to check\n" );
+		return -1;
+	}
+
 	pwpolicy = new_passwdPolicy(pb, dn);
 	slapi_pblock_get ( pb, SLAPI_PWPOLICY, &pwresponse_req );
 
@@ -969,7 +988,7 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 	}
 
 	/* get the entry and check for the password history if this is called by a modify operation */
-	if ( mod_op ) { 
+	if ( mod_op ) {
 		/* retrieve the entry */
 		e = get_entry ( pb, dn );
 		if ( e == NULL ) {
@@ -1082,7 +1101,6 @@ update_pw_history( Slapi_PBlock *pb, const Slapi_DN *sdn, char *old_pw )
 	char		*str;
 	passwdPolicy *pwpolicy = NULL;
 	const char *dn = slapi_sdn_get_dn(sdn);
-	void *txn = NULL;
 
 	pwpolicy = new_passwdPolicy(pb, dn);
 
@@ -1141,10 +1159,8 @@ update_pw_history( Slapi_PBlock *pb, const Slapi_DN *sdn, char *old_pw )
 	list_of_mods[1] = NULL;
 
 	pblock_init(&mod_pb);
-	slapi_pblock_get(pb, SLAPI_TXN, &txn);
 	slapi_modify_internal_set_pb_ext(&mod_pb, sdn, list_of_mods, NULL, NULL, 
 								 pw_get_componentID(), 0);
-	slapi_pblock_set(&mod_pb, SLAPI_TXN, txn);
 	slapi_modify_internal_pb(&mod_pb);
 	slapi_pblock_get(&mod_pb, SLAPI_PLUGIN_INTOP_RESULT, &res);
 	if (res != LDAP_SUCCESS){
@@ -1530,26 +1546,22 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 	char *attr_name;
 	Slapi_Value **sval;
 	slapdFrontendConfig_t *slapdFrontendConfig;
-	char ebuf[ BUFSIZ ];
 	int optype = -1;
 
-	/* RFE - is there a way to make this work for non-existent entries 
-	 * when we don't pass in pb?  We'll need to do this if we add support 
-	 * for password policy plug-ins. */
-	if (NULL == pb) {
-		LDAPDebug0Args(LDAP_DEBUG_ANY, 
-		               "new_passwdPolicy: NULL pblock was passed.\n");
-		return NULL;
-	}
 	slapdFrontendConfig = getFrontendConfig();
 	pwdpolicy = (passwdPolicy *)slapi_ch_calloc(1, sizeof(passwdPolicy));
 
-	slapi_pblock_get( pb, SLAPI_OPERATION_TYPE, &optype );
+	if (pb) {
+		slapi_pblock_get( pb, SLAPI_OPERATION_TYPE, &optype );
+	}
 
 	if (dn && (slapdFrontendConfig->pwpolicy_local == 1)) {
 		/*  If we're doing an add, COS does not apply yet so we check
 			parents for the pwdpolicysubentry.  We look only for virtual
 			attributes, because real ones are for single-target policy. */
+		/* RFE - is there a way to make this work for non-existent entries 
+		 * when we don't pass in pb?  We'll need to do this if we add support 
+		 * for password policy plug-ins. */
 		if (optype == SLAPI_OPERATION_ADD) {
 			char *parentdn = slapi_ch_strdup(dn);
 			char *nextdn = NULL;
@@ -1608,11 +1620,15 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 			slapi_entry_free( e );
 
 			if ( pw_entry == NULL ) {
-				LDAPDebug(LDAP_DEBUG_ANY, "loading global password policy for %s"
-					"--local policy entry not found\n", escape_string(dn, ebuf),0,0);
+				LDAPDebug1Arg(LDAP_DEBUG_ANY,
+				              "loading global password policy for %s"
+				              "--local policy entry not found\n", dn);
 				goto done;
 			}
         
+			/* set the default passwordLegacyPolicy setting */
+			pwdpolicy->pw_is_legacy = 1;
+
 			for (slapi_entry_first_attr(pw_entry, &attr); attr;
 					slapi_entry_next_attr(pw_entry, attr, &attr))
 			{
@@ -1798,7 +1814,20 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 						pw_name2scheme((char*)slapi_value_get_string(*sval));
 					}
 				}
-                        
+				else
+				if (!strcasecmp(attr_name, "passwordLegacyPolicy")) {
+					if ((sval = attr_get_present_values(attr))) {
+						pwdpolicy->pw_is_legacy =
+						pw_boolean_str2value(slapi_value_get_string(*sval));
+					}
+				}
+				else
+				if (!strcasecmp(attr_name, "passwordTrackUpdateTime")) {
+					if ((sval = attr_get_present_values(attr))) {
+						pwdpolicy->pw_track_update_time =
+						pw_boolean_str2value(slapi_value_get_string(*sval));
+					}
+				}
 			} /* end of for() loop */
 			if (pw_entry) {
 				slapi_entry_free(pw_entry);
