@@ -50,8 +50,8 @@
 /* prototypes                                                               */
 /****************************************************************************/
 /* Slapi_Entry *get_entry ( Slapi_PBlock *pb, const char *dn ); */
-static void set_retry_cnt ( Slapi_PBlock *pb, int count);
-static void set_retry_cnt_and_time ( Slapi_PBlock *pb, int count, time_t cur_time);
+static int set_retry_cnt ( Slapi_PBlock *pb, int count);
+static int set_retry_cnt_and_time ( Slapi_PBlock *pb, int count, time_t cur_time);
 
 /*
  * update_pw_retry() is called when bind operation fails 
@@ -72,6 +72,7 @@ int update_pw_retry ( Slapi_PBlock *pb )
 	char            *cur_time_str = NULL;
 	char *retryCountResetTime;
 	int passwordRetryCount;
+	int rc = 0;
 
     /* get the entry */
     e = get_entry ( pb, NULL );
@@ -93,18 +94,18 @@ int update_pw_retry ( Slapi_PBlock *pb )
         {
             /* set passwordRetryCount to 1 */
             /* reset retryCountResetTime */
-			set_retry_cnt_and_time ( pb, 1, cur_time );
+			rc = set_retry_cnt_and_time ( pb, 1, cur_time );
 			slapi_ch_free((void **) &cur_time_str );
 			slapi_entry_free( e );
-            return ( 0 ); /* success */
+            return ( rc ); /* success */
         } else {
 			slapi_ch_free((void **) &cur_time_str );
 		}
     } else {
 		/* initialize passwordRetryCount and retryCountResetTime */
-		set_retry_cnt_and_time ( pb, 1, cur_time );
+		rc = set_retry_cnt_and_time ( pb, 1, cur_time );
 		slapi_entry_free( e );
-        return ( 0 ); /* success */
+        return ( rc ); /* success */
 	}
 	passwordRetryCount = slapi_entry_attr_get_int(e, "passwordRetryCount"); 
     if (passwordRetryCount >= 0)
@@ -112,27 +113,26 @@ int update_pw_retry ( Slapi_PBlock *pb )
         retry_cnt = passwordRetryCount + 1;
    		if ( retry_cnt == 1 ) {
         	/* set retryCountResetTime */
-        	set_retry_cnt_and_time ( pb, retry_cnt, cur_time );
+        	rc = set_retry_cnt_and_time ( pb, retry_cnt, cur_time );
 		} else {
 			/* set passwordRetryCount to retry_cnt */
-			set_retry_cnt ( pb, retry_cnt );
+			rc = set_retry_cnt ( pb, retry_cnt );
 		}
     }	
 	slapi_entry_free( e );
-	return 0; /* success */
+	return rc; /* success */
 }
 
 static
-void set_retry_cnt_and_time ( Slapi_PBlock *pb, int count, time_t cur_time ) {
+int set_retry_cnt_and_time ( Slapi_PBlock *pb, int count, time_t cur_time ) {
 	const char  *dn = NULL;
 	Slapi_DN    *sdn = NULL;
 	Slapi_Mods	smods;
 	time_t      reset_time;
 	char		*timestr;
 	passwdPolicy *pwpolicy = NULL;
-	void *txn = NULL;
+	int rc = 0;
 
-	slapi_pblock_get( pb, SLAPI_TXN, &txn );
 	slapi_pblock_get( pb, SLAPI_TARGET_SDN, &sdn );
 	dn = slapi_sdn_get_dn(sdn);
 	pwpolicy = new_passwdPolicy(pb, dn);
@@ -146,14 +146,16 @@ void set_retry_cnt_and_time ( Slapi_PBlock *pb, int count, time_t cur_time ) {
 	slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "retryCountResetTime", timestr);
 	slapi_ch_free((void **)&timestr);
 
-	set_retry_cnt_mods(pb, &smods, count);
+	rc = set_retry_cnt_mods(pb, &smods, count);
 	
-	pw_apply_mods_ext(sdn, &smods, txn);
+	pw_apply_mods(sdn, &smods);
 	slapi_mods_done(&smods);
 	delete_passwdPolicy(&pwpolicy);
+
+	return rc;
 }
 
-void set_retry_cnt_mods(Slapi_PBlock *pb, Slapi_Mods *smods, int count)
+int set_retry_cnt_mods(Slapi_PBlock *pb, Slapi_Mods *smods, int count)
 {
 	char 		*timestr;
 	time_t		unlock_time;
@@ -161,6 +163,7 @@ void set_retry_cnt_mods(Slapi_PBlock *pb, Slapi_Mods *smods, int count)
 	const char *dn = NULL; 
 	Slapi_DN *sdn = NULL; 
 	passwdPolicy *pwpolicy = NULL;
+	int rc = 0;
 
 	slapi_pblock_get( pb, SLAPI_TARGET_SDN, &sdn );
 	dn = slapi_sdn_get_dn(sdn);
@@ -184,73 +187,78 @@ void set_retry_cnt_mods(Slapi_PBlock *pb, Slapi_Mods *smods, int count)
 			timestr= format_genTime ( unlock_time );
 			slapi_mods_add_string(smods, LDAP_MOD_REPLACE, "accountUnlockTime", timestr);
 			slapi_ch_free((void **)&timestr);
+			rc = LDAP_CONSTRAINT_VIOLATION;
 		}
 	}
 	delete_passwdPolicy(&pwpolicy);
-	return;
+	return rc;
 }
 
 static
-void set_retry_cnt ( Slapi_PBlock *pb, int count)
+int set_retry_cnt ( Slapi_PBlock *pb, int count)
 {
 	Slapi_DN *sdn = NULL; 
 	Slapi_Mods	smods;
-	void *txn = NULL;
-
-	slapi_pblock_get( pb, SLAPI_TXN, &txn );
+	int rc = 0;
+	
 	slapi_pblock_get( pb, SLAPI_TARGET_SDN, &sdn );
 	slapi_mods_init(&smods, 0);
-	set_retry_cnt_mods(pb, &smods, count);
-	pw_apply_mods_ext(sdn, &smods, txn);
+	rc = set_retry_cnt_mods(pb, &smods, count);
+	pw_apply_mods(sdn, &smods);
 	slapi_mods_done(&smods);
+	return rc;
 }
 
 
+/* 
+ * If "dn" is passed, get_entry returns an entry which dn is "dn".
+ * If "dn" is not passed, it returns an entry which dn is set in 
+ * SLAPI_TARGET_SDN in pblock.
+ * Note: pblock is not mandatory for get_entry (e.g., new_passwdPolicy).
+ */
 Slapi_Entry *get_entry ( Slapi_PBlock *pb, const char *dn)
 {
 	int             search_result = 0;
 	Slapi_Entry     *retentry = NULL;
 	Slapi_DN        *target_sdn = NULL;
+	char            *target_dn = (char *)dn;
 	Slapi_DN        sdn;
-	void            *txn = NULL;
 
-	if (NULL == pb) {
-		LDAPDebug(LDAP_DEBUG_ANY, "get_entry - no pblock specified.\n",
-		          0, 0, 0);
+	if (pb) {
+		slapi_pblock_get( pb, SLAPI_TARGET_SDN, &target_sdn );
+		if (target_dn == NULL) {
+			target_dn = slapi_sdn_get_dn(target_sdn);
+		}
+	}
+
+	if (target_dn == NULL) {
+		LDAPDebug0Args(LDAP_DEBUG_TRACE,
+		               "WARNING: 'get_entry' - no dn specified.\n");
 		goto bail;
 	}
 
-	slapi_pblock_get( pb, SLAPI_TARGET_SDN, &target_sdn );
-	slapi_pblock_get( pb, SLAPI_TXN, &txn );
-
-	if (dn == NULL) {
-		dn = slapi_sdn_get_dn(target_sdn);
+	if (target_dn == dn) { /* target_dn is NOT from target_sdn */
+		slapi_sdn_init_dn_byref(&sdn, target_dn);
+		target_sdn = &sdn;
 	}
 
-	if (dn == NULL) {
-		LDAPDebug (LDAP_DEBUG_TRACE, "WARNING: 'get_entry' - no dn specified.\n", 0, 0, 0);
-		goto bail;
-	}
-
-	slapi_sdn_init_dn_byref(&sdn, dn);
-
-	if (slapi_sdn_compare(&sdn, target_sdn)) { /* does not match */
-	    target_sdn = &sdn;
-	}
-
-	search_result = slapi_search_internal_get_entry_ext(target_sdn, NULL,
-														&retentry,
-														pw_get_componentID(), txn);
+	search_result = slapi_search_internal_get_entry(target_sdn, NULL,
+	                                                &retentry, 
+	                                                pw_get_componentID());
 	if (search_result != LDAP_SUCCESS) {
-		LDAPDebug (LDAP_DEBUG_TRACE, "WARNING: 'get_entry' can't find entry '%s', err %d\n", dn, search_result, 0);
+		LDAPDebug2Args(LDAP_DEBUG_TRACE,
+		               "WARNING: 'get_entry' can't find entry '%s', err %d\n",
+		               target_dn, search_result);
+	}
+	if (target_dn == dn) { /* target_dn is NOT from target_sdn */
+		slapi_sdn_done(&sdn);
 	}
-	slapi_sdn_done(&sdn);
 bail:
 	return retentry;
 }
 
 void
-pw_apply_mods_ext(const Slapi_DN *sdn, Slapi_Mods *mods, void *txn) 
+pw_apply_mods(const Slapi_DN *sdn, Slapi_Mods *mods) 
 {
 	Slapi_PBlock pb;
 	int res;
@@ -266,7 +274,6 @@ pw_apply_mods_ext(const Slapi_DN *sdn, Slapi_Mods *mods, void *txn)
 					  NULL, /* UniqueID */
 					  pw_get_componentID(), /* PluginID */
 					  OP_FLAG_SKIP_MODIFIED_ATTRS); /* Flags */
-		slapi_pblock_set(&pb, SLAPI_TXN, txn);
 		slapi_modify_internal_pb (&pb);
 		
 		slapi_pblock_get(&pb, SLAPI_PLUGIN_INTOP_RESULT, &res);
@@ -282,11 +289,6 @@ pw_apply_mods_ext(const Slapi_DN *sdn, Slapi_Mods *mods, void *txn)
 	return;
 }
 
-void
-pw_apply_mods(const Slapi_DN *sdn, Slapi_Mods *mods)
-{
-	pw_apply_mods_ext(sdn, mods, NULL);
-}
 /* Handle the component ID for the password policy */
 
 static struct slapi_componentid * pw_componentid = NULL;
