@@ -297,6 +297,10 @@ send_ldap_result_ext(
 	
 	slapi_pblock_get (pb, SLAPI_OPERATION, &operation);
 
+	if (operation->o_status == SLAPI_OP_STATUS_RESULT_SENT) {
+		return; /* result already sent */
+	}
+
 	if ( ber != NULL ) {
 	    flush_ber_element = 0;
 	}
@@ -373,7 +377,15 @@ send_ldap_result_ext(
 		dn = slapi_sdn_get_dn(sdn);
 		pwpolicy = new_passwdPolicy(pb, dn);
 		if (pwpolicy && (pwpolicy->pw_lockout == 1)) {
-			update_pw_retry ( pb );
+			if(update_pw_retry( pb ) == LDAP_CONSTRAINT_VIOLATION && !pwpolicy->pw_is_legacy){
+				/*
+				 * If we are not using the legacy pw policy behavior,
+				 * convert the error 49 to 19 (constraint violation)
+				 * and log a message
+				 */
+				err = LDAP_CONSTRAINT_VIOLATION;
+				text = "Invalid credentials, you now have exceeded the password retry limit.";
+			}
 		}
 	}
         
@@ -1426,8 +1438,7 @@ send_ldap_search_entry_ext(
 	ber = NULL; /* flush_ber will always free the ber */
 
 log_and_return:
-	if ( logit && operation_is_flag_set(operation,
-		OP_FLAG_ACTION_LOG_ACCESS)){
+	if ( logit && operation_is_flag_set(operation, OP_FLAG_ACTION_LOG_ACCESS)) {
 
 	    log_entry( op, e );
 
@@ -1799,7 +1810,6 @@ static void
 log_entry( Operation *op, Slapi_Entry *e )
 {
 	int	internal_op;
-	char	ebuf[ BUFSIZ ];
 
 	internal_op = operation_is_flag_set( op, OP_FLAG_INTERNAL );
 
@@ -1807,7 +1817,7 @@ log_entry( Operation *op, Slapi_Entry *e )
 	{
 		slapi_log_access( LDAP_DEBUG_STATS2, "conn=%" NSPRIu64 " op=%d ENTRY dn=\"%s\"\n",
 			op->o_connid, op->o_opid,
-			escape_string( slapi_entry_get_dn_const(e), ebuf ));
+			slapi_entry_get_dn_const(e));
 	}
 	else
 	{
@@ -1815,7 +1825,7 @@ log_entry( Operation *op, Slapi_Entry *e )
 		{
 			slapi_log_access( LDAP_DEBUG_ARGS, "conn=%s op=%d ENTRY dn=\"%s\"\n",
 				LOG_INTERNAL_OP_CON_ID, LOG_INTERNAL_OP_OP_ID,
-				escape_string( slapi_entry_get_dn_const(e), ebuf ));
+				slapi_entry_get_dn_const(e));
 		}
 	}
 }
