@@ -427,10 +427,9 @@ free_and_return:;
 
 static void log_search_access (Slapi_PBlock *pb, const char *base, int scope, const char *fstr, const char *msg)
 {
-	char ebuf[BUFSIZ];
 	slapi_log_access(LDAP_DEBUG_STATS,
 					 "conn=%" NSPRIu64 " op=%d SRCH base=\"%s\" scope=%d filter=\"%s\", %s\n",
 					 pb->pb_conn->c_connid, pb->pb_op->o_opid, 
-					 escape_string(base, ebuf), scope, fstr, msg ? msg : "");
+					 base, scope, fstr, msg ? msg : "");
 
 }
