@@ -971,6 +971,7 @@ struct slapdplugin {
 			IFP	plg_un_bepre_modrdn;	  /* modrdn */
 			IFP	plg_un_bepre_add;		  /* add */
 			IFP	plg_un_bepre_delete;	  /* delete */
+			IFP	plg_un_bepre_delete_tombstone;	  /* tombstone creation */
 			IFP	plg_un_bepre_close;		  /* close */
 			IFP	plg_un_bepre_backup;	  /* backup */
 		} plg_un_bepre;
@@ -1003,11 +1004,13 @@ struct slapdplugin {
 			IFP	plg_un_internal_pre_modrdn;	  /* modrdn */
 			IFP	plg_un_internal_pre_add;		  /* add */
 			IFP	plg_un_internal_pre_delete;	  /* delete */
+			IFP	plg_un_internal_pre_bind;	  /* bind */
 		} plg_un_internal_pre;
 #define plg_internal_pre_modify	plg_un.plg_un_internal_pre.plg_un_internal_pre_modify
 #define plg_internal_pre_modrdn	plg_un.plg_un_internal_pre.plg_un_internal_pre_modrdn
 #define plg_internal_pre_add	plg_un.plg_un_internal_pre.plg_un_internal_pre_add
 #define plg_internal_pre_delete	plg_un.plg_un_internal_pre.plg_un_internal_pre_delete
+#define plg_internal_pre_bind	plg_un.plg_un_internal_pre.plg_un_internal_pre_bind
 
 		/* internal post-operation plugin structure */
 		struct plg_un_internal_post_operation {
@@ -1123,11 +1126,13 @@ struct slapdplugin {
 			IFP	plg_un_betxnpre_modrdn;	  /* modrdn */
 			IFP	plg_un_betxnpre_add;		  /* add */
 			IFP	plg_un_betxnpre_delete;	  /* delete */
+			IFP	plg_un_betxnpre_delete_tombstone;	  /* delete tombstone */
 		} plg_un_betxnpre;
 #define plg_betxnpremodify	plg_un.plg_un_betxnpre.plg_un_betxnpre_modify
 #define plg_betxnpremodrdn	plg_un.plg_un_betxnpre.plg_un_betxnpre_modrdn
 #define plg_betxnpreadd	plg_un.plg_un_betxnpre.plg_un_betxnpre_add
 #define plg_betxnpredelete	plg_un.plg_un_betxnpre.plg_un_betxnpre_delete
+#define plg_betxnpredeletetombstone	plg_un.plg_un_betxnpre.plg_un_betxnpre_delete_tombstone
 
         /* backend txn post-operation plugin structure */
 		struct plg_un_betxnpost_operation {
@@ -1239,7 +1244,8 @@ enum
 	BE_STATE_STOPPED = 1,	/* backend is initialized but not started */
 	BE_STATE_STARTED,		/* backend is started */
 	BE_STATE_CLEANED,		/* backend was cleaned up */
-	BE_STATE_DELETED		/* backend is removed */
+	BE_STATE_DELETED,		/* backend is removed */
+	BE_STATE_STOPPING       /* told to stop but not yet stopped */
 };
 
 struct conn;
@@ -1351,6 +1357,26 @@ typedef struct op {
 												client (or we tried to do
 												so and failed) */
 
+
+/* simple paged structure */
+typedef struct _paged_results {
+    Slapi_Backend *pr_current_be;         /* backend being used */
+    void          *pr_search_result_set;  /* search result set for paging */
+    int           pr_search_result_count; /* search result count */
+    int           pr_search_result_set_size_estimate; /* estimated search result set size */
+    int           pr_sort_result_code;    /* sort result put in response */
+    time_t        pr_timelimit;           /* time limit for this request */
+    int           pr_flags;
+    ber_int_t     pr_msgid;               /* msgid of the request; to abandon */
+} PagedResults;
+
+/* array of simple paged structure stashed in connection */
+typedef struct _paged_results_list {
+    int prl_maxlen;          /* size of the PagedResults array */ 
+    int prl_count;           /* count of the list in use */ 
+    PagedResults *prl_list;  /* pointer to pr_maxlen length PageResults array */
+} PagedResultsList;
+
 /*
  * represents a connection from an ldap client
  */
@@ -1406,14 +1432,7 @@ typedef struct conn {
     int				c_local_valid; /* flag true if the uid/gid are valid */
     uid_t			c_local_uid;  /* uid of connecting process */
     gid_t			c_local_gid;  /* gid of connecting process */
-    /* PAGED_RESULTS */
-    Slapi_Backend   *c_current_be;         /* backend being used */
-    void            *c_search_result_set;  /* search result set for paging */
-    int             c_search_result_count; /* search result count */
-    int             c_search_result_set_size_estimate; /* estimated search result set size */
-    int             c_sort_result_code;    /* sort result put in response */
-    time_t          c_timelimit;           /* time limit for this connection */
-    /* PAGED_RESULTS ENDS */
+    PagedResultsList c_pagedresults; /* PAGED_RESULTS */
     /* IO layer push/pop */
     Conn_IO_Layer_cb c_push_io_layer_cb; /* callback to push an IO layer on the conn->c_prfd */
     Conn_IO_Layer_cb c_pop_io_layer_cb; /* callback to pop an IO layer off of the conn->c_prfd */
@@ -1501,6 +1520,7 @@ typedef struct slapi_pblock {
 	int		pb_config_lineno;
 	int		pb_config_argc;
 	char		**pb_config_argv;
+	int		plugin_tracking;
 
 	/* [pre|post]add arguments */
 	struct slapi_entry	*pb_target_entry; /* JCM - Duplicated */
@@ -1636,6 +1656,7 @@ typedef struct slapi_pblock {
 	IFP		pb_mr_index_sv_fn; /* values and keys are Slapi_Value ** */
 	int		pb_syntax_filter_normalized; /* the syntax filter types/values are already normalized */
 	void		*pb_syntax_filter_data; /* extra data to pass to a syntax plugin function */
+	int	pb_paged_results_index;    /* stash SLAPI_PAGED_RESULTS_INDEX */
 } slapi_pblock;
 
 /* index if substrlens */
@@ -1856,6 +1877,7 @@ typedef struct _slapdEntryPoints {
 #define CONFIG_ACCESSLOG_LOGGING_ENABLED_ATTRIBUTE "nsslapd-accesslog-logging-enabled"
 #define CONFIG_ERRORLOG_LOGGING_ENABLED_ATTRIBUTE "nsslapd-errorlog-logging-enabled"
 #define CONFIG_AUDITLOG_LOGGING_ENABLED_ATTRIBUTE "nsslapd-auditlog-logging-enabled"
+#define CONFIG_AUDITLOG_LOGGING_HIDE_UNHASHED_PW "nsslapd-auditlog-logging-hide-unhashed-pw"
 #define CONFIG_ROOTDN_ATTRIBUTE "nsslapd-rootdn"
 #define CONFIG_ROOTPW_ATTRIBUTE "nsslapd-rootpw"
 #define CONFIG_ROOTPWSTORAGESCHEME_ATTRIBUTE "nsslapd-rootpwstoragescheme"
@@ -1937,6 +1959,8 @@ typedef struct _slapdEntryPoints {
 #define CONFIG_PW_RESETFAILURECOUNT_ATTRIBUTE "passwordResetFailureCount"
 #define CONFIG_PW_ISGLOBAL_ATTRIBUTE "passwordIsGlobalPolicy"
 #define CONFIG_PW_GRACELIMIT_ATTRIBUTE "passwordGraceLimit"
+#define CONFIG_PW_IS_LEGACY "passwordLegacyPolicy"
+#define CONFIG_PW_TRACK_LAST_UPDATE_TIME "passwordTrackUpdateTime"
 #define CONFIG_ACCESSLOG_BUFFERING_ATTRIBUTE "nsslapd-accesslog-logbuffering"
 #define CONFIG_CSNLOGGING_ATTRIBUTE "nsslapd-csnlogging"
 #define CONFIG_RETURN_EXACT_CASE_ATTRIBUTE "nsslapd-return-exact-case"
@@ -1953,6 +1977,7 @@ typedef struct _slapdEntryPoints {
 #define CONFIG_ERRORLOG_LIST_ATTRIBUTE "nsslapd-errorlog-list"
 #define CONFIG_AUDITLOG_LIST_ATTRIBUTE "nsslapd-auditlog-list"
 #define CONFIG_REWRITE_RFC1274_ATTRIBUTE "nsslapd-rewrite-rfc1274"
+#define CONFIG_PLUGIN_BINDDN_TRACKING_ATTRIBUTE "nsslapd-plugin-binddn-tracking"
 
 #define CONFIG_CONFIG_ATTRIBUTE "nsslapd-config"
 #define CONFIG_INSTDIR_ATTRIBUTE "nsslapd-instancedir"
@@ -1973,6 +1998,11 @@ typedef struct _slapdEntryPoints {
 #define CONFIG_ENTRYUSN_IMPORT_INITVAL	"nsslapd-entryusn-import-initval"
 #define CONFIG_ALLOWED_TO_DELETE_ATTRIBUTE	"nsslapd-allowed-to-delete-attrs"
 #define CONFIG_DEFAULT_NAMING_CONTEXT "nsslapd-defaultnamingcontext"
+#define CONFIG_DISK_MONITORING "nsslapd-disk-monitoring"
+#define CONFIG_DISK_THRESHOLD "nsslapd-disk-monitoring-threshold"
+#define CONFIG_DISK_GRACE_PERIOD "nsslapd-disk-monitoring-grace-period"
+#define CONFIG_DISK_PRESERVE_LOGGING "nsslapd-disk-monitoring-preserve-logging"
+#define CONFIG_DISK_LOGGING_CRITICAL "nsslapd-disk-monitoring-logging-critical"
 
 #ifdef MEMPOOL_EXPERIMENTAL
 #define CONFIG_MEMPOOL_SWITCH_ATTRIBUTE "nsslapd-mempool"
@@ -2018,6 +2048,8 @@ typedef struct passwordpolicyarray {
   long pw_lockduration;
   long pw_resetfailurecount;
   int pw_gracelimit;
+  int pw_is_legacy;
+  int pw_track_update_time;
   struct pw_scheme *pw_storagescheme;
 } passwdPolicy;
 
@@ -2078,6 +2110,7 @@ typedef struct _slapdFrontendConfig {
   char **backendconfig;
   char **include;
   char **plugin;
+  int plugin_track;
   struct pw_scheme *pw_storagescheme;
 
   int pwpolicy_local;
@@ -2134,6 +2167,7 @@ typedef struct _slapdFrontendConfig {
   int  auditlog_minfreespace;
   int  auditlog_exptime;
   char *auditlog_exptimeunit;
+  int  auditlog_logging_hide_unhashed_pw;
 
   int return_exact_case;	/* Return attribute names with the same case 
 				 * as they appear in at.conf */
@@ -2197,6 +2231,13 @@ typedef struct _slapdFrontendConfig {
   char *entryusn_import_init;   /* Entry USN: determine the initital value of import */
   int pagedsizelimit;
   char *default_naming_context; /* Default naming context (normalized) */
+
+  /* disk monitoring */
+  int disk_monitoring;
+  int disk_threshold;
+  int disk_grace_period;
+  int disk_preserve_logging;
+  int disk_logging_critical;
 } slapdFrontendConfig_t;
 
 /* possible values for slapdFrontendConfig_t.schemareplace */
